package HSPC::ConfigTie;

use SWsoft::protect;

use Carp;

sub new {
	my $pkg = shift;
	$pkg->TIEHASH(@_);
}

sub TIEHASH {
	my $classname = shift;
	my $filename = shift;
	
  my $self = {};
  bless ($self, $classname);

	$self->{filename} = $filename;
	return $self;
}

sub STORE {
	croak "Attempting modify read-only hash.";
}

sub DELETE {
	croak "Attempting modify read-only hash.";
}

sub CLEAR {
	croak "Attempting modify read-only hash.";
}

sub FETCH {
	my ($self, $key) = @_;
	my $h = &check_config($self);
	return $h->{$key};
}

sub EXISTS {
	my ($self, $key) = @_;
	my $h = &check_config($self);
	return exists $h->{$key};
}

sub DEFINED {
	my ($self, $key) = @_;
	my $h = &check_config($self);
	return defined $h->{$key};
}

sub FIRSTKEY {
	my $self = shift;
	my $h = &check_config($self);
	my $a = scalar keys %$h;
	return each %$h;
}

sub NEXTKEY  {
	my $self = shift;
	my $h = &check_config($self);
	return each %$h;
}

sub check_config {
	my $self = shift;
	my ($key, $value, $lmod);

	my $config_path = "/etc/hspc" . $self->{filename};

	croak "File '$config_path' not found" unless $lmod = stat($config_path);

	return $self->{$config_path}{data} if exists $self->{$config_path} && $self->{$config_path}{lmod} == $lmod;

	$self->{$config_path}{lmod} = $lmod;
	$self->{$config_path}{data} = {};

	my $old_slash=$/;
	$/="\n";

	open CONF, $config_path or die "Can't open file '$config_path'";
	while(<CONF>) {
		chomp;
		s/(^\s*|\s)#.*//;
		s/^\s+//;
		s/\s+$//;
		next unless length;
		($key, $value) = split /\s*=\s*/, $_, 2;
		if ($key =~ m/^([^.]+)\.(.+)$/) {
			if (ref( $self->{$config_path}{data}{$1} ) eq 'HASH') {
				$self->{$config_path}{data}{$1}->{$2} = $value;
			}
			else {
				$self->{$config_path}{data}{$1} = {$2 => $value};
			}
		}
		else {
			$self->{$config_path}{data}{$key} = $value;
		}		
	}
	close CONF;
  $/=$old_slash;

	return $self->{$config_path}{data};
}

1;
