#!/usr/bin/perl

package HSPC::ErrorCodes;
use SWsoft::protect; ## automatic


use strict;

use vars qw(
    @ISA @EXPORT_OK @EXPORT %EXPORT_TAGS
);

@ISA = qw(Exporter);
@EXPORT = qw(
   get_error_message get_module_code get_low_error_code
   make_full_error_code
);

#-----------------------------------------------------------------------------

my @module;
$module[0] = 'UNDEFINED';
# Codes 1..32 (0x1..0x20) reserved for feature use
$module[0x21] = 'CCC::Apache';
$module[0x22] = 'CCC::Users';
$module[0x23] = 'CCC::Sendmail';
$module[0x24] = 'CCC::Sendmail::Forward';
$module[0x25] = 'CCC::Majordomo';
$module[0x26] = 'CCC::ProFTPD';
$module[0x27] = 'CCC::Sendmail::Lochost';
$module[0x28] = 'CCC::Firewall';
$module[0x29] = 'CCC::Groups';
$module[0x2A] = 'CCC::InitD';

#-----------------------------------------------------------------------------

my @error;

# module code 0x21 (Apache)
$error[0x2101] = 'maxclients_num'; #'MaxClients: Should be a number';
$error[0x2102] = 'port_num'; #'Port: Should be a number';
$error[0x2103] = 'port_num_16'; #'Port: Should be a number in range 1..65536';
$error[0x2104] = 'svradm_email_err'; #'ServerAdmin directive: Incorrect email format';
$error[0x2105] = 'svrname_hostname_err'; #'ServerName: Incorrect host name';
$error[0x2106] = 'startsrvs_num'; #'StartServers: Should be a number';
$error[0x2107] = 'cant_read_apache_config'; #'Can not read config file';
$error[0x2108] = 'empty_log_file_name'; #'Empty log file name';
$error[0x2109] = 'script_alias_empty'; #'Script alias or path are empty';
$error[0x210a] = 'incorrect_addr_bindaddr'; # Incorrect address in BindAddress directive
$error[0x210b] = 'incorrect_addr_listen'; #'Incorrect address or port in Listen directive';
$error[0x210c] = 'fe_cant_write_config'; #Fatal error: Cannot write config file

# module code 0x22 (Users)
$error[0x2201] = 'invalid_login';   # 'Invalid login name';
$error[0x2202] = 'useradd_error';   #'useradd internal error. Check entered parameters and try again. Or contact system administrator.';
$error[0x2203] = 'usermod_error';   #'usermod internal error. Check entered parameters and try again. Or contact system administrator.';
$error[0x2204] = 'userdel_error';   #'userdel internal error. Check entered parameters and try again. Or contact system administrator.';
$error[0x2205] = 'invalid_comment'; #'Invalid comment field';
$error[0x2206] = 'invalid_homedir'; #'Invalid home directory: should be entered full path and not from root directory.';
$error[0x2207] = 'invalid_shell';   #'Invalid shell: should be entered full path and file must exists.';
$error[0x2208] = 'passwd_error';    #'passwd internal error. Check entered parameters and try again. Or contact system administrator.';
$error[0x2209] = 'cant_delete_systemuser'; #'Can not delete SYSTEM user: web interface not allowed to do it.';
$error[0x220A] = 'change_passwd_error';    #'Error while changing password. Check if password correctly typed.';
$error[0x220B] = 'passwd_identity_error';  #'First and second passord are not identically.';
# useradd errors
$error[0x220C] = 'internal_err_code_1';  #'Can\'t update password file (internal error code: 1)';
$error[0x220D] = 'internal_err_code_2';  #'Bad command syntax (internal error code: 2)';
$error[0x220E] = 'internal_err_code_3';  #'Invalid argument to option (internal error code: 3)';
$error[0x220F] = 'internal_err_code_4';  #'UID already in use (and no -o option) (internal error code: 4)';
$error[0x2210] = 'internal_err_code_6';  #'Specified group doesn\'t exist (internal error code: 6)';
$error[0x2211] = 'internal_err_code_9';  #'Username already in use (internal error code: 9)';
$error[0x2212] = 'internal_err_code_10'; #'Can\'t update group file (internal error code: 10)';
$error[0x2213] = 'internal_err_code_12'; #'Can\'t create home directory (internal error code: 12)';
$error[0x2214] = 'sys_error_contact_support'; ## Can\'t allocate memory

# module code 0x23 (Sendmail)
$error[0x2301] = 'invalid_relay_name'; #'Invalid relay name - please check relay list and correct errors.';
$error[0x2302] = 'invalid_spam_filter'; #'Invalid spam filter - should be IP, hostname or email';
$error[0x2303] = 'spam_filter_empty'; #'Invalid spam filter - address or rule fields are empty';
$error[0x2304] = 'max_msg_size'; #'Max Message Size: Should be a number';
$error[0x2305] = 'cannot_compile_sendmail_config'; #'Can not compile Sendmail config: maybe M4 not installed in system or Sendmail M4 configs not found';

# module code 0x24 (Sendmail::Forward)
$error[0x2401] = 'invalid_email'; #'Envalid email in the forward list';

# module code 0x25 (Majordomo)
$error[0x2501] = 'invalid_mail_list_name'; #'Invalid mail list name';
$error[0x2502] = 'invalid_email_short';
$error[0x2503] = 'invalid_mail_list_introduction_msg'; #'Invalid Mail list introduction message';
$error[0x2504] = 'err_while_saving_mailing_list_check_majordomo_installation_and_retry';
$error[0x2505] = 'ml_owner_must_differ_with_ml_name';
$error[0x2506] = 'ml_subscriber_must_differ_with_ml_name';

# module code 0x26 (ProFTPD)
$error[0x2601] = 'maxinst_shouldbe_num'; #'MaxInstances: Should be a number';
$error[0x2602] = 'svrname_hostname_err'; #'ServerName: Incorrect host name';
$error[0x2603] = 'port_num'; #'Port: Should be a number';
$error[0x2604] = 'port_num_16'; #'Port: Should be a number in range 1..65536';
$error[0x2605] = 'cant_read_proftpd_config'; #'Can not read config file';

# module code 0x27 (Sendmail::Lochost)
$error[0x2701] = 'invalid_mail_host_name'; # 'Invalid mail host name';

# module code 0x28 (Firewall)
$error[0x2801] = 'incorrect_host_name'; #'Incorrect host name';
$error[0x2802] = 'err_while_saving_changes';
$error[0x2803] = 'port_should_be_num'; #'Port: Should be a number';
$error[0x2804] = 'port_should_be_num_range'; #'Port: Should be a number in range 1..65536';

# module code 0x29 (Group management)
$error[0x2901] = 'invalid_group_name'; # 'Invalid group name';
$error[0x2902] = 'int_err_code_2'; #'Invalid command syntax (internal error code: 2)';
$error[0x2903] = 'int_err_code_3'; #'Invalid argument to option (internal error code: 3)';
$error[0x2904] = 'int_err_code_4'; #'GID not unique (when -o not used) (internal error code: 4)';
$error[0x2905] = 'groupd_exists_enter_another_name'; #'int_err_code_9'; #'Group name not unique (internal error code: 9)';
$error[0x2906] = 'int_err_code_10'; #'Can\'t update group file (internal error code: 10)';
$error[0x2907] = 'groupadd_int_err_1'; #'groupadd internal error. Check entered parameters and try again. Or contact system administrator.';
$error[0x2908] = 'groupadd_int_err_2'; #'groupmod internal error. Check entered parameters and try again. Or contact system administrator.';
$error[0x2909] = 'int_err_code_6'; #'Specified group doesn\'t exist (internal error code: 6)';
$error[0x290A] = 'int_err_code_8'; #'Can\'t remove user\'s primary group (internal error code: 8)';
$error[0x290B] = 'groupadd_int_err_3'; #'groupdel internal error. Check entered parameters and try again. Or contact system administrator.';
$error[0x290C] = 'invalid_user_name_in_group_members'; #'Invalid user name in group members';
$error[0x290D] = 'cant_add_members_into_group'; #'Can\'t add members into group - unknown error';

# module code 0x30 (init.d)
$error[0x3001] = 'cant_chg_runlevel'; #'Can\'t change runlevel';

#-----------------------------------------------------------------------------

sub get_error_message
{
    my $code = shift;
    return $error[$code];
}

#-----------------------------------------------------------------------------

sub get_module_code
{
    my $code = shift;

    return $code >> 8;
}

#-----------------------------------------------------------------------------

sub get_low_error_code
{
    my $code = shift;

    return $code & 0xFF;
}

#-----------------------------------------------------------------------------

sub make_full_error_code
{
    my ($module_code, $low_error_code) = @_;

    return $module_code * 0x100 + $low_error_code;
}

#-----------------------------------------------------------------------------

1;
