## Copyright (C) 2000 - 2006  SWsoft
## All rights reserved.
## Abstract class of exceptions used in plug-in framework

package HSPC::Exception::Plugin;

use strict;
use SWsoft::protect;

use vars qw{@ISA};

use HSPC::String;
use HSPC::Trace;

use HSPC::Exc;
@ISA = qw{HSPC::Exc};

sub new {
    my $class = shift;
    my %h = @_;

    my $self = $class->SUPER::new();

	$self->__init(%h);

    return $self;
};

sub user_message {
	my $self = shift;
	return $self->{user_message};
}

sub vendor_message {
	my $self = shift;
	return $self->{vendor_message};
}

sub log_message {
	my $self = shift;
	return $self->{log_message};
}

## Usage $@ as string will return this method result.
sub stringify {
    my $self = shift;

    return $self->{message};
};

sub __init {
    my $self = shift;
    my %h = (
    	user_message   => undef,
    	vendor_message => undef,
    	log_message    => undef,
		default        => undef,
    	@_
    );

	if ($h{log_message}) {
		if ( ref($h{log_message}) eq 'HASH' ) {
			$self->{log_message} = HSPC::String->new(
				string_id   => $h{log_message}->{string_id},
				string_args => $h{log_message}->{string_args}
			) if ($h{log_message}->{string_id});
		} else {
			$self->{log_message} = $h{log_message};
		}
	}

	if ($h{vendor_message}) {
		if ( ref($h{vendor_message}) eq 'HASH' ) {
			$self->{vendor_message} = HSPC::String->new(
				string_id   => $h{vendor_message}->{string_id},
				string_args => $h{vendor_message}->{string_args}
			) if ($h{vendor_message}->{string_id});
		} else {
			$self->{vendor_message} = $h{vendor_message};
		}
	} else {
		$self->{vendor_message} = HSPC::String->new(
			string_id   => 'default_exc_vendor_message'
		);
	}

	if ($h{user_message}) {
		if ( ref($h{user_message}) eq 'HASH' ) {
			$self->{user_message} = HSPC::String->new(
				string_id   => $h{user_message}->{string_id},
				string_args => $h{user_message}->{string_args}
			) if ($h{user_message}->{string_id});
		} else {
			$self->{user_message} = $h{user_message};
		}
	} else {
		$self->{user_message} = HSPC::String->new(
			string_id   => 'default_exc_user_message'
		);
	}

	$self->{message} = $self->log_message || $self->vendor_message ||
		$self->user_message || $h{default} || '[exp: Exeption exception has been thrown]';

	HSPC::Trace->add_comment_to_latest_trace($self->{message});

	return undef;
}

1;
