## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
## Constants for credit cards payment methods

package HSPC::MT::CCP::Constants;

use strict;

use SWsoft::protect;    ## Mark this module for encryption upon build
use HSPC::Localization qw(string);

## constants for permission in plugin configuration
use constant SW_CCP_PERMISSION_DISABLED => 0;
use constant SW_CCP_PERMISSION_NONE => 1;
use constant SW_CCP_PERMISSION_ENABLED => 2;
use constant SW_CCP_PERMISSION_ENABLED_PER_RESELLER => 3;
use constant SW_CCP_PERMISSION_DISABLED_PER_RESELLER => 4;

use constant SW_CCP_PERMISSION_TXT => { 0 => 'disabled_for_resellers',
                                        1 => 'not_set',
				                        2 => 'enabled_for_resellers',
                                        3 => 'enabled_per_reseller',
                                        4 => 'disabled_per_reseller' };

## constants for status in transaction log
use constant SW_CCP_TRANS_UNKNOWN  => 0;
use constant SW_CCP_TRANS_APPROVED => 1;
use constant SW_CCP_TRANS_DECLINED => 2;
use constant SW_CCP_TRANS_FRAUD    => 3;
use constant SW_CCP_TRANS_ERROR    => 4;
use constant SW_CCP_TRANS_REFUNDED => 5;
use constant SW_CCP_TRANS_AUTHCALL => 6;
use constant SW_CCP_TRANS_DELETED  => 7;
use constant SW_CCP_TRANS_PENDING  => 8;
use constant SW_CCP_TRANS_3DSECURE => 9;

use constant SW_CCP_TRANS_STATUS_TXT => {
	0 => 'unknown',
	1 => 'approved_tr',
	2 => 'declined_tr',
	3 => 'fraud_tr',
	4 => 'error_tr',
	5 => 'refunded',
	6 => 'authcall',
	7 => 'deleted_tr',
	8 => 'pending_tr',
	9 => 'pending_tr',
};

## constants for errors occurred during transaction 
use constant SW_CCP_PROCESS_OK	=> 0;
use constant SW_CCP_PROCESS_CARD_CONF_ERROR	=> 1;
use constant SW_CCP_PROCESS_PLUGIN_CONF_ERROR => 2;
use constant SW_CCP_PROCESS_TRANS_GENERAL_ERROR => 3;
use constant SW_CCP_PROCESS_TRANS_ERROR => 4;
use constant SW_CCP_PROCESS_INTERNAL_ERROR => 5;
use constant SW_CCP_PROCESS_AUTHCALL => 6;

use constant SW_CCP_PROCESS_TXT => {
	0 => 'ok',
	1 => 'card_conf_error',
	2 => 'plugin_conf_error',
	3 => 'general_plugin_error',
	4 => 'plugin_error',
	5 => 'int_billing_error_uc',
	6 => 'wait_4auth_call_uc'
};

use constant SW_CCP_CREDIT_TYPE_NONE => 0; 
use constant SW_CCP_CREDIT_TYPE_TRANS=> 1;
use constant SW_CCP_CREDIT_TYPE_ALL  => 2;

use constant SW_CCP_TRANS_TYPE_REFUND => 2;           ## reserve amount
use constant SW_CCP_TRANS_TYPE_PREAUTH => 3;          ## reserve amount
use constant SW_CCP_TRANS_TYPE_PREAUTH_REVERSAL => 4; ## void pre-auth
use constant SW_CCP_TRANS_TYPE_CAPTURE => 5;          ## capture reserved amount
use constant SW_CCP_TRANS_TYPE_CAPTURE_REVERSAL => 6; ## void captured amount
use constant SW_CCP_TRANS_TYPE_SALE   => 7;           ## preauth/capture at once
use constant SW_CCP_TRANS_TYPE_CREDIT => 8;           ## put amount back to the CC
use constant SW_CCP_TRANS_TYPE_CHECK_STATUS => 9;     ## check status of transction

use constant SW_CCP_TRANS_TYPE_TXT => {
	1 => 'payment', &SW_CCP_TRANS_TYPE_REFUND => 'refund', ## for compatibility only
	&SW_CCP_TRANS_TYPE_PREAUTH		=> 'preauth',
	&SW_CCP_TRANS_TYPE_PREAUTH_REVERSAL	=> 'auth_reversal',
	&SW_CCP_TRANS_TYPE_CAPTURE		=> 'tt_capture',
	&SW_CCP_TRANS_TYPE_CAPTURE_REVERSAL	=> 'reversal',
	&SW_CCP_TRANS_TYPE_SALE			=> 'sale',
	&SW_CCP_TRANS_TYPE_CREDIT		=> 'tt_credit',
};

use Exporter;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
@ISA = qw(Exporter);

@EXPORT = ();
@EXPORT_OK = (qw/SW_CCP_TRANS_UNKNOWN SW_CCP_TRANS_APPROVED SW_CCP_TRANS_DECLINED
			  SW_CCP_TRANS_FRAUD SW_CCP_TRANS_ERROR SW_CCP_TRANS_STATUS_TXT
 SW_CCP_PERMISSION_NONE SW_CCP_PERMISSION_ENABLED SW_CCP_PERMISSION_DISABLED SW_CCP_PERMISSION_TXT
 SW_CCP_PERMISSION_ENABLED_PER_RESELLER SW_CCP_PERMISSION_DISABLED_PER_RESELLER
 SW_CCP_PROCESS_OK SW_CCP_PROCESS_CARD_CONF_ERROR SW_CCP_PROCESS_PLUGIN_CONF_ERROR
 SW_CCP_PROCESS_AUTHCALL SW_CCP_TRANS_3DSECURE
 SW_CCP_PROCESS_TRANS_GENERAL_ERROR SW_CCP_PROCESS_TRANS_ERROR SW_CCP_PROCESS_INTERNAL_ERROR
 SW_CCP_TRANS_REFUNDED SW_CCP_TRANS_AUTHCALL	SW_CCP_TRANS_DELETED SW_CCP_TRANS_PENDING
 SW_CCP_PROCESS_TXT SW_CCP_CREDIT_TYPE_NONE SW_CCP_CREDIT_TYPE_TRANS SW_CCP_CREDIT_TYPE_ALL
 SW_CCP_TRANS_TYPE_PREAUTH SW_CCP_TRANS_TYPE_PREAUTH_REVERSAL
 SW_CCP_TRANS_TYPE_CAPTURE SW_CCP_TRANS_TYPE_CAPTURE_REVERSAL
 SW_CCP_TRANS_TYPE_SALE SW_CCP_TRANS_TYPE_CREDIT SW_CCP_TRANS_TYPE_REFUND SW_CCP_TRANS_TYPE_CHECK_STATUS
 SW_CCP_TRANS_TYPE_TXT
/);

%EXPORT_TAGS = (
		'status' => [qw/SW_CCP_TRANS_UNKNOWN SW_CCP_TRANS_APPROVED SW_CCP_TRANS_DECLINED
 SW_CCP_TRANS_FRAUD SW_CCP_TRANS_ERROR SW_CCP_TRANS_STATUS_TXT SW_CCP_TRANS_3DSECURE
 SW_CCP_TRANS_REFUNDED SW_CCP_TRANS_AUTHCALL SW_CCP_TRANS_DELETED SW_CCP_TRANS_PENDING/],

        'permissions' => [qw/SW_CCP_PERMISSION_NONE SW_CCP_PERMISSION_ENABLED
 SW_CCP_PERMISSION_DISABLED SW_CCP_PERMISSION_TXT SW_CCP_PERMISSION_ENABLED_PER_RESELLER
 SW_CCP_PERMISSION_DISABLED_PER_RESELLER/],

		'process'=> [qw/SW_CCP_TRANS_UNKNOWN SW_CCP_TRANS_APPROVED SW_CCP_TRANS_DECLINED
 			  SW_CCP_TRANS_FRAUD SW_CCP_TRANS_ERROR SW_CCP_TRANS_STATUS_TXT
 SW_CCP_PROCESS_OK SW_CCP_PROCESS_CARD_CONF_ERROR SW_CCP_PROCESS_PLUGIN_CONF_ERROR
 SW_CCP_PROCESS_TRANS_GENERAL_ERROR SW_CCP_PROCESS_TRANS_ERROR SW_CCP_PROCESS_INTERNAL_ERROR
 SW_CCP_PROCESS_AUTHCALL SW_CCP_PROCESS_TXT SW_CCP_TRANS_3DSECURE/],

		'credit_types'=>[qw/SW_CCP_CREDIT_TYPE_NONE SW_CCP_CREDIT_TYPE_TRANS 
 SW_CCP_CREDIT_TYPE_ALL/],

		'trans_types'=>[qw/
 SW_CCP_TRANS_TYPE_PREAUTH SW_CCP_TRANS_TYPE_PREAUTH_REVERSAL SW_CCP_TRANS_TYPE_REFUND
 SW_CCP_TRANS_TYPE_CAPTURE SW_CCP_TRANS_TYPE_CAPTURE_REVERSAL
 SW_CCP_TRANS_TYPE_SALE SW_CCP_TRANS_TYPE_CREDIT SW_CCP_TRANS_TYPE_CHECK_STATUS SW_CCP_TRANS_TYPE_TXT/],
);

1;
