package HSPC::MT::DM::Constants;
use strict;

use SWsoft::protect;    # Mark this module for encryption upon build

use Exporter;
use base qw|Exporter|;

#----------------------------------------------------------------------------

## EV constants for DM
use constant EV_DOM_REG   =>14;
use constant EV_DOM_REGERR=>15;
use constant EV_DOM_TRANS =>16;
use constant EV_DOM_DNSHOSTING=>32;
use constant EV_DOM_TRANS_ERR => 505;
use constant EV_DOM_RENEW_DONE => 506;
use constant EV_DOM_RENEW_ERR => 507;
use constant EV_DOM_TERM_QUEUED => 508;
use constant EV_DOM_TERM_ERR => 509;
use constant EV_DOM_TERM_DONE => 510;
use constant EV_DOM_DNS_HOST_OVER => 511;


## default parameters for tld "DNS zone start-up settings"
use constant DM_TLD_DNS_ZONE_STARTUP_TIME_LIMIT => 5;
use constant DM_TLD_DNS_ZONE_NUMBER_OF_ATTEMPTS => 10;

## default parameters for 'Domain Data Consistency' options in DM>Setup>General
use constant DM_DOM_EXP_DATE_DEF_RECHECK_ENABLED => 1;
use constant DM_DOM_EXP_DATE_DEF_RECHECK_PERIOD => 7;
use constant DM_DOM_EXP_DATE_DEF_AUTOFIX_ENABLED => 1;
use constant DM_DOM_EXP_DATE_DEF_AUTOFIX_THRESHOLD => 7;

## Number of domains to check for real expiry date in task per run
use constant DM_EXP_DATE_CHECKER_DOM_BATCH_SIZE => 100;

use constant DM_DNS_TMPL_USAGE_TYPE_COMMON => 1;
use constant DM_DNS_TMPL_USAGE_TYPE_REG_VPS => 2;
use constant DM_DNS_TMPL_USAGE_TYPE_NBH_VPS => 3;
use constant DM_DNS_TMPL_USAGE_TYPE_PD => 4;
use constant DM_DNS_TMPL_USAGE_TYPE_PC => 5;


use constant DM_DNS_TMPL_USAGE_TYPES => {
	&DM_DNS_TMPL_USAGE_TYPE_COMMON => { name => "dm_tmpl_type_common_uc", },
	&DM_DNS_TMPL_USAGE_TYPE_REG_VPS => { name => "dm_tmpl_type_regular_vps_uc", },
	&DM_DNS_TMPL_USAGE_TYPE_NBH_VPS => { name => "dm_tmpl_type_nbh_vps_uc", },
	&DM_DNS_TMPL_USAGE_TYPE_PD => { name => "dm_tmpl_type_plesk_shared_uc", },
	&DM_DNS_TMPL_USAGE_TYPE_PC => { name => "dm_tmpl_type_plesk_client_uc", },
};


## DNS template placeholders
use constant DM_DNS_TMPL_PH_DOMAIN => '%domain%';
use constant DM_DNS_TMPL_PH_IP => '%ip%';
use constant DM_DNS_TMPL_PH_SVE_IP => '%sve_ip%';

use constant DM_DNS_TMPL_PH_ALLOWED => [
	DM_DNS_TMPL_PH_DOMAIN(),
	DM_DNS_TMPL_PH_IP(),
	DM_DNS_TMPL_PH_SVE_IP()
];

## EPP TLDs for transfer
use constant EPP_TLDS => qw|biz cn com.cn net.cn org.cn in info org us name com.au net.au|;

## TLDs which may be 3rd level, e.g. 'john.smith.name'
use constant THIRD_LEVEL_TLDS => qw|name|;

use constant SW_DM_LOOKUP_OK  	=> 1;
use constant SW_DM_LOOKUP_WARN	=> 2;
use constant SW_DM_LOOKUP_ERR	=> 3;

use constant SW_DM_LOOKUP_MODE_PLUG  => 1;
use constant SW_DM_LOOKUP_MODE_WHOIS => 2;
## for registration and transfer can be used different lookup modes
## SW_DM_LOOKUP_MODE_REG_WHOIS_TR_API means that registration uses WHOIS and 
## transfer uses API
use constant SW_DM_LOOKUP_MODE_REG_WHOIS_TR_API => 3;
## SW_DM_LOOKUP_MODE_REG_API_TR_WHOIS means that registration uses API and 
## transfer uses WHOIS
use constant SW_DM_LOOKUP_MODE_REG_API_TR_WHOIS => 4;

use constant SW_DM_LOOKUP_MODE_PLUG_BULK  => 5;
use constant SW_DM_LOOKUP_MODE_REG_WHOIS_TR_BULK_API => 6;
use constant SW_DM_LOOKUP_MODE_REG_BULK_API_TR_WHOIS => 7;

use constant SW_DM_LOOKUP_TXT => {
	SW_DM_LOOKUP_MODE_PLUG() => 'lookup_mode_plugin',
	SW_DM_LOOKUP_MODE_WHOIS() => 'lookup_mode_whois',
};

use constant SW_DM_ASSIGNED_NOT => 0;
use constant SW_DM_ASSIGNED_VPS => 1;
use constant SW_DM_ASSIGNED_IP  => 2;

use constant DM_PRICE_CONFIGURED_NOT => 0;
use constant DM_PRICE_CONFIGURED_REG => 1;
use constant DM_PRICE_CONFIGURED_TRANS => 2;
use constant DM_PRICE_CONFIGURED_BOTH => 3;

use constant DM_PRICE_CONFIGURED_TXT => {
	&DM_PRICE_CONFIGURED_NOT => 'unavail', 
	&DM_PRICE_CONFIGURED_REG => 'available_for_reg',
	&DM_PRICE_CONFIGURED_TRANS => 'available_for_trans',
	&DM_PRICE_CONFIGURED_BOTH => 'available_for_both',
};

use constant DM_TLD_AVAILABILITY_NOT => DM_PRICE_CONFIGURED_NOT;
use constant DM_TLD_AVAILABILITY_REG => DM_PRICE_CONFIGURED_REG;
use constant DM_TLD_AVAILABILITY_TRANS => DM_PRICE_CONFIGURED_TRANS;
use constant DM_TLD_AVAILABILITY_BOTH => DM_PRICE_CONFIGURED_BOTH;
use constant DM_TLD_AVAILABILITY_ANY => 4;

use constant DM_TLD_AVAILABILITY_TXT => {
	&DM_TLD_AVAILABILITY_NOT => 'unavail', 
	&DM_TLD_AVAILABILITY_REG => 'available_for_reg',
	&DM_TLD_AVAILABILITY_TRANS => 'available_for_trans',
	&DM_TLD_AVAILABILITY_BOTH => 'available_for_both',
	&DM_TLD_AVAILABILITY_ANY => 'available',
};

use constant OFFLINE_PROCESSING_NOT_STARTED => 1;
use constant OFFLINE_PROCESSING_STARTED 	=> 2;
use constant OFFLINE_PROCESSING_FINISHED 	=> 3;

use constant NS_STATUS_PENDING		=> 1;
use constant NS_STATUS_INSTALLED	=> 2;
use constant NS_STATUS_INSTALLING	=> 3;
use constant NS_STATUS_ERROR		=> 4;
use constant NS_STATUS_UNREACHABLE	=> 5;

use constant NS_TYPE_NONMANAGEABLE	=> 'NMNS';
use constant NS_TYPE_CUSTOMER		=> 'NMCNS';

use constant DM_CUSTOM_NS_COUNT		=> 5;

use constant ALIASES_FILE => '/etc/aliases';
use constant EML_GW_PROG => 'hspc-dm-mailgate.pl';

use constant WHOIS_REQUESTS_PROG => 'hspc-dm-parallel-whois-requests.pl';
use constant NUMBER_OF_DOMAINS_IN_BUNDLE_WHOIS => 30;
use constant DM_WHOIS_SHAREFILE => '/etc/hspc/whois_sharefile';

use constant ALLOW_REGISTER_NONE     => 0;
use constant ALLOW_REGISTER_HSP      => 1;
use constant ALLOW_REGISTER_RESELLER => 2;

use constant DM_DOMAIN_ACTION_HOST	=> 1;
use constant DM_DOMAIN_ACTION_REG	=> 2;
use constant DM_DOMAIN_ACTION_TRANS	=> 3;

use constant DM_DOMAIN_ACTION_TXT => {
	&DM_DOMAIN_ACTION_HOST => 'dom_dns_uc',
	&DM_DOMAIN_ACTION_REG => 'dom_registration_uc',
	&DM_DOMAIN_ACTION_TRANS => 'dom_transfer_uc'
};

use constant BASE_CONTACT_FIELDS => [
	'fname',
	'lname',
	'address',
	'city',
	'state',
	'zip',
	'country',
	'email',
	'phone',
	'fax',
	'is_corporate',
	'org_name',
];

use constant REG_STATUS_UNREGISTERED	=> 0;
use constant REG_STATUS_REGISTERED		=> 1;
use constant REG_STATUS_REGISTERING		=> 2;
use constant REG_STATUS_TRANSFERRING	=> 3;
use constant REG_STATUS_ERROR			=> 4;
use constant REG_STATUS_RENEWING		=> 5;
use constant REG_STATUS_ZONESTARTING	=> 6;
use constant REG_STATUS_TERMINATING		=> 7;
use constant REG_STATUS_TERMINATED		=> 8;

use constant REG_STATUSES => [
	REG_STATUS_UNREGISTERED(),
	REG_STATUS_REGISTERED(),
	REG_STATUS_REGISTERING(),
	REG_STATUS_TRANSFERRING(),
	REG_STATUS_ERROR(),
	REG_STATUS_RENEWING(),
	REG_STATUS_ZONESTARTING(),
	REG_STATUS_TERMINATING(),
	REG_STATUS_TERMINATED(),
];

use constant NS_STATUS_TEXT => {
	NS_STATUS_PENDING()	=> 'pending',
	NS_STATUS_INSTALLING()	=> 'installing',
	NS_STATUS_INSTALLED()	=> 'installed',
	NS_STATUS_ERROR()	=> 'error',
	NS_STATUS_UNREACHABLE() => 'unreachable'
};

use constant NS_POLICY_PROV => 1;
use constant NS_POLICY_RES => 2;

use constant NS_POLICY_TEXT => {
	NS_POLICY_PROV() => 'hp_only',
	NS_POLICY_RES() => 'hp_and_res',
};

use constant REG_STATUS_TXT => {
	REG_STATUS_UNREGISTERED()	=> 'unregistered',
	REG_STATUS_REGISTERED()		=> 'registered',
	REG_STATUS_RENEWING()		=> 'renewing', 
	REG_STATUS_REGISTERING()	=> 'registering',
	REG_STATUS_TRANSFERRING()	=> 'transferring',
	REG_STATUS_ZONESTARTING()	=> 'zone_starting',
	REG_STATUS_ERROR()			=> 'error',
	REG_STATUS_TERMINATING()	=> 'dm_terminating',
	REG_STATUS_TERMINATED()		=> 'dm_terminated',
};

use constant DNS_TYPE_HSP	=> 1;
use constant DNS_TYPE_REGISTRAR	=> 2;

use constant DM_PLUGIN_HSP => 'HSP';

use constant DM_WHOIS_EXISTENCE_PATTERN => 1;
use constant DM_WHOIS_ABSENCE_PATTERN => 2;

use constant DM_REGISTRAR_ORDER_PROCESSED => 1;
use constant DM_REGISTRAR_ORDER_INPROGRESS => 2;
use constant DM_REGISTRAR_ORDER_CANCELLED => 3;
use constant DM_REGISTRAR_ORDER_ABSENT => 4;
use constant DM_REGISTRAR_ORDER_TERMINATED => 5;

use constant DM_HOLDER_TYPE_VPS => 1;
use constant DM_HOLDER_TYPE_PLESK => 2;
use constant DM_HOLDER_TYPE_PLESK_CLIENT => 3;

use constant DM_AUTHCODE_MINLEN => 6;
use constant DM_AUTHCODE_MAXLEN => 16;

## Default plug-in used in initial HSPC setup
use constant DM_PLUGIN_DEFAULT => 'Dummy';

## Vendor id for default plugin config
use constant SW_DM_PLUGIN_DEFAULT_VENDOR_ID => 0;

use constant ACTION_LOG_INSTALL_NS 	=> 1;
use constant ACTION_LOG_REGISTER_DOMAIN => 2;
use constant ACTION_LOG_REG_NS 		=> 3;
use constant ACTION_LOG_CHECK_NS 	=> 4;

use constant CHECKDOMAIN_OK				=> 0;
use constant CHECKDOMAIN_ERR_INVALID	=> 1;
use constant CHECKDOMAIN_ERR_EXISTS		=> 2;
use constant CHECKDOMAIN_ERR_RESERVED	=> 3;

##
## DM contact type ids
##
use constant DM_ACC_OWNER		=> 'owner';
use constant DM_ACC_ADMIN		=> 'admin';
use constant DM_ACC_BILLING		=> 'billing';
use constant DM_ACC_TECHNICAL	=> 'technical';
use constant DM_ACC_ZONE		=> 'zone';

##
## DM contact type localization ids (by default)
##
use constant DM_ACC_TYPE_LC =>
	{
		DM_ACC_OWNER()		=> 'dm_contact_title_owner',
		DM_ACC_ADMIN()		=> 'dm_contact_title_admin',
		DM_ACC_BILLING()	=> 'dm_contact_title_bill',
		DM_ACC_TECHNICAL()	=> 'dm_contact_title_tech',
		DM_ACC_ZONE()		=> 'dm_contact_title_zone'
	};

use constant DM_ACC_MAP =>
	{
		DM_ACC_OWNER()		=> 'owner',
		DM_ACC_ADMIN()		=> 'admin',
		DM_ACC_BILLING()	=> 'billing',
		DM_ACC_TECHNICAL()	=> 'technical',
		DM_ACC_ZONE()		=> 'technical'
	};

use constant DM_CONT_SAVE_TYPE_UPDATE	=> 0;
use constant DM_CONT_SAVE_TYPE_COPY		=> 1;

use constant DM_TERM_TYPE_AFTER_SUBSCR_TERM => 0;
use constant DM_TERM_TYPE_BEFORE_DOM_EXP => 1;
use constant DM_TERM_TYPE_AFTER_DOM_EXP => 2;

use constant DM_TERM_PERIOD_TYPE_HOURS => 0;
use constant DM_TERM_PERIOD_TYPE_DAYS => 1;

use constant DM_EXTDATA_HOLDER_DOMAIN		=> 1;
use constant DM_EXTDATA_HOLDER_ACC_OWNER	=> 2;
use constant DM_EXTDATA_HOLDER_ACC_ADMIN	=> 3;
use constant DM_EXTDATA_HOLDER_ACC_BILLING	=> 4;
use constant DM_EXTDATA_HOLDER_ACC_TECH		=> 5;

use constant DM_TERM_PERIOD_TYPES => {
	&DM_TERM_PERIOD_TYPE_HOURS	=> 'hours',
	&DM_TERM_PERIOD_TYPE_DAYS	=> 'days',
};

## Domain Management events
use constant DM_EV_DOM_REG_DONE			=> EV_DOM_REG();
use constant DM_EV_DOM_REG_ERR			=> EV_DOM_REGERR();
use constant DM_EV_DOM_TRANS_DONE		=> EV_DOM_TRANS();
use constant DM_EV_DOM_TRANS_ERR		=> EV_DOM_TRANS_ERR();
use constant DM_EV_DOM_DNSHOSTING_DONE	=> EV_DOM_DNSHOSTING();
use constant DM_EV_DOM_RENEW_DONE		=> EV_DOM_RENEW_DONE();
use constant DM_EV_DOM_RENEW_ERR		=> EV_DOM_RENEW_ERR();
use constant DM_EV_DOM_TERM_QUEUED		=> EV_DOM_TERM_QUEUED();
use constant DM_EV_DOM_TERM_ERR			=> EV_DOM_TERM_ERR();
use constant DM_EV_DOM_TERM_DONE		=> EV_DOM_TERM_DONE();

use constant DM_EVENTS_MAP => {
	DM_EV_DOM_REG_DONE() => {
		reg_statuses => [
			&REG_STATUS_REGISTERING,
			&REG_STATUS_ERROR,
			&REG_STATUS_UNREGISTERED,
		],
		actions => [
			&DM_DOMAIN_ACTION_REG
		],
		new_reg_status => &REG_STATUS_REGISTERED,
		priority => 10,
	},
	DM_EV_DOM_REG_ERR() => {
		reg_statuses => [
			&REG_STATUS_REGISTERING,
			&REG_STATUS_ERROR,
			&REG_STATUS_UNREGISTERED,
		],
		actions => [
			&DM_DOMAIN_ACTION_REG
		],
		new_reg_status => &REG_STATUS_ERROR,
		priority => 10,
	},
	DM_EV_DOM_TRANS_DONE() => {
		reg_statuses => [
			&REG_STATUS_TRANSFERRING,
			&REG_STATUS_ERROR,
			&REG_STATUS_UNREGISTERED,
		],
		actions => [
			&DM_DOMAIN_ACTION_TRANS
		],
		new_reg_status => &REG_STATUS_REGISTERED,
		priority => 10,
	},
	DM_EV_DOM_TRANS_ERR() => {
		reg_statuses => [
			&REG_STATUS_TRANSFERRING,
			&REG_STATUS_ERROR,
			&REG_STATUS_UNREGISTERED,
		],
		actions => [
			&DM_DOMAIN_ACTION_TRANS
		],
		new_reg_status => &REG_STATUS_ERROR,
		priority => 10,
	},
	DM_EV_DOM_DNSHOSTING_DONE() => {
		reg_statuses => [
			&REG_STATUS_ERROR,
			&REG_STATUS_UNREGISTERED,
		],
		actions => [
			&DM_DOMAIN_ACTION_HOST
		],
		new_reg_status => &REG_STATUS_REGISTERED,
		priority => 10,
	},
	DM_EV_DOM_RENEW_DONE() => {
		reg_statuses => [
			&REG_STATUS_RENEWING,
		],
		actions => [
			&DM_DOMAIN_ACTION_REG,
			&DM_DOMAIN_ACTION_TRANS,
		],
		new_reg_status => &REG_STATUS_REGISTERED,
		priority => 100,
	},
	DM_EV_DOM_RENEW_ERR() => {
		reg_statuses => [
			&REG_STATUS_RENEWING,
		],
		actions => [
			&DM_DOMAIN_ACTION_REG,
			&DM_DOMAIN_ACTION_TRANS,
		],
		new_reg_status => &REG_STATUS_ERROR,
		priority => 100,
	},
	DM_EV_DOM_TERM_QUEUED() => {
		reg_statuses => [
			&REG_STATUS_REGISTERED,
		],
		actions => [
			&DM_DOMAIN_ACTION_REG,
			&DM_DOMAIN_ACTION_TRANS,
		],
		new_reg_status => &REG_STATUS_TERMINATING,
		priority => 10,
	},
	DM_EV_DOM_TERM_ERR() => {
		reg_statuses => [
			&REG_STATUS_TERMINATING,
		],
		actions => [
			&DM_DOMAIN_ACTION_REG,
			&DM_DOMAIN_ACTION_TRANS,
		],
		new_reg_status => &REG_STATUS_REGISTERED,
		priority => 10,
	},
	DM_EV_DOM_TERM_DONE() => {
		reg_statuses => [
			&REG_STATUS_TERMINATING,
		],
		actions => [
			&DM_DOMAIN_ACTION_REG,
			&DM_DOMAIN_ACTION_TRANS,
		],
		new_reg_status => &REG_STATUS_TERMINATED,
		priority => 10,
	},
};

##-- New DM Plugin Engine ------------------------------------------------------

use constant DM_FALSE => 0;
use constant DM_TRUE => 1;

use constant DM_BOOLEAN_STATUSES => [
	&DM_FALSE,
	&DM_TRUE
];

use constant DM_CHECK_NOT_AVAILABLE => 0;
use constant DM_CHECK_AVAILABLE => 1;
use constant DM_CHECK_ERROR => 2;
use constant DM_CHECK_SKIPPED => 3;

use constant DM_CHECK_STATUSES => [
	&DM_CHECK_NOT_AVAILABLE,
	&DM_CHECK_AVAILABLE,
	&DM_CHECK_ERROR,
	&DM_CHECK_SKIPPED
];

use constant DM_ACTION_REGISTER => 'register';
use constant DM_ACTION_TRANSFER => 'transfer';
use constant DM_ACTION_RENEW => 'renew';
use constant DM_ACTION_TERMINATE => 'terminate';
use constant DM_ACTION_UPDATE_CONTACTS => 'update_contacts';
use constant DM_ACTION_UPDATE_DOMAIN => 'update_domain';

use constant DM_ACTIONS => [
	&DM_ACTION_REGISTER,
	&DM_ACTION_TRANSFER,
	&DM_ACTION_RENEW,
	&DM_ACTION_TERMINATE,
	&DM_ACTION_UPDATE_CONTACTS
];

use constant DM_CONTACT_TYPES => {
	'owner' => 'dm_default_title_owner',
	'admin' => 'dm_default_title_admin',
	'technical' => 'dm_default_title_technical',
	'billing' => 'dm_default_title_billing',
	'zone' => 'dm_default_title_zone'
};

use constant DM_DOMAIN_STATUS_ERROR => 'error';
use constant DM_DOMAIN_STATUS_REGISTERING => 'registering';
use constant DM_DOMAIN_STATUS_REGISTERED => 'registered';
use constant DM_DOMAIN_STATUS_TRANSFERRING => 'transferring';
use constant DM_DOMAIN_STATUS_TRANSFERRED => 'transferred';
use constant DM_DOMAIN_STATUS_RENEWING => 'renewing';
use constant DM_DOMAIN_STATUS_RENEWED => 'renewed';
use constant DM_DOMAIN_STATUS_TERMINATING => 'terminating';
use constant DM_DOMAIN_STATUS_TERMINATED => 'terminated';
use constant DM_DOMAIN_STATUS_ZONESTARTING => 'zonestarting';
use constant DM_DOMAIN_STATUS_UNREGISTERED => 'unregistered';

use constant DM_DOMAIN_STATUSES => {
	&DM_DOMAIN_STATUS_ERROR => &REG_STATUS_ERROR,
	&DM_DOMAIN_STATUS_REGISTERING => &REG_STATUS_REGISTERING,
	&DM_DOMAIN_STATUS_REGISTERED => &REG_STATUS_REGISTERED,
	&DM_DOMAIN_STATUS_TRANSFERRING => &REG_STATUS_TRANSFERRING,
	&DM_DOMAIN_STATUS_TRANSFERRED => &REG_STATUS_REGISTERED,
	&DM_DOMAIN_STATUS_RENEWING => &REG_STATUS_RENEWING,
	&DM_DOMAIN_STATUS_RENEWED => &REG_STATUS_REGISTERED,
	&DM_DOMAIN_STATUS_TERMINATING => &REG_STATUS_TERMINATING,
	&DM_DOMAIN_STATUS_TERMINATED => &REG_STATUS_TERMINATED,
	&DM_DOMAIN_STATUS_ZONESTARTING => &REG_STATUS_ZONESTARTING,
	&DM_DOMAIN_STATUS_UNREGISTERED => &REG_STATUS_UNREGISTERED,
};

use constant DM_SLD_PLACEHOLDER => 'anydomain.';


## Mapping from numeric domain actions (used in DM) 
## into string actions (used in plug-ins)
use constant DM_ACTION_MAP_NUM_STRING => {
	&DM_DOMAIN_ACTION_REG => &DM_ACTION_REGISTER,
	&DM_DOMAIN_ACTION_TRANS => &DM_ACTION_TRANSFER
};


use constant DM_NS_STATUS_UNSYNCHRONIZED => 0;
use constant DM_NS_STATUS_SYNCHRONIZED   => 1;
use constant DM_NS_STATUS_SYNCHRONIZING  => 2;

use constant DM_DOMAIN_NS_STATUS_SYNCHRONIZED => 'synchronized';
use constant DM_DOMAIN_NS_STATUS_SYNCHRONIZING => 'synchronizing';
use constant DM_DOMAIN_NS_STATUS_UNSYNCHRONIZED => 'unsynchronized';

use constant DM_DOMAIN_NS_STATUSES_MAP => {
	&DM_DOMAIN_NS_STATUS_SYNCHRONIZED => &DM_NS_STATUS_SYNCHRONIZED,
	&DM_DOMAIN_NS_STATUS_SYNCHRONIZING => &DM_NS_STATUS_SYNCHRONIZING,
	&DM_DOMAIN_NS_STATUS_UNSYNCHRONIZED => &DM_NS_STATUS_UNSYNCHRONIZED,
};

##------------------------------------------------------------------------------

our @EXPORT = qw(

	DM_TLD_DNS_ZONE_STARTUP_TIME_LIMIT
	DM_TLD_DNS_ZONE_NUMBER_OF_ATTEMPTS

	DM_ACC_OWNER
	DM_ACC_ADMIN
	DM_ACC_BILLING
	DM_ACC_TECHNICAL
	DM_ACC_ZONE

	DM_ACC_TYPE_LC
	DM_ACC_MAP
	
	DM_ACTION_MAP_NUM_STRING

	DM_CONT_SAVE_TYPE_UPDATE
	DM_CONT_SAVE_TYPE_COPY

	EPP_TLDS
	THIRD_LEVEL_TLDS

	NS_STATUS_INSTALLED
	NS_STATUS_INSTALLING
	NS_STATUS_ERROR
	NS_STATUS_PENDING
	NS_STATUS_UNREACHABLE
	NS_STATUS_TEXT

	DM_CUSTOM_NS_COUNT

	NS_TYPE_NONMANAGEABLE
	NS_TYPE_CUSTOMER

	NS_POLICY_PROV 
	NS_POLICY_RES 
	NS_POLICY_TEXT

	ALIASES_FILE
	EML_GW_PROG
	WHOIS_REQUESTS_PROG
	NUMBER_OF_DOMAINS_IN_BUNDLE_WHOIS
	DM_WHOIS_SHAREFILE

	DNS_TYPE_HSP
	DNS_TYPE_REGISTRAR
	DM_PLUGIN_HSP
	DM_RR_TYPES
	DM_SUPPORTED_IDN_RR_TYPES
	DM_WHOIS_EXISTENCE_PATTERN 
	DM_WHOIS_ABSENCE_PATTERN 

	DM_REGISTRAR_ORDER_PROCESSED 
	DM_REGISTRAR_ORDER_INPROGRESS 
	DM_REGISTRAR_ORDER_CANCELLED 
	DM_REGISTRAR_ORDER_ABSENT
	DM_REGISTRAR_ORDER_TERMINATED

	DM_HOLDER_TYPE_VPS
	DM_HOLDER_TYPE_PLESK
	DM_HOLDER_TYPE_PLESK_CLIENT
	
	DM_DOMAIN_ACTION_HOST
	DM_DOMAIN_ACTION_REG
	DM_DOMAIN_ACTION_TRANS
	DM_DOMAIN_ACTION_TXT

	REG_ACTION_REGISTRATION
	REG_ACTION_TRANSFER
	REG_ACTION_DNSHOSTING
	REG_ACTION_DOMAINPOINTER
	REG_ACTION_TXT

	REG_ACTION_BM_MAP
	
	REG_STATUS_REGISTERED
	REG_STATUS_UNREGISTERED
	REG_STATUS_REGISTERING
	REG_STATUS_TRANSFERRING
	REG_STATUS_RENEWING
	REG_STATUS_ERROR
	REG_STATUS_ZONESTARTING
	REG_STATUS_TXT
	REG_STATUS_TERMINATING
	REG_STATUS_TERMINATED

	REG_STATUSES

	ALLOW_REGISTER_NONE
	ALLOW_REGISTER_HSP
	ALLOW_REGISTER_RESELLER

	DM_PRICE_CONFIGURED_NOT 
	DM_PRICE_CONFIGURED_REG 
	DM_PRICE_CONFIGURED_TRANS
	DM_PRICE_CONFIGURED_BOTH 
	DM_PRICE_CONFIGURED_TXT

	DM_TLD_AVAILABILITY_NOT
	DM_TLD_AVAILABILITY_REG
	DM_TLD_AVAILABILITY_TRANS
	DM_TLD_AVAILABILITY_BOTH
	DM_TLD_AVAILABILITY_ANY
	DM_TLD_AVAILABILITY_TXT

	DM_PLUGIN_DEFAULT

	DM_AUTHCODE_MINLEN
	DM_AUTHCODE_MAXLEN
	
	OFFLINE_PROCESSING_NOT_STARTED
	OFFLINE_PROCESSING_STARTED
	OFFLINE_PROCESSING_FINISHED

	SW_DM_ASSIGNED_NOT
	SW_DM_ASSIGNED_VPS
	SW_DM_ASSIGNED_IP

	SW_DM_LOOKUP_MODE_PLUG
	SW_DM_LOOKUP_MODE_WHOIS
	SW_DM_LOOKUP_MODE_REG_WHOIS_TR_API
	SW_DM_LOOKUP_MODE_REG_API_TR_WHOIS
	SW_DM_LOOKUP_MODE_PLUG_BULK
	SW_DM_LOOKUP_MODE_REG_WHOIS_TR_BULK_API
	SW_DM_LOOKUP_MODE_REG_BULK_API_TR_WHOIS
	SW_DM_LOOKUP_OK
	SW_DM_LOOKUP_WARN
	SW_DM_LOOKUP_ERR
	SW_DM_LOOKUP_TXT

	ACTION_LOG_INSTALL_NS 
	ACTION_LOG_REG_NS 
	ACTION_LOG_CHECK_NS 
	ACTION_LOG_REGISTER_DOMAIN

	DM_DNS_TMPL_PH_DOMAIN
	DM_DNS_TMPL_PH_IP
	DM_DNS_TMPL_PH_SVE_IP
	DM_DNS_TMPL_PH_ALLOWED

	CHECKDOMAIN_OK
	CHECKDOMAIN_ERR_INVALID
	CHECKDOMAIN_ERR_EXISTS
	CHECKDOMAIN_ERR_RESERVED

	DM_TERM_TYPE_AFTER_SUBSCR_TERM
	DM_TERM_TYPE_BEFORE_DOM_EXP
	DM_TERM_TYPE_AFTER_DOM_EXP

	DM_TERM_PERIOD_TYPE_HOURS
	DM_TERM_PERIOD_TYPE_DAYS
	DM_TERM_PERIOD_TYPES

	DM_EXTDATA_HOLDER_DOMAIN
	DM_EXTDATA_HOLDER_ACC_OWNER
	DM_EXTDATA_HOLDER_ACC_ADMIN
	DM_EXTDATA_HOLDER_ACC_BILLING
	DM_EXTDATA_HOLDER_ACC_TECH

	DM_EV_DOM_REG_DONE
	DM_EV_DOM_REG_ERR
	DM_EV_DOM_TRANS_DONE
	DM_EV_DOM_TRANS_ERR
	DM_EV_DOM_DNSHOSTING_DONE
	DM_EV_DOM_RENEW_DONE
	DM_EV_DOM_RENEW_ERR
	DM_EV_DOM_TERM_QUEUED
	DM_EV_DOM_TERM_ERR
	DM_EV_DOM_TERM_DONE

	DM_EVENTS_MAP
	
	SW_DM_PLUGIN_DEFAULT_VENDOR_ID

	DM_DOM_EXP_DATE_DEF_RECHECK_ENABLED
	DM_DOM_EXP_DATE_DEF_RECHECK_PERIOD
	DM_DOM_EXP_DATE_DEF_AUTOFIX_ENABLED
	DM_DOM_EXP_DATE_DEF_AUTOFIX_THRESHOLD

	DM_EXP_DATE_CHECKER_DOM_BATCH_SIZE



	DM_FALSE
	DM_TRUE

	DM_BOOLEAN_STATUSES

	DM_CHECK_NOT_AVAILABLE
	DM_CHECK_AVAILABLE
	DM_CHECK_ERROR
	DM_CHECK_SKIPPED

	DM_CHECK_STATUSES

	DM_ACTION_REGISTER
	DM_ACTION_TRANSFER
	DM_ACTION_RENEW
	DM_ACTION_TERMINATE
	DM_ACTION_UPDATE_CONTACTS
	DM_ACTION_UPDATE_DOMAIN

	DM_ACTIONS

	DM_CONTACT_TYPES

	DM_DOMAIN_STATUS_ERROR
	DM_DOMAIN_STATUS_REGISTERING
	DM_DOMAIN_STATUS_REGISTERED
	DM_DOMAIN_STATUS_TRANSFERRING
	DM_DOMAIN_STATUS_TRANSFERRED
	DM_DOMAIN_STATUS_RENEWING
	DM_DOMAIN_STATUS_RENEWED
	DM_DOMAIN_STATUS_TERMINATING
	DM_DOMAIN_STATUS_TERMINATED
	DM_DOMAIN_STATUS_ZONESTARTING
	DM_DOMAIN_STATUS_UNREGISTERED

	DM_DOMAIN_STATUSES

	DM_SLD_PLACEHOLDER

	DM_DNS_TMPL_USAGE_TYPE_COMMON
	DM_DNS_TMPL_USAGE_TYPE_REG_VPS
	DM_DNS_TMPL_USAGE_TYPE_NBH_VPS
	DM_DNS_TMPL_USAGE_TYPE_PD
	DM_DNS_TMPL_USAGE_TYPE_PC
	DM_DNS_TMPL_USAGE_TYPES
	
	DM_NS_STATUS_UNSYNCHRONIZED
	DM_NS_STATUS_SYNCHRONIZED
	DM_NS_STATUS_SYNCHRONIZING

	DM_DOMAIN_NS_STATUS_SYNCHRONIZED
	DM_DOMAIN_NS_STATUS_SYNCHRONIZING
	DM_DOMAIN_NS_STATUS_UNSYNCHRONIZED

	DM_DOMAIN_NS_STATUSES_MAP
	
	BASE_CONTACT_FIELDS
);

1;
