package HSPC::MT::DM::RegEngine;

use strict;
use SWsoft::protect;    # Mark this module for encryption upon build

use Data::Dumper;
use HSPC::GeoData;

use HSPC::Logger qw(sw_atrace sw_log_debug sw_log_warn sw_log sw_die);
use HSPC::Localization::Date qw(date_now str_to_time time_to_secs);
use HSPC::Localization qw(string current_lang);
use HSPC::Util::DM qw(d2ascii);

sub new {
	my $class = shift;
	my $self = {};
	bless ($self, $class);	
	return $self;
}

#----------------------------------------------------------------------------

sub _regex {
	my $self = shift;
	my $trace = sw_atrace($ENV{__dmregex});
	return $ENV{__dmregex} if length $ENV{__dmregex};

##	my @suffix = map {$_->[0]} @{
##		[
	my @suffix = (
			"br.com",
			"cn.com",
			"co.ee",
			"co.in",
			"co.uk",
			"com.cn",
			"com.es",
			"com.mx",
			"com.pt",
			"com.tw",
			"cpa.pro",
			"de.com",
			"eu.com",
			"firm.in",
			"gen.in",
			"hu.com",
			"idv.tw",
			"ind.in",
			"kids.us",
			"law.pro",
			"me.uk",
			"med.pro",
			"net.cn",
			"net.in",
			"no.com",
			"org.cn",
			"org.es",
			"org.in",
			"org.tw",
			"org.uk",
			"qc.com",
			"ru.com",
			"sa.com",
			"se.com",
			"se.net",
			"uk.com",
			"uk.net",
			"us.com",
			"uy.com",
			"za.com",
##		]
##	};
	);
	@suffix =
		map { d2ascii($_)->{value} ne $_ ? ( $_, d2ascii($_)->{value} ) : $_ }
		@suffix;

	my $regex = join '|', @suffix;
	$regex	=~s![.]!\\$&!gs;
	## store the regex for further uses;
	$ENV{__dmregex}	= $regex;
	$trace->add($regex);
	return $regex;
}

sub extract_tld_from_domain
{
	my $self = shift;
	my %arg = (
		domain	=> undef, 
		@_
	);
	my $trace = sw_atrace($arg{domain});
	if (! defined $arg{domain}) {
		sw_die("Required argument 'domain' not defined");
	}

	my $tld;
	(undef, $tld) = $self->_split_domain_parts(%arg);
	$tld = d2ascii($tld)->{value};
	$trace->add($tld);
	return $tld;
}

sub extract_shortname_from_domain {
	my $self = shift;
	my %arg = (
		domain	=> undef, 
		@_
	);
	my $trace = sw_atrace();
	if (! defined $arg{domain}) {
		sw_die("Required argument 'domain' not defined");
	}

	my $shortname;
	($shortname, undef) = $self->_split_domain_parts(@_);
	return $shortname;
}

sub extract_sld_from_domain {
	my $self = shift;
	my %arg = (
		domain => undef,
		@_
	);
	my $trace = sw_atrace($arg{domain});
	if (! defined $arg{domain}) {
		sw_die("Required argument 'domain' not defined");
	}

	my $short_domain = $self->extract_shortname_from_domain(
		domain => $arg{domain}
	);
	my $tld = $self->extract_tld_from_domain(domain => $arg{domain});
	my $domain_sld = $short_domain;
	$domain_sld =~ s#.+\.##g;
	$domain_sld .= '.'.$tld;
	$trace->add($domain_sld);
	return $domain_sld;
}

sub _split_domain_parts
{
	my $self = shift;
	my %arg = (
		domain	=> undef, @_,
		regex	=> $self->_regex(),
	);
	my $trace = sw_atrace($arg{domain});
	my $domain = $arg{domain};
	my $tld;

	if (! defined $domain) {
		sw_die("Required argument 'domain' not defined");
	}

	if ($domain =~ m!\.($arg{regex})$!) {
		$tld = $1;
    } elsif ($domain =~ m|\.([[:alnum:]]+)$|) {
		$tld = $1;
    } else {
        sw_log_warn("Unknown TLD for domain '$domain'");
    }
    $domain =~ s|\.$tld$||i;

	$trace->add("`$domain' - `$tld'");
	return ($domain, $tld);
}

sub is_cc_tld {
	my $self = shift;
	my %h = (
		tld => undef,
		@_
	);
	my $trace = sw_atrace();
	## we needed tld
	unless($h{tld}){
		sw_die(
			 "'tld' arg. needed tld: '$h{tld}'"
		);
	}
	my $tld = lc($h{tld});
	## get all countries and test our tld on 'cc' 
	my $countries = HSPC::GeoData::get_countries();
	my $is_cc_tld = 0;
	foreach my $country (@{$countries}){
			my $country_code = lc($country->[0]);
			## In HSPC 'uk' tld it GB country
			if ($tld =~ /^uk$|\.uk$/
				or $tld =~ /^$country_code$|\.$country_code$/)
				{
					$is_cc_tld = 1;
					last;
				}
	}

	return $is_cc_tld;
}

sub prepare_help_url {
	return "";
}
sub get_help_url {
	return "";
}
sub get_help_page {
	return "";
}

1;

__END__
