package HSPC::MT::DM::ToolsInt;

use strict;
use SWsoft::protect; # Mark this module for encryption upon build

use HSPC::Logger qw(sw_atrace sw_log_debug sw_log_warn sw_log_ok sw_log sw_die);

## System Libs
use Net::DNS::Resolver;
use Encode qw(_utf8_on);
use Net::IDN::Encode;
use Data::Dumper;

sub is_zone_started {
	my $class = shift;
	my %arg = (
		domain => undef,
		nses => [],
		@_
	);
	my $trace = sw_atrace();

	my $domain = $arg{domain};
	my $nses = $arg{nses};

	if (!$domain || (!ref $nses || !scalar @$nses)) {
		sw_log_warn("domain or nses undefined: ".Dumper(\%arg));
		return 0;
	}

	## Check DNS
	my $res = 1;
	my @ns_ips = map {$_->{ip}} @$nses;
	my @ns_hosts = map {$_->{host}} @$nses;

	my $dns_resolver = Net::DNS::Resolver->new();

	my $cur_res;
	foreach my $ns_ip (@ns_ips) {
		$dns_resolver->nameservers($ns_ip);
		my $packet = $dns_resolver->query("$domain", "NS");

		my @rrs;
		if ($packet) {
			@rrs = ($packet->answer());
			sw_log("NS records from DNS query ($ns_ip): ".Dumper(\@rrs));

			foreach my $ns_host (@ns_hosts) {
				$cur_res = scalar(
					grep {ref $_ && (lc $_->nsdname() eq lc $ns_host)} @rrs
				);

				$res &&= $cur_res;
			}
		} else {
			$cur_res = 0;
			$res = 0;
		}

		sw_log_debug("The zone($domain) is not sarted yet on NS($ns_ip)")
			if (!$cur_res);
	}

	sw_log("The zone already started?: ".($res ? 'yes' : 'no'));
	return $res;
}

sub domain2ascii {
	my $class = shift;
	my $domain = shift;

	my $trace = sw_atrace($domain);
	my $prefix = '';
	if ($domain =~ s|^(\*\.)||){
		$prefix = $1;		
	}
	Encode::_utf8_on($domain); ## Set utf8 flag

	my $ascii_domain;
	eval {
		$ascii_domain = domain_to_ascii($domain);
	};

	## Convert error, please see NOTE in description
	if ($@) {
		sw_log_warn("Impossible to convert UTF domain name to ASCII");
		sw_die(
			"Domain name is not valid IDN domain, "
			."call DM checks before convert"
		);
	}
	Encode::_utf8_off($ascii_domain);
	$trace->add($prefix.$ascii_domain);
	return $prefix.$ascii_domain;
}

1;

__END__
