package HSPC::MT::DM::WhoisClient;

use SWsoft::protect;

use strict;


use HSPC::Logger qw(sw_die);
use IO::Socket;
use IO::Select;
use Data::Dumper;

sub new{
	my $class = shift;
	my $self = {};

	bless($self, $class);
	return $self;
}

sub whois_query {
	my $class = shift;
    my %arg = (
		server => undef,
		port => 43,
		timeout => 60,
		query => undef,
		@_
	);

	my $srv = $arg{server};
	my $prt = $arg{port} || 43;
	my $query = $arg{query};

	sw_die("[WhoisClient] server undefined!") unless ($srv);
	sw_die("[WhoisClient] port undefined!") unless ($prt);
	sw_die("[WhoisClient] query undefined!") unless ($query);

	my $sock;
	my @lines = ();
	my $prev_alarm = alarm 0;
	eval {
		local $SIG{'ALRM'} = sub { die "Connection timeout to server '$srv'\n" };
		alarm $arg{timeout} if $arg{timeout};
		$sock = new IO::Socket::INET("$srv:$prt") || die "$srv: $!\n";
	};
	alarm $prev_alarm;
	my $err = $@;
	if (!$err) {
		my $whoisquery = $arg{query};
		print $sock "$whoisquery\r\n";
		$prev_alarm = alarm 0;
		eval {
			local $SIG{'ALRM'} = sub { die "Read timeout from server '$srv'. "
				."Query: '$whoisquery'\n" };
			alarm $arg{timeout} if $arg{timeout}; 
			@lines = <$sock>;
		};
		alarm $prev_alarm;
		$err = $@;
		if ($err) {
			return {res => \@lines, err => $err};
		}
		close($sock);
	}

	return {res => \@lines, err => $err};
}

1;
