package HSPC::MT::LangValue;


use strict;
use SWsoft::protect;

use HSPC::Util qw(sw_purify_data);
use HSPC::Localization qw(current_lang);


use overload (
	'""'       =>   'stringify',
	'ne'       =>   'stringify',
	'eq'       =>   'stringify',
	'cmp'      =>   'stringify',
);


sub new {
	my $class = shift;

	my $self = {};
	bless $self,$class;
	$self->set_hash({}) unless ($self->get_hash());

	return $self;
}##/new


sub stringify {
	my $self = shift;

	my $value = $self->{hash_values}->{current_lang()};
	$value = $self->{hash_values}->{def} unless (length($value));

	return $value;
}##/stringify


sub get_hash {
	my $self = shift;
	return $self->{hash_values};
}##/get_lang_hash


sub set_hash {
	my $self = shift;
	my $value = shift;
	if ( ref($value) eq 'HASH' ) {
		$self->{hash_values} = $value;
	} else {
		$self->{hash_values}->{current_lang()} = $value;
		$self->{hash_values}->{def} = $value;
	}
}##/set_hash


sub from_db_format {
	my $self = shift;
	my $db_str = shift;
	$db_str = sw_purify_data($db_str);

	my $hash = {};
	while ( $db_str =~ /<lang_([a-zA-Z]{2,3})>\n(.*?)\n<\/lang_\1>/gms ) {
		$hash->{$1} = $2;
	}

	if (scalar(%$hash) == 0) {
		$hash->{def} = $db_str;
	}

	$self->set_hash($hash);
}##/from_db_format


sub to_db_format {
	my $self = shift;
	
	my $db_str = '';
	foreach my $l ( keys(%{$self->{hash_values}}) ) {
		$db_str .= "<lang_$l>\n".sw_purify_data($self->{hash_values}->{$l}).
		"\n</lang_$l>\n";
	}
	return $db_str;
}##/to_db_format


sub get_value {
	my $self = shift;
	my $lang = shift || current_lang();

	my $value = $self->{hash_values}->{$lang};
	$value = $self->{hash_values}->{def} unless (length($value));
	return $value;
}##/get_value

## just to unificate with HSPC::String
sub text_for_lang {
    my $self = shift;
    my %h = (
        lang    => undef, ## language for localize.
		@_
	);
	return $self->get_value( $h{lang} );
}


1;
