## Copyright (C) 2000, 2001, 2002, 2003, 2004  SWsoft
# All rights reserved.
#
# Error/Warning reporting system for pamethod plugins

package HSPC::MT::PM::Tools;

use strict;
use SWsoft::protect;

use HSPC::Logger qw(sw_atrace sw_log sw_die);
use HSPC::Localization qw(string);

use HSPC::MT::PluginConfig;

my $__PM_TEMPLATES;

sub init {
	my $class = shift;

	## Init an empty array
	$__PM_TEMPLATES = [];

	my $templates = HSPC::MT::PluginConfig->find_templates(
		plugin_type      => 'PM',
		force_permission => 1,
	);

	foreach my $template_id (@$templates) {
		my $template = HSPC::MT::PluginConfig->find_mt_template(
			plugin_type => 'PM',
			template_id => $template_id,
		);
		next unless $template;

		eval "use $template;";
		if ($@) {
			sw_log_warn(__PACKAGE__ . "Can not load template $template: $@");
			next;
		}

		if ($template->can('provided_payment_method_types')) {
			my $plugin_type = $class->get_plugin_type(template_id => $template_id);
			foreach my $type (@{$template->provided_payment_method_types}) {
				push(@$__PM_TEMPLATES,{
					template_id   => $template_id,
					pm_title      => $template->title_id,
					plugin_type   => $plugin_type,
					type          => $type->{type},
					type_title_id => $type->{title_id},
					image_id      => $type->{image_id},
				});
			}
		} else {
			sw_log_warn(__PACKAGE__ . "Template $template does not have the mandatory provided_payment_method_types method");
			next;
		}
	}

	return undef;
}

sub get_pm_template_id_by_type {
	my $class = shift;
	my %h = (
		type    => undef,
		@_
	);
	my ($template_id) = map { $_->{template_id} } grep
		{ $_->{type} eq $h{type} } @$__PM_TEMPLATES;

	return $template_id;
}

sub get_plugin_type {
	my $class = shift;
	my %h = (
		template_id => undef,
		@_
	);

	my $trace = sw_atrace("template_id=$h{template_id}");

	$h{template_id} =~ /(\w+)_[\w\d]+/;

	my $plugin_type = lc($1);
	$trace->add($plugin_type);
	return $plugin_type;
}

sub get_pm_template_description {
	return []; ## Not needed for shared PBA
}
sub get_paytypes_supported_by_plugin {
	return []; ## Not needed for shared PBA
}
sub get_plugins_supported_by_paytype {
	return []; ## Not needed for shared PBA
}
sub get_presentation_engine {
	return undef; ## Not needed for shared PBA
}
sub get_mt_engine {
	return undef; ## Not needed for shared PBA
}
sub pm_allowed_types {
	return []; ## Not needed for shared PBA
}
sub delete_paymethods_due_to_key_droped {
	return undef; ## Not needed for shared PBA
};
sub get_active_paymethods {
	return []; ## Not needed for shared PBA
}
sub find_paymethod_plugins {
	return []; ## Not needed for shared PBA
}
sub get_vendor_skey {
	return undef; ## Not needed for shared PBA
}
sub get_vendor_config {
	return undef; ## Not needed for shared PBA
}

##-------------- OLD METHODS ----------------------------------

sub set_paytypes_supported_by_plugin {
	return undef; ## Not needed for shared PBA
};
sub delete_paytypes_supported_by_plugin {
	return undef; ## Not needed for shared PBA
};
sub install_paymethod_plugin {
	return undef; ## Not needed for shared PBA
}

1;
