package HSPC::MT::Plugin::DM::OpenProvider::Constants;
use strict;

use base qw(Exporter);

use constant DM_PLUGIN_OPENPROVIDER => 'OpenProvider';
use constant SW_DM_OP_OPS => {
	'OP_NEW_DOMAIN'			=> 'newDomain',
	'OP_RENEW_DOMAIN' 		=> 'renewDomain',
	'OP_TRANS_DOMAIN' 		=> 'transferDomain',
	'OP_DOMAIN_STATUS'      => 'statusDomain',
	'OP_NEW_CONTACT'		=> 'newContact',
	'OP_NEW_COMPANY'		=> 'newCompany',
	'OP_NEW_INDIVIDUAL'		=> 'newIndividual',
	'OP_MODIFY_CONTACT'		=> 'modifyContact',
	'OP_MODIFY_COMPANY'		=> 'modifyCompany',
	'OP_MODIFY_INDIVIDUAL'	=> 'modifyIndividual',
	'OP_DELETE_COMPANY'		=> 'deleteCompany', ## not implemented yet
	'OP_DELETE_HANDLE'		=> 'deleteHandle', ## not implemented yet
	'OP_NEW_NS_GROUP'		=> 'newNameservergroup',
	'OP_MODIFY_NS_GROUP'	=> 'modifyNameservergroup', ## not implemented yet
	'OP_DELETE_NS_GROUP'	=> 'deleteNameservergroup', ## not implemented yet
	'OP_MODIFY_DOMAIN'		=> 'modifyDomain',
	'OP_VIEW_CONTACT'		=> 'viewContact',
	'OP_VIEW_COMPANY'		=> 'viewCompany',
	'OP_TERMINATE_DOMAIN'	=> 'deleteDomain',
	'OP_DOMAIN_INFO'		=> 'infoDomain',
	'OP_CHECK_DOMAIN'		=> 'checkDomainRequest',
};

use constant OP_DEFAULT_API_URL => "https://api.openprovider.eu";

## Name servers
use constant OPENPROVIDER_NS_NUMBER => 4;	## Number of NSes in set

## tlds which requires auth_key for transfer
use constant OP_TRANS_TLDS => qw{com net info biz org pl biz.pl com.pl info.pl net.pl org.pl};

## Special ext data TLDs
use constant OP_SPEC_ED_TLDS => qw{it};

## Domain lookup 
use constant SW_DM_OP_LOOKUP_TAKEN 	=> 0;	## domain is taken
use constant SW_DM_OP_LOOKUP_AVAIL 	=> 1;	## domain is available
use constant SW_DM_OP_LOOKUP_2SHORT	=> -307;	## dName length too short
use constant SW_DM_OP_LOOKUP_2LONG 	=> -308;	## dName length too long
use constant SW_DM_OP_LOOKUP_DN_INV	=> -309;	## domain name invalid

## Domain registration
use constant SW_DM_OP_ADDNS_EXISTS => 202;	## nameservers group exists

## An unknown domain-error has occurred
use constant SW_DM_OP_UNKOWN_DOMAIN_ERROR => 399;	

## OpenProvider returned codes
use constant OPENPROVIDER_RETURN_OK	=> 0;	## returned successive value
use constant OPENPROVIDER_ERROR_CODE=> 1;	## internal error value

## Company name already exists
use constant OPENPROVIDER_COMPANY_NAME_EXISTS => 182; 
use constant OPENPROVIDER_DOMAIN_NOT_KNOWN => 320;
use constant OPENPROVIDER_DOMAIN_IN_ACCOUNT => 326;

use constant SW_DM_OP_SYNTAX_ERR => 8;	## syntax error internal code
use constant OPENPROVIDER_TEL_TYPES => ['phone', 'fax'];

use vars qw(@ISA @EXPORT);

@ISA    = qw(Exporter);
@EXPORT = qw(
	DM_PLUGIN_OPENPROVIDER
	OP_TRANS_TLDS
	OP_SPEC_ED_TLDS
	SW_DM_OP_OPS
	SW_DM_OP_SYNTAX_ERR
	OP_CONTACT_LOC
	OP_DEFAULT_API_URL

	SW_DM_OP_LOOKUP_TAKEN 
	SW_DM_OP_LOOKUP_AVAIL 
	SW_DM_OP_LOOKUP_2SHORT
	SW_DM_OP_LOOKUP_2LONG
	SW_DM_OP_LOOKUP_DN_INV
	SW_DM_OP_UNKOWN_DOMAIN_ERROR

	SW_DM_OP_ADDNS_EXISTS
	OPENPROVIDER_RETURN_OK
	OPENPROVIDER_ERROR_CODE
	OPENPROVIDER_COMPANY_NAME_EXISTS
	OPENPROVIDER_DOMAIN_NOT_KNOWN
	OPENPROVIDER_DOMAIN_IN_ACCOUNT
	OPENPROVIDER_NS_NUMBER
	OPENPROVIDER_TEL_TYPES
);


1;

