package HSPC::MT::Plugin::PM::BT_Clieop03;

use strict;
use HSPC::PluginToolkit::General qw(throw_exception datetime_gmt_now gmt_to_local);

sub provided_payment_method_types {
	my $class = shift;

	return [
		{type => 'BT_Clieop03', title_id => 'bapm_clieop03_type_clieop03', image_id => ''},
	];
}

sub title_id {
	return 'bapm_clieop03_ba_uc';
}

sub get_public_data {
	my $class = shift;
	my %h = (
		data => undef,
		@_
	);

	my %public_data = (
		holder_name  => $h{data}->{holder_name},
		agree        => $h{data}->{agree},
		no_agreement => $h{data}->{no_agreement}
	);
	
	## store the date when user agrees with the license by request
	if ( $h{data}->{date} ) {
		$public_data{date} = $h{data}->{date};
	} else {
		$public_data{date} = gmt_to_local(datetime_gmt_now())
			if ($h{data}->{agree});
	}
	
	return \%public_data;
}

sub get_secure_data {
	my $class = shift;
	my %h = (
		data             => undef,
		keep_secure_code => undef,
		@_
	);

	my %secure_data = (
		account_number => $h{data}->{account_number},
	);

	return \%secure_data;
}

sub get_public_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return substr($h{secure_data}->{account_number},-4);
}

sub get_secure_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return $h{secure_data}->{account_number};
}

sub validate {
	my $class = shift;
	my %h = (
		expire_date => undef,
		secure_data => undef,
		public_data => undef,
		type        => undef,
		@_
	);

	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_clieop03_holder_name_not_defined'}
	) unless $h{public_data}->{holder_name};
	return unless $h{secure_data};
	
	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_clieop03_account_number_required'}
	) unless $h{secure_data}->{account_number};
	
	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_clieop03_not_agreed'}
	) if (!$h{public_data}->{agree} && !$h{public_data}->{no_agreement} );
	
	unless ($class->account_number_check($h{secure_data}->{account_number})){
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_clieop03_account_number_is_wrong',
				string_args => {
					acc => $h{secure_data}->{account_number}
				}
			}
		);
	}

	return undef;
}

sub get_paymethod_type {
	return 'BT_Clieop03';
}

####
## Eleven check bank account numbers according to CLieop03 documentation
##

sub account_number_check {
	my $class = shift;
	my $account_number = shift;

	return 0 if ($account_number == 0 || $account_number !~ /^\d+$/);

	$account_number = sprintf("%010d", $account_number)
		if (length($account_number) == 9);

	## check postbank postbank accounts aren't checked for eleven check
	my $result = 0;
	if ( length($account_number) == 6 ){
		$result = ($account_number%111111) ? 1 : 0;
	} elsif ( length($account_number) == 7 ){
		$result = ($account_number%1111111) ? 1 : 0;
	} elsif ( length($account_number) == 10 ) {
		my @account_number = split //, $account_number;

		my $n=10;
		my $sum=0;

		map { $sum += $_ * $n-- } @account_number;

		$result = !($sum%11);
	} else {
		$result = 0;
	}

	return $result;
}

1;
