package HSPC::MT::Plugin::PP::BT_Clieop03;

use strict;

use HSPC::PluginToolkit::General qw(datetime_gmt_now string throw_exception gmt_to_local);
use HSPC::MT::Plugin::PP::BT_Clieop03::File;
use HSPC::MT::Plugin::PP::BT_Clieop03::Batch;
use HSPC::MT::Plugin::PP::BT_Clieop03::Item;

sub get_title {
	my $class = shift;
	
	return string('bapm_clieop03_plugin_title_uc');
}

sub process_batch_content {
	my $class = shift;
	my %h = (
		config           => undef,
		transaction_list => undef,
		vendor_info      => undef,
		file_id          => undef,
		@_
	);

	my $config  = $h{config};
	my $file_id = $h{file_id};
	my $transaction_list = $h{transaction_list} ||
		throw_exception(
			type        => 'error',
			log_message => 'Transaction list is not specified'
		);

	my $cdate = gmt_to_local( datetime_gmt_now() );
	
	my $seq_number = $class->get_file_sequence_number(
		current_date => $cdate,
		config       => $config
	);
	
	my $file = HSPC::MT::Plugin::PP::BT_Clieop03::File->create(
		file_date    => $cdate,
		sender_id    => $config->{sender_id},	
		sequence_num => $seq_number
	);

	my $batch = HSPC::MT::Plugin::PP::BT_Clieop03::Batch->create(
		account_num_ordering => $config->{account_number},
		batch_sequence_num   => $file_id,
		fixed_desc           => $config->{fixed_desc},
		date                 => $cdate,
		order_party_name     => $config->{order_party_name},
		test_code            => $config->{test_code}
	);
	
	foreach my $t (@$transaction_list) {
		## prepare transaction description at plugin configuration page
		my $doc_num = $t->{document_info}->{number},;
		my $value   = $t->{account_info}->{id};
		
		my $item_description = $config->{item_desc};
		
		$item_description =~ s/\Q\%ACC_ID\%\E/$value/g;
		$item_description =~ s/\Q\%INV_NUM\%\E/$doc_num/g;
		my $item = HSPC::MT::Plugin::PP::BT_Clieop03::Item->create(
			amount                     => $t->{transaction_amount},
			account_number             => $t->{payment_method}->{secure_data}->{account_number},
			account_number_beneficiary => $config->{account_number},
			name_payer                 => $t->{payment_method}->{public_data}->{holder_name},
			payment_reference          => $doc_num,
			description                => $item_description,
		);

		$batch->add_item(item => $item);
	}

	$file->add_batch (batch => $batch);

	my $txt = $file->prepare_content();

	return {
		BATCH_CONTENT   => $txt,
		BATCH_FILE_NAME => sprintf("clieop03_file_%06d.txt", $file_id),
		CONFIG          => $config
	};
}

sub get_supported_payment_method_types{
	my $class = shift;
	
	return [
		'BT_Clieop03',
	];
}

sub get_currencies_supported{
	return [
		'EUR', # Euro
	];
}

sub get_file_sequence_number {
	my $class = shift;

	my %h = (
		current_date => undef,
		config       => undef,
		@_
	);

	my $current_date = $h{current_date};
	$current_date =~ s/\d+-\d+-(\d+)\s[\d:]+/$1/;

	my $result=0;
	
	if ( $current_date == $h{config}->{last_sequence_date} ) {
		$result = $h{config}->{file_sequence_num}++;
	} else {
		$h{config}->{file_sequence_num} = ++$result;
		$h{config}->{last_sequence_date} = $current_date;
	}
	
	return $result;
}

1;
