## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
package HSPC::MT::Plugin::PP::BT_Norma19::Tools;
use strict;

use HSPC::PluginToolkit::General qw(throw_exception);

##--------------------------------------------------
## Functions
##-------------------------------------------------- 

sub trim {
	my $class = shift;
	my $line = shift;
	$line =~ s/^\s*(.*?)\s*$/$1/;
	return $line;
}

sub date_DDMMYY_to_hspc_format {
	my $class = shift;
	my $date = shift;

	my $result;
	if ($date =~ /^(\d\d)(\d\d)(\d\d)$/) {
		$result = "20$3-$2-$1";
	}

	return $result;
};

sub splitline {
	my $class = shift;
	my %h = (
		line             => undef,
		field_definition => undef,
		@_
	);

	my $line = $h{line} ||
		throw_exception(
			type        => 'error',
			log_message => 'line parameter is mandatory'
		);
	my $field_definition = $h{field_definition} ||
		throw_exception(
			type        => 'error',
			log_message => 'field_definition parameter is mandatory'
		);

	my $result = {};
	while ( my ($field_id, $field_def) = each (%$field_definition)) {
		my $start_pos = $field_def->{start_pos} ||
			throw_exception(
				type        => 'error',
				log_message => 'start_pos is undefined'
			);
		my $len = $field_def->{len} ||
			throw_exception(
				type        => 'error',
				log_message => 'len is undefined'
			);
		$result->{$field_id} = substr(
			$line,
			$start_pos - 1,
			$len 
		);
	}

	return $result;
};

sub format_client_identifier {
	my $class = shift;
	my $identifier = shift;

	my $result = uc($identifier);
	my $len = length($result);
	if ($len < 12) {
		$result .= "0" x ( 12 - $len );
	}
	throw_exception(
		type        => 'error',
		log_message => 'Internal error, cant format client identifier'
	) if (length($result) != 12);
	return $result;
}

1;
