package HSPC::MT::Plugin::PP::OP_Assist::V1;

use strict;

use HSPC::PluginToolkit::General
			qw(secs_to_time get_data_dir throw_exception log log_debug log_warn string round2 argparam uriparam);
use HSPC::PluginToolkit::PP qw(callback_url);

use Data::Dumper;

use constant ASSIST_REQUEST_FIELDS => {
	ORDERNUMBER    => 'ref_number',
	RESPONSE_CODE  => 'response_code',
	RECOMMENDATION => 'recommendation',
	MESSAGE        => 'msg',
	COMMENT        => 'desc',
##		DATE
##		TOTAL
##		CURRENCY
##		CARDTYPE
##		CARDNUMBER
##		LASTNAME
##		FIRSTNAME
##		MIDDLENAME
##		ADDRESS
	EMAIL         => 'email',
##		COUNTRY
##		RATE
##		APPROVALCODE
##		CARDSUBTYPE
##		CVC2
##		CARDHOLDER
	IPADDRESS     => 'customer_ip',
##		PROTOCOLTYPENAME
##		BILLNUMBER
##		BANKNAME
	STATUS        => 'status',
	ERROR_CODE    => 'error_code',
	ERROR_COMMENT => 'error_desc',
##		ProcessingName
};

###############################################################################
## PLUGIN CONSTANTS
###############################################################################

use constant MAP_ASSIST2HSPC => {
	AS000 => {
		is_active => 0,
		mapping   => [{'SALE' => 'APPROVED'}]
	},
	AS001 => {
		is_active => 0,
		mapping   => [{'SALE' => 'APPROVED'}]
	},
	AS010 => {
		is_active => 0,
		mapping   => [{'PREAUTH' => 'APPROVED'}]
	},
	AS011 => {
		is_active => 0,
		mapping   => [{'PREAUTH' => 'APPROVED'}]
	},
	AS020 => {
		is_active => 0,
		mapping   => [{'CAPTURE' => 'APPROVED'}]
	},
	AS100 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS101 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS102 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS104 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS105 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS106 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS107 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS108 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS109 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS200 => {
		is_active => 1,
		mapping   => [
			{'SALE'    => 'ERROR'},
			{'PREAUTH' => 'ERROR'},
			{'CAPTURE' => 'ERROR'},
		]
	},
	AS300 => {
		is_active => 1,
		mapping   => [
			{'SALE'    => 'PENDING'},
			{'PREAUTH' => 'PENDING'},
		]
	},
	AS400 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
			{'CAPTURE' => 'DECLINED'},
		]
	},
	AS998 => {
		is_active => 1,
		mapping   => [
			{'SALE'    => 'ERROR'},
			{'PREAUTH' => 'ERROR'},
			{'CAPTURE' => 'ERROR'},
		]
	},
};

use constant ASSIST_DELIMITER      => ';';

use constant CURRENCIES_MAP => {
	'USD'   => 'USD',
	'RUB'   => 'RUR',
};

###############################################################################
## PUBLIC METHODS
###############################################################################

sub get_title {
	my $class = shift;

	return string('assist_title');
}

sub process_sale{
	my $class = shift;
	return $class->_process_redirect(@_);
}

sub collect_transaction_refno{
	my $class = shift;
	my $ref_no = uriparam('refno');

	return $ref_no;
}

sub process_callback{
	my $class = shift;
	my %h = (
			config  => undef,
			@_
	);

	my $config = $h{config};
	my $ref_no = uriparam('refno');
	my $result = uriparam('Result');

	my $desc;
	my $action;
	my $status;
	my $details;
	my $next_txn_gap;

	if ($result eq 'Success'){
		$status = 'PENDING';
	} elsif ($result eq 'Failure') {
		$status = 'ERROR';
	} else {
		$status = 'ERROR';
	}


	return {
		STATUS               => $status,
		TEXT                 => {
			customer_message => $desc,
		},
		TRANSACTION_DETAILS  => $details,
		NEXT_TRANSACTION_GAP => $next_txn_gap,
		ACTION               => $action,
	};
}

sub _poll_compose_request {
	my $self = shift;
	my %h    = (
		config => undef,
		refno  => undef,
		@_
	);
	my $config = $h{config};
	my $refno  = $h{refno} || '%';

	my $shop_id = $config->{merchant_id};
	my $login   = $config->{login};
	my $passwd  = $config->{passwd};

	my $s_fields = join(';', map {&ASSIST_REQUEST_FIELDS->{$_} . "=$_"} keys %{&ASSIST_REQUEST_FIELDS});

	my ($eday, $emonth, $eyear) = (localtime(time - 60*60*24*3))[3,4,5];
	my ($sday, $smonth, $syear) = ($eday, $emonth + 1, $eyear + 1900);
	return [
		SHOPORDERNUMBER => $refno,
		SHOP_ID         => $shop_id,
		LOGIN           => $login,
		PASSWORD        => $passwd,
		SUCCESS         => 2, ## get all orders
		STARTDAY        => $eday,
		STARTMONTH      => $smonth,
		STARTYEAR       => $syear,
		MEANTYPE        => 0, ## all possible paymethods
		FORMAT          => 1, ## default
		ZIPFLAG         => 0, ## browser
		ENGLISH         => 1, ## english
		HEADER          => 0,
		HEADER1         => 1, ## no need
		DELIMITER       => &ASSIST_DELIMITER,
		OPENDELIMITER   => '[', ## default
		CLOSEDELIMITER  => ']', ## default
		ROWDELIMITER    => 10,  ## newline
		S_FIELDS        => $s_fields,
	];
}

sub _parse_url {
	my $self = shift;
	my $url  = shift;
	$url =~ m{^http(s{0,1})://([\w-.]+)/(.*)};
	my $prot = $1 ? 'https' : 'http';
	my $server = $2;
	my $port   = $prot eq 'https' ? 443 : 80;
	my $path   = length($3) ? ('/' . $3) : '';

	return {
		protocol => $prot,
		server   => $server,
		port     => $port,
		path     => $path,
	}
}

sub _parse_token {
	my $self = shift;
	my %h    = (
		token  => undef,
		fields => undef,
		@_
	);

	my $str = $h{token};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: token undefined'
	) unless $str;

	throw_exception(
		type        => 'error',
		log_message => 'Internal error: fields undefined'
	) unless $h{fields};

	my @fields = @{$h{fields}};
	my $ret;
	my $i = 0;

	my @tokens = split(&ASSIST_DELIMITER, $str);
	for (@tokens) {
		$ret->{$fields[$i]} = $_;
		$i++;
	}

	return $ret;
}

sub _parse_result {
	my $self = shift;
	my %h    = (
		result => undef,
		@_
	);

	my $page = $h{result};

	throw_exception(
		type        => 'error',
		log_message => 'Internal error: No result undefined'
	) unless $page;

	my @tokens = split("\n", $page);
	my $ret;

	## drop empty lines
	while ($tokens[0] =~ /^\s*$/) {shift @tokens};

	my @fields = split(&ASSIST_DELIMITER, shift @tokens);
	throw_exception(
		type        => 'error',
		log_message => "Internal error: Error returned by Assist. Error: $fields[1]"
	) if ($fields[0] eq 'ERROR');

	for (@tokens){
		next if (/^\s*$/);
		push @{$ret}, $self->_parse_token(token => $_, fields => \@fields);
	}

	return $ret;
}

sub _poll_send_request {
	my $self = shift;
	my %h    = (
		config => undef,
		refno  => undef,
		@_
	);
	my $refno  = $h{refno} || '%';
	my $config = $h{config};
	my $ret;

	require LWP::UserAgent;
	my $browser = LWP::UserAgent->new;

	my $req = $self->_poll_compose_request(
		config => $config,
		refno  => $refno,
	);

	my $response = $browser->post($config->{result_url}, $req);

	throw_exception(
		type        => 'error',
		log_message => "Internal error: " . string('srv_comm_err') . ". Reason: " . $response->status_line,
	) unless $response->is_success;

	return $self->_parse_result(result => $response->content);
}

## Description: This methods returns transaction status for the Assist
## response_code.
sub _get_status4 {
	my $self= shift;
	my %h    = (
		status => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	if (&MAP_ASSIST2HSPC->{$st}
			&& scalar @{&MAP_ASSIST2HSPC->{$st}->{mapping}})
	{
		return (values %{&MAP_ASSIST2HSPC->{$st}->{mapping}->[0]})[0];
	}
	else
	{
		return undef;
	}
}

## Description: This methods returns all possible transaction types
##	for given response_code.
sub _get_types4 {
	my $self= shift;
	my %h    = (
		status    => undef,
		curr_type => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	if (&MAP_ASSIST2HSPC->{$st} && scalar @{&MAP_ASSIST2HSPC->{$st}->{mapping}}) {
		return map {keys %{$_}} @{&MAP_ASSIST2HSPC->{$st}->{mapping}};
	} else {
		return undef;
	}
}

## Description: This methods returns corresponding transaction type
## for the Assist status.
##	order status.
sub _get_native_type4 {
	my $self= shift;
	my %h    = (
		status    => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	## Check extended status (for REFUSED with reason REFFERED)
	if (&MAP_ASSIST2HSPC->{$st}
			&& scalar @{&MAP_ASSIST2HSPC->{$st}->{mapping}})
	{
		return (keys %{&MAP_ASSIST2HSPC->{$st}->{mapping}->[0]})[0];
	}
	else
	{
		return undef;
	}
}

sub _get_type4 {
	my $self= shift;
	my %h    = (
		status    => undef,
		curr_type => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	my $t  = $h{curr_type};
	my $ret;

	if (! grep {$t != $_} $self->_get_types4(status => $st)) {
		## Current transaction type differs from available for given status.
		## We must return new trans_type.
		$ret = $self->_get_native_type4(status => $st);
	} else {
		$ret = $t;
	}

	return $ret;
}

sub _compose_refnum {
	my $self = shift;
	my %h = (
				config   => undef,
				order_id => undef,
				@_
			);
	my $ret;
	my $config   = $h{config};
	my $order_id = sprintf "% 7d", $h{order_id};

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(time);
	my $t = sprintf
				"%04s-%02s-%02s %02s:%02s:%02s",
				1900 + $year, $mon, $mday, $hour, $min, $sec;

	my $p = $config->{prefix};
	$ret =
		($p ? "$p " : "")
		. $order_id
		. " "
		. $t;
	## replace all symbols equal to delimeter by spaces
	my $delim = &ASSIST_DELIMITER;
	$ret =~ s/$delim/ /g;

	return $ret;
}

sub process_check_status{
	my $class = shift;
	my %h    = (
		config                    => undef,
		transaction_refnum        => undef,
		previous_transaction_data => undef,
		@_
	);
	my $refno   = $h{transaction_refnum};
	my $config  = $h{config};
	my $details = $h{previous_transaction_data};
	my $res;
	$res = $class->_poll_send_request(
		config => $config,
		refno  => $refno,
	);

	my $error_message;
	my $next_txn_gap;
	my $status = 'PENDING';

	for $details (@{$res}) {
		my $st     = $details->{response_code};
		my $refno  = $details->{ref_number};

		$status = $class->_get_status4(
			status => $st
		);

		my $type   = $class->_get_type4(
			status    => $st,
			curr_type => 'SALE'
		);
	}

	$details->{assist_check_try_number_check}++;
	if ($status eq 'PENDING') {
		if ($details->{assist_check_try_number_check}
			< $config->{num_of_tries}
		) {
			$status       = 'PENDING';
			$next_txn_gap = $config->{check_status_period_min} * 60;
		} else {
			$status        = 'ERROR';
			$error_message = string('assist_tries_over');
		}
	}

	return {
		STATUS               => $status,
		TRANSACTION_DETAILS  => $details,
		TEXT => {
			vendor_message   => $error_message,
		},
		NEXT_TRANSACTION_GAP => $next_txn_gap,
	};
}

sub get_currencies_supported{

	return [
		'RUB',## Russian Ruble
		'USD',## US Dollar
	];
}

sub _process_redirect {
	my $class = shift;
	my %h = (
			document_info => undef,
			ref_no        => undef,
			config        => undef,
			@_
	);

	my $config             = $h{config};
	my $transaction_amount = $h{transaction_amount};
	my $currency_iso       = $h{currency_iso};
	my $document_info      = $h{document_info};
	my $order_id           = $document_info->{id};
	my $refno              = $h{ref_no};
##	$class->_compose_refnum(
##		config   => $config,
##		order_id => $order_id,
##	);

	my $url                = $config->{purchase_url};
	my $merchant_id        = $config->{merchant_id};
	my $test_mode          = $config->{test_mod};
	my $test_response      = $config->{test_response};

	my $description        = $document_info->{description};

	my $next_txn_gap;

	my $redirect_hash = {
		url    => $url,
		method => "POST",
		charset => 'cp1251',
		attrs => {
			Shop_IDP      => $merchant_id,
			Order_IDP     => $refno,
			Subtotal_P    => $transaction_amount,
			Currency      => &CURRENCIES_MAP->{$currency_iso},
			Comment       => $description,
			Delay         => 0, ## Now support sale transaction only
			DemoResult    => $test_mode
								? $test_response
								: '',
			URL_RETURN_NO => callback_url(
				suffix => 'result',
				Result => 'Failure',
				refno  => $refno
			),
			URL_RETURN_OK => callback_url(
				suffix => 'result',
				Result => 'Success',
				refno  => $refno
			),
			CardPayment        => 1,
			WalletPayment      => 1,
			WebMoneyPayment    => 1,
			RapidaPayment      => 1,
			PayCashPayment     => 1,
			EPortPayment       => 1,
			KreditPilotPayment => 1,
			AssistIDCCPayment  => 1,
			QiwiBeelinePayment => 1,
		}
	};

	my $next_txn_gap = $config->{check_status_period_min} * 60;

	return {
		STATUS               => 'REDIRECT',
		REDIRECT_HASH        => $redirect_hash,
		NEXT_TRANSACTION_GAP => $next_txn_gap,
	};
}

1;