package HSPC::MT::Plugin::PP::OP_Assist::V2;
use strict;

use vars qw( $THIRD_DIRNAME );
BEGIN {
	use File::Basename;
	$THIRD_DIRNAME = dirname($INC{"HSPC/MT/Plugin/PP/OP_Assist/V2.pm"}) . "/www";
}

use HSPC::PluginToolkit::General
			qw(secs_to_time get_data_dir throw_exception log log_debug log_warn string round2 argparam uriparam);
use HSPC::PluginToolkit::PP qw(callback_url);

use Data::Dumper;

use lib $THIRD_DIRNAME;
use Mozilla::CA;

require $THIRD_DIRNAME."/LWP.pm";

require $THIRD_DIRNAME."/LWP/Debug.pm";
require $THIRD_DIRNAME."/LWP/DebugFile.pm";
require $THIRD_DIRNAME."/LWP/UserAgent.pm";
require $THIRD_DIRNAME."/LWP/ConnCache.pm";
require $THIRD_DIRNAME."/LWP/MemberMixin.pm";
require $THIRD_DIRNAME."/LWP/RobotUA.pm";

require $THIRD_DIRNAME."/LWP/Protocol/http.pm";

require $THIRD_DIRNAME."/HTTP/Config.pm";
require $THIRD_DIRNAME."/HTTP/Headers.pm";
require $THIRD_DIRNAME."/HTTP/Message.pm";
require $THIRD_DIRNAME."/HTTP/Request.pm";
require $THIRD_DIRNAME."/HTTP/Response.pm";
require $THIRD_DIRNAME."/HTTP/Status.pm";
require $THIRD_DIRNAME."/HTTP/Headers/Auth.pm";
require $THIRD_DIRNAME."/HTTP/Headers/ETag.pm";
require $THIRD_DIRNAME."/HTTP/Headers/Util.pm";
require $THIRD_DIRNAME."/HTTP/Request/Common.pm";

use constant ASSIST_REQUEST_FIELDS => {
	ORDERNUMBER    => 'ref_number',
	RESPONSE_CODE  => 'response_code',
	RECOMMENDATION => 'recommendation',
	MESSAGE        => 'msg',
	COMMENT        => 'desc',
##		DATE
##		TOTAL
##		CURRENCY
##		CARDTYPE
##		CARDNUMBER
##		LASTNAME
##		FIRSTNAME
##		MIDDLENAME
##		ADDRESS
	EMAIL         => 'email',
##		COUNTRY
##		RATE
##		APPROVALCODE
##		CARDSUBTYPE
##		CVC2
##		CARDHOLDER
	IPADDRESS     => 'customer_ip',
##		PROTOCOLTYPENAME
##		BILLNUMBER
##		BANKNAME
	STATUS        => 'status',
	ERROR_CODE    => 'error_code',
	ERROR_COMMENT => 'error_desc',
##		ProcessingName
};

###############################################################################
## PLUGIN CONSTANTS
###############################################################################

use constant MAP_ASSIST2HSPC => {
	AS000 => {
		is_active => 0,
		mapping   => [{'SALE' => 'APPROVED'}]
	},
	AS001 => {
		is_active => 0,
		mapping   => [{'SALE' => 'APPROVED'}]
	},
	AS010 => {
		is_active => 0,
		mapping   => [{'PREAUTH' => 'APPROVED'}]
	},
	AS011 => {
		is_active => 0,
		mapping   => [{'PREAUTH' => 'APPROVED'}]
	},
	AS020 => {
		is_active => 0,
		mapping   => [{'CAPTURE' => 'APPROVED'}]
	},
	AS100 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS101 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS102 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS104 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS105 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS106 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS107 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS108 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS109 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
		]
	},
	AS200 => {
		is_active => 1,
		mapping   => [
			{'SALE'    => 'ERROR'},
			{'PREAUTH' => 'ERROR'},
			{'CAPTURE' => 'ERROR'},
		]
	},
	AS300 => {
		is_active => 1,
		mapping   => [
			{'SALE'    => 'PENDING'},
			{'PREAUTH' => 'PENDING'},
		]
	},
	AS400 => {
		is_active => 0,
		mapping   => [
			{'SALE'    => 'DECLINED'},
			{'PREAUTH' => 'DECLINED'},
			{'CAPTURE' => 'DECLINED'},
		]
	},
	AS998 => {
		is_active => 1,
		mapping   => [
			{'SALE'    => 'ERROR'},
			{'PREAUTH' => 'ERROR'},
			{'CAPTURE' => 'ERROR'},
		]
	},
};

use constant ASSIST_DELIMITER      => ';';

use constant CURRENCIES_MAP => {
	'USD'   => 'USD',
	'RUB'   => 'RUR',
};

###############################################################################
## PUBLIC METHODS
###############################################################################

sub get_title {
	my $class = shift;

	return string('assist_title');
}

sub process_sale{
	my $class = shift;
	return $class->_process_redirect(@_);
}

sub collect_transaction_refno{
	my $class = shift;
	my $ref_no = uriparam('refno');

	return $ref_no;
}

sub process_callback{
	my $class = shift;
	my %h = (
			config  => undef,
			@_
	);

	my $config = $h{config};
	my $ref_no = uriparam('refno');
	my $result = uriparam('Result');

	my $desc;
	my $action;
	my $status;
	my $details;
	my $next_txn_gap;

	if ($result eq 'Success'){
		$status = 'PENDING';
	} elsif ($result eq 'Failure') {
		$status = 'ERROR';
	} else {
		$status = 'ERROR';
	}


	return {
		STATUS               => $status,
		TEXT                 => {
			customer_message => $desc,
		},
		TRANSACTION_DETAILS  => $details,
		NEXT_TRANSACTION_GAP => $next_txn_gap,
		ACTION               => $action,
	};
}

sub process_check_status{
	my $class = shift;
	my %h    = (
		config                    => undef,
		transaction_refnum        => undef,
		previous_transaction_data => undef,
		@_
	);
	my $refno   = $h{transaction_refnum};
	my $config  = $h{config};
	my $details = $h{previous_transaction_data};
	my $res;
	$res = $class->_poll_send_request(
		config => $config,
		refno  => $refno,
	);

	my $error_message;
	my $next_txn_gap;
	my $status = 'PENDING';

	for my $process_details (@{$res}) {
		if ($refno != $process_details->{ordernumber}) {
			next;
		}
		$status = $class->_get_status4(status => $process_details->{responsecode});
	}

	$details->{assist_check_try_number_check}++;
	if ($status eq 'PENDING') {
		if ($details->{assist_check_try_number_check}
			< $config->{num_of_tries}
		) {
			$status       = 'PENDING';
			$next_txn_gap = $config->{check_status_period_min} * 60;
		} else {
			$status        = 'ERROR';
			$error_message = string('assist_tries_over');
		}
	}

	return {
		STATUS               => $status,
		TRANSACTION_DETAILS  => $details,
		TEXT => {
			vendor_message   => $error_message,
		},
		NEXT_TRANSACTION_GAP => $next_txn_gap,
	};
}

sub get_currencies_supported{

	return [
		'RUB',## Russian Ruble
		'USD',## US Dollar
	];
}

###############################################################################
## PRIVATE METHODS
###############################################################################

sub _poll_compose_request {
	my $self = shift;
	my %h    = (
		config => undef,
		refno  => undef,
		@_
	);
	my $config = $h{config};
	my $refno  = $h{refno} || '%';

	my $s_fields = join(';', map {&ASSIST_REQUEST_FIELDS->{$_} . "=$_"} keys %{&ASSIST_REQUEST_FIELDS});

	my ($emin, $ehour, $eday, $emonth, $eyear) = (gmtime(time))[1,2,3,4,5];
	my ($smin, $shour, $sday, $smonth, $syear) = (gmtime(time - 60*60*24))[1,2,3,4,5];
	return [
		##SHOPORDERNUMBER => $refno,
		Merchant_ID     => $config->{merchant_id},
		Login           => $config->{login},
		Password        => $config->{passwd},
		StartDay        => sprintf('%02d', $sday),
		StartMonth      => sprintf('%02d', $smonth + 1),
		StartYear       => sprintf('%04d', $syear + 1900),
		StartHour		=> sprintf('%02d', $shour),
		StartMin		=> sprintf('%02d', $smin),
		EndDay          => sprintf('%02d', $eday),
		EndMonth        => sprintf('%02d', $emonth + 1),
		EndYear         => sprintf('%04d', $eyear) + 1900,
		EndHour		    => sprintf('%02d', $ehour),
		EndMin		    => sprintf('%02d', $emin),
##		MeanType_ID        => 0, ## all possible paymethods
		Format          => 1, ## default
		ZipFlag         => 0, ## browser
		Language        => "EN",
		TestMode		=> $config->{test_mod},
##		S_FIELDS        => $s_fields,
	];
}

sub _parse_url {
	my $self = shift;
	my $url  = shift;
	$url =~ m{^http(s{0,1})://([\w-.]+)/(.*)};
	my $prot = $1 ? 'https' : 'http';
	my $server = $2;
	my $port   = $prot eq 'https' ? 443 : 80;
	my $path   = length($3) ? ('/' . $3) : '';

	return {
		protocol => $prot,
		server   => $server,
		port     => $port,
		path     => $path,
	}
}

sub _parse_token {
	my $self = shift;
	my %h    = (
		token  => undef,
		fields => undef,
		@_
	);

	my $str = $h{token};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: token undefined'
	) unless $str;

	throw_exception(
		type        => 'error',
		log_message => 'Internal error: fields undefined'
	) unless $h{fields};

	my @fields = @{$h{fields}};
	my $ret;
	my $i = 0;

	my @tokens = split(&ASSIST_DELIMITER, $str);
	for (@tokens) {
		$ret->{$fields[$i]} = $_;
		$i++;
	}

	return $ret;
}

sub unquote {
	my $self = shift;
	my $quoted_text = shift;
	$quoted_text =~ s/^(")(.*)(")$/$2/g;
	$quoted_text =~ s/\\\"/\"/g;
	return $quoted_text;
}

sub _parse_result {
	my $self = shift;
	my %h    = (
		result => undef,
		@_
	);

	my $csv = $h{result};

	throw_exception(
		type        => 'error',
		log_message => 'Internal error: No result undefined'
	) unless $csv;

	if ($csv =~ m/error;([^;]*);([^;]*);/) {
		throw_exception(
			type        => 'error',
			log_message => "Internal error: Error returned by Assist. Error: $1:$2"
		);
		return;
	}

	my @csv_lines = split(/\n/, $csv);
	my $column_line = shift @csv_lines;
	my @columns = map {$self->unquote($_)} split(/;/, $column_line);
	my @result = ();

	while (my $line = shift @csv_lines) {
		my @values = split(/;/, $line);
		my %csv_params = ();
		for (my $i = 0; $i < @columns; $i++) {
			$csv_params{$columns[$i]} = $self->unquote($values[$i]);
		}
		push @result, \%csv_params;
	}

	return \@result;
}

sub _poll_send_request {
	my $self = shift;
	my %h    = (
		config => undef,
		refno  => undef,
		@_
	);
	my $refno  = $h{refno} || '%';
	my $config = $h{config};
	my $ret;

##	die(Dumper(%INC));
	my $browser = LWP::UserAgent->new;
	$browser->ssl_opts(
		verify_hostname => 1,
		SSL_ca_file => Mozilla::CA::SSL_ca_file(),
	);
	$browser->agent("Mozilla/5.0");
	$browser->default_header("Accept" => "*/*");

	my $req = $self->_poll_compose_request(
		config => $config,
		refno  => $refno,
	);

	log("[OP_Assist] Send request: ".Dumper($req));
	my $response = $browser->post($config->{result_url}, $req);

	throw_exception(
		type        => 'error',
		log_message => "Internal error: " . string('srv_comm_err') . ". Reason: " . $response->status_line,
	) unless $response->is_success;

	log("[OP_Assist] Response: ".Dumper($response->content));

	return $self->_parse_result(result => $response->content);
}

## Description: This methods returns transaction status for the Assist
## response_code.
sub _get_status4 {
	my $self= shift;
	my %h    = (
		status => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	if (&MAP_ASSIST2HSPC->{$st}
			&& scalar @{&MAP_ASSIST2HSPC->{$st}->{mapping}})
	{
		return (values %{&MAP_ASSIST2HSPC->{$st}->{mapping}->[0]})[0];
	}
	else
	{
		return undef;
	}
}

## Description: This methods returns all possible transaction types
##	for given response_code.
sub _get_types4 {
	my $self= shift;
	my %h    = (
		status    => undef,
		curr_type => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	if (&MAP_ASSIST2HSPC->{$st} && scalar @{&MAP_ASSIST2HSPC->{$st}->{mapping}}) {
		return map {keys %{$_}} @{&MAP_ASSIST2HSPC->{$st}->{mapping}};
	} else {
		return undef;
	}
}

## Description: This methods returns corresponding transaction type
## for the Assist status.
##	order status.
sub _get_native_type4 {
	my $self= shift;
	my %h    = (
		status    => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	## Check extended status (for REFUSED with reason REFFERED)
	if (&MAP_ASSIST2HSPC->{$st}
			&& scalar @{&MAP_ASSIST2HSPC->{$st}->{mapping}})
	{
		return (keys %{&MAP_ASSIST2HSPC->{$st}->{mapping}->[0]})[0];
	}
	else
	{
		return undef;
	}
}

sub _get_type4 {
	my $self= shift;
	my %h    = (
		status    => undef,
		curr_type => undef,
		@_
	);
	my $st = $h{status};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: status undefined'
	) unless $st;

	my $t  = $h{curr_type};
	my $ret;

	if (! grep {$t != $_} $self->_get_types4(status => $st)) {
		## Current transaction type differs from available for given status.
		## We must return new trans_type.
		$ret = $self->_get_native_type4(status => $st);
	} else {
		$ret = $t;
	}

	return $ret;
}

sub _compose_refnum {
	my $self = shift;
	my %h = (
				config   => undef,
				order_id => undef,
				@_
			);
	my $ret;
	my $config   = $h{config};
	my $order_id = sprintf "% 7d", $h{order_id};

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(time);
	my $t = sprintf
				"%04s-%02s-%02s %02s:%02s:%02s",
				1900 + $year, $mon, $mday, $hour, $min, $sec;

	my $p = $config->{prefix};
	$ret =
		($p ? "$p " : "")
		. $order_id
		. " "
		. $t;
	## replace all symbols equal to delimeter by spaces
	my $delim = &ASSIST_DELIMITER;
	$ret =~ s/$delim/ /g;

	return $ret;
}

sub _process_redirect {
	my $class = shift;
	my %h = (
			document_info => undef,
			ref_no        => undef,
			config        => undef,
			@_
	);

	my $config             = $h{config};
	my $transaction_amount = $h{transaction_amount};
	my $currency_iso       = $h{currency_iso};
	my $document_info      = $h{document_info};
	my $account_info       = $h{account_info};
	my $order_id           = $document_info->{id};
	my $refno              = $h{ref_no};
##	$class->_compose_refnum(
##		config   => $config,
##		order_id => $order_id,
##	);

	my $url                = $config->{purchase_url};
	my $merchant_id        = $config->{merchant_id};
	my $test_mode          = $config->{test_mod};
	my $test_response      = $config->{test_response};

	my $description        = $document_info->{description};

	my $next_txn_gap;

	my $redirect_hash = {
		url    => $url,
		method => "POST",
		charset => 'cp1251',
		attrs => {
			## Merchant info
			Merchant_ID		=> $merchant_id,

			## Order info
			OrderNumber		=> $refno,
			OrderAmount		=> $transaction_amount,
			OrderCurrency	=> &CURRENCIES_MAP->{$currency_iso},
			OrderComment	=> $description,

			## Contact info
			Firstname		=> $account_info->{fname},
			Lastname		=> $account_info->{lname},
			Email			=> $account_info->{email},

			## Payment method info
			CardPayment		=> 1,
			WebMoneyPayment	=> 1,
			AssistIDPayment	=> 1,
			QIWIPayment		=> 1,

			## API settings
			Delay			=> 0, ## Now support sale transaction only
			TestMode		=> $test_mode,
			URL_RETURN_NO	=> callback_url(
				suffix => 'result',
				Result => 'Failure',
				refno  => $refno
			),
			URL_RETURN_OK	=> callback_url(
				suffix => 'result',
				Result => 'Success',
				refno  => $refno
			),
		}
	};

	my $next_txn_gap = $config->{check_status_period_min} * 60;

	return {
		STATUS               => 'REDIRECT',
		REDIRECT_HASH        => $redirect_hash,
		NEXT_TRANSACTION_GAP => $next_txn_gap,
	};
}

sub _format_phone
{
	my $me = shift;
	my $phone_src = shift;
	my $phone = "";

	if ($phone_src->{country_code}) {
		$phone = "+".$phone_src->{country_code}." ";
	}
	if ($phone_src->{area_code}) {
		$phone .= $phone_src->{area_code}." ";
	}
	if ($phone_src->{number}) {
		$phone .= $phone_src->{number}." ";
	}
	if ($phone_src->{extension}) {
		$phone .= $phone_src->{extension};
	}
	$phone =~ s/\s+$//g;

	return $phone;
}

1;