package HSPC::MT::Plugin::PP::OP_Atos;

use strict;

use HSPC::PluginToolkit::General qw(throw_exception log get_data_dir string round log_debug uriparam);
use HSPC::PluginToolkit::PP qw(callback_url);
use Data::Dumper;

sub get_title {
	my $class = shift;
	
	return string('atos_plugin_title');
}

sub process_sale {
	my $class = shift;
    my %h = (
        config             => undef,
        transaction_amount => undef,
        currency_iso       => undef,
        ref_no             => undef,
        document_info      => undef,
        @_
    );
	log_debug ("prepare_trans_data():: Called! Let's prepare data.");

    my $config             = $h{config};
    my $transaction_amount = $h{transaction_amount};
    my $currency_iso       = $h{currency_iso};
    my $ref_no             = $h{ref_no};
	my $document_info      = $h{document_info};

    ## shortcut
    my $parm = "merchant_id=" . $config->{merchant_id};
    $parm .= " merchant_country=fr";

    $parm .= " amount=" . round( $transaction_amount * 100 );
    $parm .= " currency_code=" . &SUPPORTED_CURRENCIES->{$currency_iso};
	$parm .= " caddie=" . $ref_no;
##    $parm .= " templatefile=" . $plugin->template_filename ## TODO
##    if $plugin->template_filename;
    $parm .= " pathfile=" . $class->path_file;
	$parm .= " order_id=" . $document_info->{id};
	$parm .= " normal_return_url=" .
					callback_url(
						action => 'result',
						result => 'ok',
						ref_no => $ref_no,
					);
	$parm .= " cancel_return_url=" .
					callback_url(
							action => 'result',
							result => 'fail',
							ref_no => $ref_no,
					);
	$parm .= " automatic_response_url=" .
					callback_url(
							action => 'notify',
							ref_no => $ref_no,
					);
	
	my $path_bin = $class->request_binary;
	log_debug ("process_sale():: PARM=".$parm."\t PATH_BIN=".$path_bin);

	my $result = 0;
	my $i;
	open(INFO, $path_bin . " " . $parm . "|");
	for ($i = 0; <INFO>; $i++)	{
		$result = $result . $_;
	}
	close(INFO);

	my @tableau = split("!",$result);

	my $code	= $tableau[1];
	my $error	= $tableau[2];
	my $message	= $tableau[3];

	if (( $code eq "" ) && ( $error eq "" ) ) {
        throw_exception(
            type           => 'error',
            log_message    => "prepare_trans_data():: Error binary path",
            vendor_message => "Error binary path or binary 'request' broken",
        );
 	}
	elsif ( $code != 0 ) {
        throw_exception(
            type           => 'error',
            log_message    => "process_sale():: Error during construct encoded string: $error",
            vendor_message => "Error in parameters for 'request' binary",
        );
	}
	my $redirect_hash = { content => $message, };

	return {
		STATUS        => 'REDIRECT',
		REDIRECT_HASH => $redirect_hash,
	};
}
#########################################
## CallBack functions
#########################################
sub collect_transaction_refno{
	my $class = shift;
	my $ref_no = uriparam('ref_no');
	return $ref_no;
}

sub process_callback{
	my $class = shift;
	my %h = (@_);

	my $action = uriparam('action');
	my $ret;

	if ($action eq 'notify') {
		$ret = $class->_process_notification(%h);
	} else {
		my $result = uriparam('result');
		my $status;
		my $desc;

		if ($result eq 'ok') {
			$status = 'APPROVED';
		} elsif ($result eq 'fail') {
			$status = 'DECLINED';
			$desc   = string('err_trans_explained');
		}

		$ret = {
			STATUS               => $status,
			TEXT                 => {
				customer_message => $desc,
			},
			ACTION               => 'restore_session',
		};
	}
	return $ret;
}

sub _process_notification {
	my $class = shift;
	my $err;
	my %h = (
		config             => undef,
		currency_iso       => undef,
		document_info      => undef,
		transaction_amount => undef,
		@_
	);
	
	my $transaction_amount = $h{transaction_amount};
	my $document_info      = $h{document_info};
	my $config             = $h{config};
	my $currency_iso       = $h{currency_iso};
	my $ret;

	log_debug("_process_notification: starts..");
	##
	## Read input
	##
	my %response = %{$ENV{args}};

	log_debug ("callback():: Atos got result of transaction: ".
				  join('; ', map { "$_ => $response{$_}" } keys %response));
	## decode string
	my $message  = "message=" . $response{DATA};
	my $pathfile = "pathfile=" . $class->path_file;
	my $path_bin = $class->response_binary;
	my $parm     = $message . " " . $pathfile;

	my $result = 0;

	open(INFO, $path_bin . " " . $parm . "|");
	for (my $i = 0 ; <INFO> ; $i++) {
		$result = $result . $_;
	}
	close(INFO);

	my @tableau = split("!", $result);

	## get result of transaction
	my %res = ();
	$res{code}                = $tableau[1];
	$res{error}               = $tableau[2];
	$res{merchant_id}         = $tableau[3];
	$res{merchant_country}    = $tableau[4];
	$res{amount}              = $tableau[5];
	$res{transaction_id}      = $tableau[6];
	$res{payment_means}       = $tableau[7];
	$res{transmission_date}   = $tableau[8];
	$res{payment_time}        = $tableau[9];
	$res{payment_date}        = $tableau[10];
	$res{response_code}       = $tableau[11];
	$res{payment_certificate} = $tableau[12];
	$res{authorisation_id}    = $tableau[13];
	$res{currency_code}       = $tableau[14];
	$res{card_number}         = $tableau[15];
	$res{cvv_flag}            = $tableau[16];
	$res{cvv_response_code}   = $tableau[17];
	$res{bank_response_code}  = $tableau[18];
	$res{complementary_code}  = $tableau[19];
	$res{return_context}      = $tableau[20];
	$res{caddie}              = $tableau[21];
	$res{receipt_complement}  = $tableau[22];
	$res{merchant_language}   = $tableau[23];
	$res{language}            = $tableau[24];
	$res{customer_id}         = $tableau[25];
	$res{order_id}            = $tableau[26];
	$res{customer_email}      = $tableau[27];
	$res{customer_ip_address} = $tableau[28];
	$res{capture_day}         = $tableau[29];
	$res{capture_mode}        = $tableau[30];
	$res{data}                = $tableau[31];

	log_debug ("callback():: Parameters hash=" . Dumper(\%res));
	## Check input
	my $curr = $res{currency_code};
	my $order_id = $res{order_id};
	my $total = $res{amount};

	if ($total != round($transaction_amount * 100)
	) {
		## Bad total
		$err = "Order total (order # $order_id) should be "
				. round($transaction_amount * 100) . ". "
				. "Order total received: $total. ";
	}
	if (!$err
		&& $curr ne &SUPPORTED_CURRENCIES->{$currency_iso}
	) {
		## Bad merchant shop settings
		$err .= 
			"Currency does not equal "
			. "one received from the gateway. "
			. &SUPPORTED_CURRENCIES->{$currency_iso}
			. ". "
			. "Currency recieved: $curr.";
	}
    throw_exception(
        type           => 'error',
        log_message    => $err,
        vendor_message => $err,
    ) if $err;
	##
	## All seems fine. Do real job.
	## Resolve details
	##
	my $details;
	$details->{atos_transaction_id}     = $res{transaction_id};
	$details->{atos_payment_time}       = $res{payment_time};
	$details->{atos_cvv_flag}           = $res{cvv_flag};
	$details->{atos_code}               = $res{code};
	$details->{atos_error}              = $res{error} if $res{error};
	$details->{atos_response_code}      = $res{response_code};
	$details->{atos_payment_date}       = $res{payment_date};
	$details->{atos_cvv_response_code}  = $res{cvv_response_code};
	$details->{atos_bank_response_code} = $res{bank_response_code};
	$details->{atos_transmission_date}  = $res{transmission_date};
	$details->{atos_authorisation_id}   = $res{authorisation_id};
	##
	## Resolve type and status of transaction
	##
	my $status;
	my $refnum = $res{transaction_id};
	my $resp_desc = &RESPONSE_MESSAGES->{$res{response_code}};
	my $resp_code = $res{response_code};
	if($resp_code eq '02') {
		$status = "AUTHCALL";
	}
	elsif($resp_code eq '00') {
		$status = "APPROVED";
	}
	## service temporary unabailable
	elsif($resp_code eq '90') {
		log("process_sale():: ATOS service temporary unabailable. Return TRANS_ERROR.");
		$status = "ERROR";
		$err = string('atos_service_temp_unavailable');
		$details->{atos_error} = $err;
	}
	else {
		log ("process_sale():: Error while performing transaction or transaction was declined by ATOS.");
		## Unknown error
		$status = "DECLINED";
		$err = $resp_desc ||
					string('unexpected_error_trying_to_process_payment');
		$details->{atos_error} = $err;
	}
	$ret = {
		STATUS               => $status,
		ACTION               => 'update',
		CUSTOM_RESPONSE      => '',
		TRANSACTION_DETAILS  => $details,
	};
	log_debug("_process_notification: completed.");
	return $ret;
}

#########################################
## (END) CallBack functions
#########################################

#########################################
## File handle and path functions 
#########################################
sub request_binary {
	my $self = shift;
	return &ATOS_SHARE_PATH.'request';
}

sub response_binary {
	my $self = shift;
	return &ATOS_SHARE_PATH.'response';
}

sub cert_file {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config = $h{config};
	
	return get_data_dir . '/certif.fr.'.$config->{merchant_id};
}

sub path_file {
    my $class     = shift;
    my $path_file = get_data_dir . "/pathfile";

    unless ( -e $path_file ) {
        log_debug( "path_file():: Creating pathfile (" . $path_file . ")" );
        open( FILEOUT, "> " . $path_file )
          || throw_exception(
            type           => 'error',
            log_message    => "Can't create file $path_file",
            vendor_message => "Patchfile error",
          );
        print FILEOUT "D_LOGO!"    . &ATOS_LOGO_PATH  . "!\n";
        print FILEOUT "F_CERTIFICATE!" . get_data_dir . "/certif!\n";
        print FILEOUT "F_PARAM!"       . get_data_dir . "/parmcom!\n";
        print FILEOUT "F_DEFAULT!"
          . &ATOS_SHARE_PATH
          . 'parmcom.sogenactif' . "!\n";
        close(FILEOUT);
    }
	return $path_file;
}

sub parmcom_file {
    my $class = shift;
    my %h     = (
        config => undef,
        @_
    );
    my $config       = $h{config};
    my $parmcom_file = get_data_dir . '/parmcom.' . $config->{merchant_id};

    unless ( -e $parmcom_file ) {
        log_debug("restore_from_args():: Creating parmcom file");
        open( FILEOUT, "> " . $parmcom_file ) || throw_exception(
            type           => 'error',
            log_message    => "Can't create file $parmcom_file",
            vendor_message => "parmcom file error",
        );
        print FILEOUT
          "LOGO!merchant.gif!\nCANCEL_URL!http://!\nRETURN_URL!http://!\n";
        close(FILEOUT);
    }
	return $parmcom_file;
}

sub sogenactif_parmcom_file {
	return &ATOS_SHARE_PATH . 'parmcom.sogenactif';
}

#########################################
## END (File handle and path functions)
#########################################

#########################################
## 
#########################################
use constant ATOS_SHARE_PATH => '/usr/share/hspc-plugin-pp-op-atos/';
use constant ATOS_LOGO_PATH => '/data/atos_logo/';

use constant SUPPORTED_CURRENCIES => {
	'USD'	=> 840,
	'EUR'	=> 978,
	'GBP'	=> 826,
	'CHF'	=> 756,
	'CAD'	=> 124,
	'JPY'	=> 392,
};

sub get_currencies_supported{
	my @curricies = keys %{&SUPPORTED_CURRENCIES};

	return \@curricies;
}

use constant RESPONSE_MESSAGES => {
	'00'	=> 'Approved',
	'02'	=> 'referral : authorization needed by phone',
	'03'	=> 'invalid merchant_id, check the value filled during the call, and contact your SIPS correspondent.',
	'05'	=> 'declined, no detail given for system security',
	'12'	=> 'invalid transaction, unrelated fields, example payment_means : AMEX with no AMEX contract associated to this merchant_id',
	'13'	=> 'invalid amount, check the value filled during the call, and contact your SIPS correspondent',
	'14'	=> 'customer_id invalid',
	'17'	=> 'customer cancellation',
	'30'	=> 'format error, check your implementation and contact your SIPS correspondent.',
	'63'	=> 'security rules not respected, exchange stopped, inform your SIPS correspondent',
	'75'	=> 'allowable number of tries exceeded (Max 3 tries)',
	'90'	=> 'service temporarily not available',
	'94'	=> 'duplicate request, transaction reference already processed',
};
1;
