package HSPC::MT::Plugin::PP::OP_BancaSellaSSL::OP_BancaSellaAPI;
use HSPC::PluginToolkit::General qw(log_debug);
use strict;

use LWP::UserAgent;

##$VERSION = '0.41';
sub new {
	my $class = shift;
	my $self = {
		# public methods
		AlertCode          => '',
		AlertDescription   => '',
		Amount             => '',
		AuthorizationCode  => '',
		BankTransactionID  => '',
		BuyerEmail         => '',
		BuyerName          => '',
		CardNumber         => '',
		Country            => '', # add 0.30
		Currency           => '',
		CustomInfo         => '',
		CVV                => '', # add 0.30
		EncryptedString    => '',
		Encryption         => '', # add 0.30
		ErrorCode          => '',
		ErrorDescription   => '',
		ExpMonth           => '',
		ExpYear            => '',
		Language           => '',
		MIN                => '', # add 0.30
		PasswordEncrypt    => '', # add 0.30
		ShopLogin          => '',
		ShopTransactionID  => '',
		TransactionResult  => '',
		VBV                => '', # add 0.30
		VBVrisp            => '', # add 0.30
		# private methods
		Decripted          => '',

#        DomainName         => 'ecomm.sella.it',
		ScriptEncrypt      => '/CryptHTTP/Encrypt.asp',
		ScriptDecrypt      => '/CryptHTTP/Decrypt.asp',

		ProtocolAuthServer	=> "https://",
		DomainName			=> "ecomm.sella.it",
		ToBeEncript        => '',
	};
	return bless $self, $class;
}

#
# implements the Get<Attribute> and Set<Attribute> methods
#
sub AUTOLOAD {
	my ($self,$value) = @_;
	my %permission = (
		AlertCode          => 'g',
		AlertDescription   => 'g',
		Amount             => 'gs',
		AuthorizationCode  => 'g',
		BankTransactionID  => 'g',
		BuyerEmail         => 'gs',
		BuyerName          => 'gs',
		CardNumber         => 'gs',
		Currency           => 'gs',
		Country            => 'g', # add 0.30
		CustomInfo         => 'gs',
		CVV                => 's', # add 0.30
		EncryptedString    => 'gs',
		Encryption         => 's', # add 0.30
		ErrorCode          => 'g',
		ErrorDescription   => 'g',
		ExpMonth           => 'gs',
		ExpYear            => 'gs',
		Language           => 's',
		MIN                => 's', # add 0.30
		PasswordEncrypt    => 's', # add 0.30
		ShopLogin          => 'gs',
		ShopTransactionID  => 'gs',
		TransactionResult  => 'gs',
		VBV                => 'g', # add 0.30
		VBVrisp            => 'g', # add 0.30
	);
	my $method = $HSPC::MT::Plugin::PP::OP_BancaSellaSSL::OP_BancaSellaAPI::AUTOLOAD; # add 0.40
	$method =~ /::(Get|Set)(.*)$/;
	if ( $1 eq 'Get' && ( $permission{$2} eq 'g' || $permission{$2} eq 'gs' )  ) {
		return $self->{$2};
	} elsif ( $1 eq 'Set' && ( $permission{$2} eq 's' || $permission{$2} eq 'gs' ) ) {
		$self->{$2} = $value;
		return;
	} else {
		my ($package,$filename,$line) = caller(); # add 0.40
		die "The method $method don't exists at $filename line $line\n";
	}
}

# add 0.30
sub SetWithoutEncryption {
	my $self = shift;
	$self->{'Encryption'} = 0;
	return;
}

# add 0.30
sub SetShopTransactionID {
	my ($self,$string) = @_;
	$self->{'ShopTransactionID'} = url_encode(trim($string));
	return;
}

# add 0.30
sub SetBuyerName {
	my ($self,$string) = @_;
	$self->{'BuyerName'} = url_encode(trim($string));
	return;
}

# add 0.30
sub SetBuyerEmail {
	my ($self,$string) = @_;
	$self->{'BuyerEmail'} = trim($string);
	return;
}

# add 0.30
sub SetLanguage {
	my ($self,$string) = @_;
	$self->{'Language'} = trim($string);
	return;
}

# add 0.30
sub SetCustomInfo {
	my ($self,$string) = @_;
	$self->{'CustomInfo'} = url_encode(trim($string));
	return;
}

# add 0.30
sub GetShopTransactionID {
	my $self = shift;
	return url_decode($self->{'ShopTransactionID'});
}

# add 0.30
sub GetBuyerName {
	my $self = shift;
	return url_decode($self->{'BuyerName'});
}

# add 0.30
sub GetCustomInfo {
	my $self = shift;
	return url_decode($self->{'CustomInfo'});
}

sub Encrypt {
	my $self = shift;

	$self->{'ErrorCode'} = '0';
	$self->{'ErrorDescription'} = '';

	# verify the attributes
	if ( $self->{'ShopLogin'} eq '' ) {
		$self->{'ErrorCode'} = '546';
		$self->{'ErrorDescription'} = 'IDshop not valid';
		return 0;
	}
	if ( $self->{'Currency'} eq '' ) {
		$self->{'ErrorCode'} = '552';
		$self->{'ErrorDescription'} = 'Currency not valid';
		return 0;
	}
	if ( $self->{'Amount'} eq '' ) {
		$self->{'ErrorCode'} = '553';
		$self->{'ErrorDescription'} = 'Amount not valid';
		return 0;
	}
	if ( $self->{'ShopTransactionID'} eq '' ) {
		$self->{'ErrorCode'} = '551';
		$self->{'ErrorDescription'} = 'Shop Transaction ID not valid';
		return 0;
	}

	# prepare the string to crypt
	my @to_be_encript = ();
	push @to_be_encript, ("PAY1_CVV=$self->{'CVV'}") if $self->{'CVV'} ne ''; # add 0.30
	push @to_be_encript, ("PAY1_MIN=$self->{'MIN'}") if $self->{'MIN'} ne ''; # add 0.30
	push @to_be_encript, ("PAY1_UICCODE=$self->{'Currency'}") if $self->{'Currency'} ne '';
	push @to_be_encript, ("PAY1_AMOUNT=$self->{'Amount'}") if $self->{'Amount'} ne '';
	push @to_be_encript, ("PAY1_SHOPTRANSACTIONID=$self->{'ShopTransactionID'}") if $self->{'ShopTransactionID'} ne '';
	push @to_be_encript, ("PAY1_CHNAME=$self->{'BuyerName'}") if $self->{'BuyerName'} ne '';
	push @to_be_encript, ("PAY1_CHEMAIL=$self->{'BuyerEmail'}") if $self->{'BuyerEmail'} ne '';
	push @to_be_encript, ("PAY1_IDLANGUAGE=$self->{'Language'}") if $self->{'Language'} ne '';

	push @to_be_encript, ("PAY1_CARDNUMBER=$self->{'CardNumber'}") if $self->{'CardNumber'} ne '';
	push @to_be_encript, ("PAY1_EXPMONTH=$self->{'ExpMonth'}") if $self->{'ExpMonth'} ne '';
	push @to_be_encript, ("PAY1_EXPYEAR=$self->{'ExpYear'}") if $self->{'ExpYear'} ne '';

	push @to_be_encript, ($self->{'CustomInfo'}) if $self->{'CustomInfo'} ne '';
	$self->{'ToBeEncript'} = join('*P1*',@to_be_encript);
#    $self->{'ToBeEncript'} =~ s/ //g; # add comment 0.30
	my $dump_str = $self->{'ToBeEncript'};
	$dump_str =~ s/PAY1_CARDNUMBER=\d+/PAY1_CARDNUMBER=****/;

	log_debug ("Encrypted string: " . $dump_str );

	# crypt the string
	return $self->query_server();
}

sub Decrypt {
	my $self = shift;

	$self->{'ErrorCode'} ='';
	$self->{'ErrorDescription'} = '';

	# verify the attributes
	if ( $self->{'ShopLogin'} eq '' ) {
		$self->{'ErrorCode'} = '546';
		$self->{'ErrorDescription'} = 'IDshop not valid';
		return 0;
	}
	if ( $self->{'EncryptedString'} eq '' ) {
		$self->{'ErrorCode'} = '1009';
		$self->{'ErrorDescription'} = 'String to Decrypt not valid';
		return 0;
	}

	# decrypt the string
	unless ( $self->query_server() ) {
		return 0;
	}

	# get the attributes from the string
#    $self->{'Decripted'} =~ s// /g; # add comment 0.30
#    if ( $self->{'Decripted'} eq '' ) {
	if ( trim($self->{'Decripted'}) eq '' ) { # set 0.30
		$self->{'ErrorCode'} = '99999';
		$self->{'ErrorDescription'} = 'Void String';
		return 0;
	}
	log_debug ("Decripted string: " . $self->{'Decripted'} );
	my %fields = (
		PAY1_ALERTCODE         => 'AlertCode',
		PAY1_ALERTDESCRIPTION  => 'AlertDescription',
		PAY1_AMOUNT            => 'Amount',
		PAY1_AUTHORIZATIONCODE => 'AuthorizationCode',
		PAY1_BANKTRANSACTIONID => 'BankTransactionID',
		PAY1_CARDNUMBER        => 'CardNumber',
		PAY1_CHEMAIL           => 'BuyerEmail',
		PAY1_CHNAME            => 'BuyerName',
		PAY1_COUNTRY           => 'Country',  # add 0.30
		PAY1_ERRORCODE         => 'ErrorCode',
		PAY1_ERRORDESCRIPTION  => 'ErrorDescription',
		PAY1_EXPMONTH          => 'ExpMonth',
		PAY1_EXPYEAR           => 'ExpYear',
		PAY1_IDLANGUAGE        => 'Language',
		PAY1_SHOPTRANSACTIONID => 'ShopTransactionID',
		PAY1_TRANSACTIONRESULT => 'TransactionResult',
		PAY1_UICCODE           => 'Currency',
		PAY1_VBV               => 'VBV', #add 0.30
		PAY1_VBVRISP           => 'VBVrisp', #add 0.30
	);
	foreach my $field ( keys %fields ) {
		if ( $self->{'Decripted'} =~ s/(^|\*P1\*)$field=(.*?)(\*P1\*|$)/$1 && $3 ? '*P1*' : ''/e ) {
			$self->{$fields{$field}} = $2;
		}
	}
	$self->{'CustomInfo'} = trim($self->{'Decripted'});
##	$self->{'TransactionResult'} = 'OK';  ### UNCOMMENT FOR TEST SUCESFULL TRANSACTION
	## DO NOT FORGET UNCOMMENT IN LIVE
	return 1;
}

sub query_server {
	my $self = shift;

	my $type = $self->{'ToBeEncript'} ne '' ? 'encrypt' : 'decrypt';

	my $urlString;
	my $cgi = $self->{ProtocolAuthServer}.$self->{DomainName};

	if ($type eq 'encrypt') {
		$urlString =  "a=$self->{'ShopLogin'}&b=$self->{'ToBeEncript'}&c=2.0";
		$cgi.=$self->{ScriptEncrypt};
	}
	else
	{
		$urlString = "a=$self->{'ShopLogin'}&b=$self->{'EncryptedString'}&c=2.0";
		$cgi.=$self->{ScriptDecrypt};
	}

	my $response = $self->cat_server($urlString,$cgi);
	return 0 if $response eq '';

	if ( $type eq 'encrypt' && $response =~ /#cryptstring#(.*?)#\/cryptstring#/ ) {
		$self->{'EncryptedString'} = $1 if ( $1 ne '' );
	}
	if ( $type eq 'decrypt' && $response =~ /#decryptstring#(.*?)#\/decryptstring#/ ) {
		$self->{'Decripted'} = $1 if ( $1 ne '' );
	}
	if ( $response =~ /#error#(.*?)-(.*?)#\/error#/ ) {
		close(Server);
		$self->{'ErrorCode'} = $1;
		$self->{'ErrorDescription'} = $2;
		return 0;
	}

	close(Server);
	return 1;
}

sub cat_server {
	my ($self,$request,$url) = @_;

	my $response = '';
	my $ua = new LWP::UserAgent;
	my $req = new HTTP::Request "POST",$url;
	$req->content_type("application/x-www-form-urlencoded");
	$req->content($request);

	my $res = $ua->request($req);

	if ($res->is_error) {
		$response = undef;
	} else {
		$response = $res->content;
	}

	if ( $response eq '' ) {
		$self->{'ErrorCode'} = '9999';
		$self->{'ErrorDescription'} = "Unable to get the response from GestPay server: ".$res->error_as_HTML;
		return;
	}
	return $response;
}

# add 0.30
sub trim {
	my $string = shift;
	$string =~ s/^ +//;
	$string =~ s/ +$//;
	return $string;
}

# add 0.30
my %escape;
for my $char ( 0..255 ) {
	$escape{chr($char)} = sprintf("%%%02X",$char);
}

# add 0.30
sub url_encode {
	my $string = shift;
	$string =~ s/([^A-Za-z0-9\-_.* ])/$escape{$1}/g;
	$string =~ tr/ /+/;
	return $string;
}

# add 0.300
sub url_decode {
	my $string = shift;
	$string =~ tr/+/ /;
	$string =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/eg;
	return $string;
}

sub DESTROY { } # add 0.40
1;
