## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
## Abstract base class for all plugin configurations
package HSPC::MT::PluginConfig;

use strict;

use HSPC::MT::Core::Constants;
use HSPC::MT::PluginTools qw(set_request_data);
use HSPC::PluginToolkit::General qw(throw_exception);

use vars qw($AUTOLOAD $DEBUG);

sub AUTOLOAD {
    my $self = shift;
    my $class = ref($self);
    return unless $class;
    my $attr = $AUTOLOAD;
    $attr =~ s!.*::!!;
	return if ( $attr =~ /^[A-Z]*$/ ); ## skip DESTROY and all-cap methods

	$self->{$attr} = shift if @_;
	return $self->{$attr};
}

sub new {
    my $class = shift;
	my %h = @_;
	my $self = {};
	bless ($self, $class);
	return $self;
}

## find template for plugin
sub find_mt_template{
	my $self = shift;
	my %h = (
		plugin_type => undef,
		template_id => undef,
		@_
	);
	my $plugin_type = $h{plugin_type}
		|| $self->{plugin_type};
	my $template_id = $h{template_id}
		|| $self->{template_id};
	my $template = 'HSPC::MT::Plugin::'
		. $plugin_type
		. '::'
		. $template_id;

	eval "use $template";

	if ( $self->check_template( template => $template ) ) {
		return $template;
	}

	return undef;
}

## find presentation module for plugin
sub find_presentation_template{
	my $self = shift;
	my %h = (
		plugin_type => undef,
		template_id => undef,
		@_
	);
	my $plugin_type = $h{plugin_type}
		|| $self->{plugin_type};
	my $template_id = $h{template_id}
		|| $self->{template_id};
	my $template = 'HSPC::Plugin::'
		. $plugin_type
		. '::'
		. $template_id;	

	eval "use $template";

	if ( $self->check_template( template => $template ) ) {
		return $template;
	}

	return undef;
}

## check existence template in system
sub check_template {
	my $self = shift;
	my %h = (
		template => undef,
		@_
	);

	my $template = $h{template};

	$template =~ s/::/\//g;
	$template .= ".pm";

	if ( grep { $_ eq $template } keys (%INC) ) {
		return 1;
	}
	return undef;
}

## Find templates installed in system
## for $plugin_type plugins
## Return array of hashes - template_id => module name
sub find_templates{
	my $class = shift;
	my %h = (
		plugin_type      => undef,
		vendor_id        => undef,
		force_permission => undef,
		@_
	);

	my $plugin_type      = $h{plugin_type};
	my $vendor_id        = $h{vendor_id};
	my $force_permission = $h{force_permission};

	throw_exception(
		type        => 'error',
		log_message => 'plugin_type is a mandatory parameter'
	) unless $plugin_type;

	if ($h{vendor_id} == SW_HSP_ID || $force_permission) {
		use File::Basename;

		my $plugins_dir = dirname(__FILE__)."/Plugin/$plugin_type/";
		my @plugin_files = glob($plugins_dir."*.pm");
		my @templates_ids = grep { $_ =~ s|$plugins_dir([\w\d]+)\.pm|$1| } @plugin_files;
		return \@templates_ids;
	}
	else {
	    return undef;
	}
}

## Description: checks, if plugin template is available to vendor, according to reseller configs policy
## Parameters:
##   vendor_id
##   template_id
##   plugin_type
## Returns: 1|0
sub is_template_available {
	my $class = shift;
	my %h = (
		vendor_id => undef,
		template_id => undef,
		plugin_type => undef,
		@_
	);
	return 1 if $h{vendor_id} == SW_HSP_ID;
	return 0; ## Always 0 for non-root user. Shared PBA does not support resellers.
}

sub find_plugin {
	return undef; ## for shared PBA always null
}
sub find_plugins {
	return undef; ## for shared PBA always null
}
sub config_data {
	return undef; ## for shared PBA always null
}
sub save {
	return undef; ## for shared PBA always null
}

1;
