## Copyright (C) 2006  SWsoft
## All rights reserved.
## Here should be placed presentation functions for OpenProvider plug-in
package HSPC::Plugin::DM::OpenProvider;

use strict;

use HSPC::PluginToolkit::General qw (
	string
	argparam
	log
	log_debug
	log_warn
);
use HSPC::PluginToolkit::HTMLTemplate qw(
	parse_template
);
use HSPC::PluginToolkit::DM qw(
	extract_shortname_from_domain
	extract_tld_from_domain
	get_contact
	get_domain_extdata
);

use HSPC::MT::Plugin::DM::OpenProvider::Constants;
use Data::Dumper;

#### ------ config forms -------------------------------

## <---- CONFIGURATION START ---->
sub view_config_form {
	my $class = shift;
	my %arg = (
		config => undef,
		@_
	);
	my $conf = $arg{config};

	my $html = parse_template(
		name => 'item_view_text.tmpl',
		data => {
			title => string('op_remote_host'),
			value => $conf->{remote_host},
		}
	);

	$html .= parse_template(
		name => 'item_view_text.tmpl',
		data => {
			title => string('op_login'),
			value => $conf->{login},
		}
	);

	$html .= parse_template(
		name => 'item_view_text.tmpl',
		data => {
			title => string('op_password'),
			value => string('op_hidden_press_edit_btn_to_see_password'),
		}
	);

	return parse_template(
		name => 'table_view.tmpl',
		data => {
			value => $html,
		}
	);
}

sub edit_config_form  {
	my $class = shift;
	my %arg = (
		config => undef,
		@_
	);
	my $conf = $arg{config};
	my $html = '';

	$html .= parse_template(
		name => 'item_edit_text.tmpl',
		data => {
			title => string('op_remote_host'),
			name => 'host',
			value => $conf->{remote_host} || &OP_DEFAULT_API_URL,
			size => 50,
			maxlength => 255,
			not_empty => 1
		}
	);
	$html .= parse_template(
		name => 'item_edit_text.tmpl',
		data => {
			title => string('op_login'),
			name => 'login',
			value => $conf->{login},
			size => 30,
			maxlength => 50,
			not_empty => 1
		}
	);
	$html .= parse_template(
		name => 'item_edit_password.tmpl',
		data => {
			title => string('op_password'),
			name => 'password',
			value => $conf->{password},
			size => 30,
			maxlength => 50,
			not_empty => 1
		}
	);

	return parse_template(
		name => 'table_edit.tmpl',
		data => {
			value => $html,
		}
	);
}

sub collect_config_data {
	my $class = shift;
	my $conf = {};

	$conf->{login} = argparam('login');
	$conf->{password} = argparam('password');
	$conf->{remote_host} = argparam('host');

	return {
		config => $conf
	};
}

sub validate_config_data {
	my $class = shift;
	my %h = (
		config => undef,
		@_
	);
	my $config = $h{config};
	   $config = {} unless ref $config;

	my %mand_fields = (
		remote_host => string('op_remote_host'),
		login => string('op_login'),
		password => string('op_password'),
	);
	my $error_list = [];

	foreach my $field (keys %mand_fields) {
		push @$error_list, {
			field => $field,
			message => string('op_field_mandatory', field => $mand_fields{$field})
		} unless $config->{$field};
	}

	return {
		is_valid => scalar(@$error_list) ? 0 : 1,
		error_list => scalar(@$error_list) ? $error_list : undef
	};
}

## <---- CONFIGURATION END ---->
##
## =============================
##
## <----- CONTACTS START ----->

##
## Draw contact view form
##
sub view_contact_form {
	my $class = shift;
	my %arg = (
		domain			=> undef,
		action			=> undef,
		contact			=> undef,
		contact_type	=> undef,
		contact_extdata	=> undef,
		@_
	);
	my $domain			= $arg{domain};
	my $action			= $arg{action};
	my $contact			= $arg{contact};
	my $contact_type	= $arg{contact_type};
	my $contact_extdata	= $arg{contact_extdata};
	my $prefix;

	$contact_extdata = {} unless $contact_extdata;

	my %flexible_items = %{$class->_get_contact_flexible_items()};
	my $show = $class->_get_visible_flex_contact_items(
		domain => $domain,
		action => $action,
		contact => $contact,
		contact_type => $contact_type,
		contact_extdata => $contact_extdata
	);
	$flexible_items{$_} = 1 for @$show;

	my $hide = [grep {!$flexible_items{$_}} keys %flexible_items];

	my $html;

	$html .= parse_template(
		path => __PACKAGE__,
		name => 'view_contact.tmpl',
		data => {
			is_corporate	=> $contact->{is_corporate},
			org_name		=> $contact->{org_name},

			## info for base contact
			fname	=> $contact->{fname},
			lname	=> $contact->{lname},
			email	=> $contact->{email},
			address	=> $contact->{address},
			city	=> $contact->{city},
			state	=> $contact->{state},
			zip		=> $contact->{zip},
			country	=> $contact->{country},
			phone	=> $contact->{phone},
			fax		=> $contact->{fax},
			hide_fax=> 0,

			## info for contact extended data
			gender		=> $contact_extdata->{gender},
			house_num	=> $contact_extdata->{house_num},
			house_suff	=> $contact_extdata->{house_suff},
			mobile		=> $contact_extdata->{mobile},
			birth_city	=> $contact_extdata->{birth_city},
			birth_date	=> $contact_extdata->{birth_date},
			company_type=> $contact_extdata->{company_type},
			coc_no		=> $contact_extdata->{coc_no},
			coc_region	=> $contact_extdata->{coc_region},
			coc_reg_date=> $contact_extdata->{coc_reg_date},
			fiscal_no	=> $contact_extdata->{fiscal_no},
			tax_ex_number=> $contact_extdata->{tax_ex_number},

			hide => $hide,
		}
	);
	
	return parse_template(
		name => 'table_view.tmpl',
		data => {
			value => $html,
		}
	);
}

##
## Draw contact edit form
##
sub edit_contact_form {
	my $class = shift;
	my %arg = (
		domain			=> undef,
		action			=> undef,
		contact			=> undef,
		contact_type	=> undef,
		contact_extdata	=> undef,
		error_list		=> undef,
		@_
	);
	my $domain			= $arg{domain};
	my $action			= $arg{action};
	my $contact			= $arg{contact};
	my $contact_type	= $arg{contact_type};
	my $contact_extdata	= $arg{contact_extdata};
	my $error_list		= $arg{error_list};
	my $prefix;

	$contact_extdata = {} unless $contact_extdata;

	my %flexible_items = %{$class->_get_contact_flexible_items()};
	my $show = $class->_get_visible_flex_contact_items(
		domain => $domain,
		action => $action,
		contact => $contact,
		contact_type => $contact_type,
		contact_extdata => $contact_extdata
	);
	$flexible_items{$_} = 1 for @$show;

	my $hide = [grep {!$flexible_items{$_}} keys %flexible_items];

	my $prefix = $class->_create_unique_name_prefix(
		tokens => ['item', $domain, $contact_type]
	);
	
	my $house_num = $contact_extdata->{house_num}
		|| ((ref $contact->{account_ext_attr} eq 'HASH') ? $contact->{account_ext_attr}->{custom_ext_housenumber} : undef)
		|| $contact->{house_num};
	my $house_suff = $contact_extdata->{house_suff}
		|| ((ref $contact->{account_ext_attr} eq 'HASH') ? $contact->{account_ext_attr}->{custom_ext_suffix} : undef)
		|| $contact->{house_suff};
	
	my $html;

	$html .= qq~<table cellpadding=1 cellspacing=0 width=100% bgcolor="#cccccc"><tr><td>~;
	$html .= qq~<table cellspacing=0 cellpadding=4 width=100% border=0 bgcolorqq="#cccccc">~;
	$html .= parse_template(
		path => __PACKAGE__,
		name => 'edit_contact.tmpl',
		data => {
			form_prefix => $prefix,

			## info for base contact
			is_corporate	=> $contact->{is_corporate},
			org_name		=> $contact->{org_name},

			fname	=> $contact->{fname},
			lname	=> $contact->{lname},
			email	=> $contact->{email},
			address	=> $contact->{address},
			city	=> $contact->{city},
			state	=> $contact->{state},
			zip		=> $contact->{zip},
			country	=> $contact->{country},
			phone	=> $contact->{phone},
			fax		=> $contact->{fax},
			hide_fax=> 0,

			## info for contact extended data
			gender		=> $contact_extdata->{gender},
			house_num	=> $house_num,
			house_suff	=> $house_suff,
			mobile		=> $contact_extdata->{mobile},
			birth_city	=> $contact_extdata->{birth_city},
			birth_date	=> $contact_extdata->{birth_date},
			company_type=> $contact_extdata->{company_type},
			coc_no		=> $contact_extdata->{coc_no},
			coc_region	=> $contact_extdata->{coc_region},
			coc_reg_date=> $contact_extdata->{coc_reg_date},
			fiscal_no	=> $contact_extdata->{fiscal_no},
			tax_ex_number=> $contact_extdata->{tax_ex_number},

			require_house_num => 1,
			hide => $hide,
		}
	);
	$html .= qq~</table>~;
	$html .= qq~</td></tr></table>~;

	return $html;
}

##
## Collect contact forms data
##
sub collect_contacts_data {
	my $class = shift;
	my %h = (
		domain => undef,
		action => undef,
		@_
	);

	my $type_id;
	my $contact;
	my $contacts = {};
	my $contacts_extdata = {};
	my $prefix;

	foreach $type_id (qw(admin owner technical)) {
		$prefix = $class->_create_unique_name_prefix(
			tokens => ['item', $h{domain}, $type_id]
		);
		$contact = {
			fname		=> ''.argparam("$prefix\_fname"),
			lname		=> ''.argparam("$prefix\_lname"),
			address		=> ''.argparam("$prefix\_address"),
			city		=> ''.argparam("$prefix\_city"),
			state		=> ''.argparam("$prefix\_state"),
			zip			=> ''.argparam("$prefix\_zip"),
			country		=> ''.argparam("$prefix\_country"),
			email		=> ''.argparam("$prefix\_email"),
			phone		=> join ('|',
				(
					''.argparam("$prefix\_phone_country_code"),
					''.argparam("$prefix\_phone_area_code"),
					''.argparam("$prefix\_phone_number"),
					''.argparam("$prefix\_phone_extension"),
				)
			),
			fax			=> join ('|',
				(
					''.argparam("$prefix\_fax_country_code"),
					''.argparam("$prefix\_fax_area_code"),
					''.argparam("$prefix\_fax_number"),
					''.argparam("$prefix\_fax_extension"),
				)
			),
			is_corporate=> ''.argparam("$prefix\_is_corporate"),
			org_name	=> ''.argparam("$prefix\_org_name")
		};
		$contacts->{$type_id} = $contact;

		##
		## Collect contact extended data
		##
		my $contact_extdata = $class->collect_contact_extdata(
			domain => $h{domain},
			action => $h{action},
			contact_type => $type_id
		)->{contact_extdata};

		$contacts_extdata->{$type_id} = $contact_extdata;
	}

	return {
		contacts => $contacts,
		contacts_extdata => $contacts_extdata
	};
}

##
## Draw contact extended data view form
##
sub view_contact_extdata_form {
	my $class = shift;
	my %arg = (
		domain			=> undef,
		action			=> undef,
		contact			=> undef,
		contact_type	=> undef,
		contact_extdata	=> undef,
		@_
	);
	my $domain			= $arg{domain};
	my $action			= $arg{action};
	my $contact			= $arg{contact};
	my $contact_type	= $arg{contact_type};
	my $contact_extdata	= $arg{contact_extdata};
	my $prefix;

	$prefix = $class->_create_unique_name_prefix(
		tokens => ['item', $arg{domain}, $contact_type]
	);

	$contact_extdata = {} unless $contact_extdata;

	my %flexible_items = %{$class->_get_contact_flexible_items()};
	my $show = $class->_get_visible_flex_contact_items(
		domain => $domain,
		action => $action,
		contact => $contact,
		contact_type => $contact_type,
		contact_extdata => $contact_extdata
	);
	$flexible_items{$_} = 1 for @$show;

	my $hide = [grep {!$flexible_items{$_}} keys %flexible_items];

	my $html;

	$html .= qq~<table cellspacing=1 cellpadding=4 width=100% border=0 bgcolor="#cccccc">~;
	$html .= parse_template(
		path => __PACKAGE__,
		name => 'view_contact_extdata.tmpl',
		data => {
			## info for contact extended data
			gender		=> $contact_extdata->{gender},
			house_num	=> $contact_extdata->{house_num},
			house_suff	=> $contact_extdata->{house_suff},
			mobile		=> $contact_extdata->{mobile},
			birth_city	=> $contact_extdata->{birth_city},
			birth_date	=> $contact_extdata->{birth_date},
			company_type=> $contact_extdata->{company_type},
			coc_no		=> $contact_extdata->{coc_no},
			coc_region	=> $contact_extdata->{coc_region},
			coc_reg_date=> $contact_extdata->{coc_reg_date},
			fiscal_no	=> $contact_extdata->{fiscal_no},
			tax_ex_number=> $contact_extdata->{tax_ex_number},

			hide => $hide,
		}
	);
	$html .= qq~</table>~;
	
	return $html;
}

sub collect_contact_extdata {
	my $class = shift;
	my %h = (
		domain => undef,
		action => undef,
		contact_type => undef,
		@_
	);
	my $domain = $h{domain};
	my $action = $h{action};
	my $contact_type = $h{contact_type};
	my $old_contact = get_contact(
		domain => $domain,
		contact_type => $contact_type
	);
	my $prefix = $class->_create_unique_name_prefix(
		tokens => ['item', $domain, $contact_type]
	);
	my $extdata = $old_contact->{contacts_extdata}->{$contact_type}
		? $old_contact->{contacts_extdata}->{$contact_type}
		: {};


	my %flexible_items = %{$class->_get_contact_flexible_items()};
	my $show = $class->_get_visible_flex_contact_items(
		domain => $domain,
		action => $action,
		contact_type => $contact_type,
	);
	$flexible_items{$_} = 1 for @$show;

	$extdata->{gender} = argparam("$prefix\_gender");

	$extdata->{house_num} = argparam("$prefix\_house_num");
	$extdata->{house_suff} = argparam("$prefix\_house_suff");

	$extdata->{mobile} = join ('|',
		(
			''.argparam("$prefix\_mobile_country_code"),
			''.argparam("$prefix\_mobile_area_code"),
			''.argparam("$prefix\_mobile_number"),
			''.argparam("$prefix\_mobile_extension"),
		)
	);

	$extdata->{birth_city} = argparam("$prefix\_birth_city")
		if $flexible_items{birth_city};

	$extdata->{birth_date} = join ('|',
		(
			''.argparam("$prefix\_birth_date_day"),
			''.argparam("$prefix\_birth_date_month"),
			''.argparam("$prefix\_birth_date_year"),
		)
	) if $flexible_items{birth_date};

	$extdata->{company_type}= argparam("$prefix\_company_type")
		if $flexible_items{company_type};

	$extdata->{coc_no} = argparam("$prefix\_coc_no")
		if $flexible_items{coc_no};

	$extdata->{coc_region} = argparam("$prefix\_coc_region")
		if $flexible_items{coc_region};

	$extdata->{coc_reg_date} = join ('|',
		(
			''.argparam("$prefix\_coc_reg_date_day"),
			''.argparam("$prefix\_coc_reg_date_month"),
			''.argparam("$prefix\_coc_reg_date_year"),
		)
	) if $flexible_items{coc_reg_date};

	$extdata->{fiscal_no} = argparam("$prefix\_fiscal_no")
		if $flexible_items{fiscal_no};

	$extdata->{tax_ex_number} = argparam("$prefix\_tax_ex_number");

	return {
		contact_extdata => $extdata
	};
}

##
## Draw domain extended data edit form
##
sub edit_domain_extdata_form {
	my $class = shift;
	my %arg = (
		domain			=> undef,
		action			=> undef,
		domain_extdata	=> undef,
		error_list		=> undef,
		@_
	);
	my $domain			= $arg{domain};
	my $action			= $arg{action};
	my $error_list		= $arg{error_list};

	my $domain_extdata	= $arg{domain_extdata}
		? $arg{domain_extdata}
		: {};
	my $domain_shortname= extract_shortname_from_domain($domain);
	my $domain_tld		= extract_tld_from_domain($domain);

	my $prefix = $class->_create_unique_name_prefix(
		tokens => ['item', $domain, 'extdata']
	);
	my $html;

	if(lc($action) eq 'transfer' &&
		scalar(grep {$_ eq $domain_tld} &OP_TRANS_TLDS)){
		$html .= parse_template(
			name => 'item_edit_text.tmpl',
			data => {
				title => string('dm_plugin_openprovider_transfer_password'),
				name => "$prefix\_auth_code",
				value => $domain_extdata->{auth_code},
				maxlength => 30,
				not_empty => 1
			}
		);
	}

	if($action eq 'register' && $domain =~ /\.de$/i){
		my $prefill_value = ($domain_extdata->{use_op_handle} ? 1 : 0)
			if(defined($domain_extdata->{use_op_handle}));
		$html .= parse_template(
			name => 'item_edit_check.tmpl',
			data => {
				name => $prefix.'_use_op_handle',
				title => string('dm_plugin_openprovider_use_op_handle'),
				value => $prefill_value
			}
		);
	}

	if  ( $html ) {
		$html = parse_template(
			name => 'table_edit.tmpl',
			data => {
				value => $html
			}
		);
	}

	return $html;
}

##
## Draw domain extended data view form
##
sub view_domain_extdata_form {
	my $class = shift;
	my %arg = (
		domain			=> undef,
		action			=> undef,
		domain_extdata	=> undef,
		error_list		=> undef,
		@_
	);
	my $domain			= $arg{domain};
	my $action			= $arg{action};
	my $error_list		= $arg{error_list};

	my $domain_extdata	= $arg{domain_extdata}
		? $arg{domain_extdata}
		: {};
	my $domain_shortname= extract_shortname_from_domain($domain);
	my $domain_tld		= extract_tld_from_domain($domain);

	my $html;

	if (lc($action) eq 'transfer'
		&& scalar(grep {$_ eq $domain_tld} &OP_TRANS_TLDS))
	{
		$html .= parse_template(
			name => 'item_view_text.tmpl',
			data => {
				title => string('dm_plugin_openprovider_transfer_password'),
				value => $domain_extdata->{auth_code}
			}
		);
	}

	if (lc($action) eq 'register' && $domain =~ /\.de$/) {
		$html .= parse_template(
			name => 'item_view_check.tmpl',
			data => {
				title => string('dm_plugin_openprovider_use_op_handle'),
				value => $domain_extdata->{use_op_handle} ? 1 : 0
			}
		);
	}

	if  ( $html ) {
		$html = parse_template(
			name => 'table_view.tmpl',
			data => {
				value => $html
			}
		);
	}

	return $html;
}

##
## Collect domain extended data form (from CGI params)
##
sub collect_domain_extdata {
	my $class = shift;
	my %h = (
		domain => undef,
		action => undef,
		@_
	);
	my $domain = $h{domain};
	my $action = $h{action};
	my $old_extdata = get_domain_extdata(domain => $domain);
	my $prefix = $class->_create_unique_name_prefix(
		tokens => ['item', $domain, 'extdata']
	);
	my $extdata = $old_extdata->{domain_extdata}
		? $old_extdata->{domain_extdata}
		: {};

	$extdata->{auth_code} = argparam("$prefix\_auth_code");
	$extdata->{use_op_handle} = argparam("$prefix\_use_op_handle");

	return {
		domain_extdata => $extdata
	};
}

##
## ----------------- Private methods ------------------------------------------
##

sub _create_unique_name_prefix {
	my $class = shift;
	my %h = (
		tokens => [],
		@_
	);
	return undef unless scalar @{$h{tokens}};
	my $unique = join('_', @{$h{tokens}});
	$unique =~ s#[^\w\d\_]#\_#g;
	return $unique;
}

sub _get_visible_flex_contact_items {
	my $class = shift;
	my %h = (
		domain => undef,
		action => undef,
		contact_type => undef,
		@_
	);
	my $domain			= $h{domain};
	my $action			= $h{action};
	my $contact_type	= $h{contact_type};

	my $domain_shortname= extract_shortname_from_domain($domain);
	my $domain_tld		= extract_tld_from_domain($domain);

	my $show = [];

	##
	## Prepare contact info visible options
	##
	if ($domain_tld eq 'it') {
		push @$show, 'company_type';
		push @$show, 'coc_no';
		push @$show, 'coc_region';
		push @$show, 'coc_reg_date';
		push @$show, 'birth_city';
		push @$show, 'birth_date';
		push @$show, 'fiscal_no';
	}
	if ($action eq 'transfer' && scalar(grep {$_ eq $domain_tld} &OP_TRANS_TLDS)) {
		push @$show, 'auth_code';
	}

	return $show;
}

sub _get_contact_flexible_items {
	my $class = shift;
	return {
		birth_city		=> 0,
		birth_date		=> 0,
		company_type	=> 0,
		coc_no			=> 0,
		coc_region		=> 0,
		coc_reg_date	=> 0,
		fiscal_no		=> 0,
	}
}

sub get_help_page {
	my $class = shift;
	my %h = (
		action => undef,
		language => undef,
		config => undef,
		@_
	);
	my $action = $h{action};
	my $language = $h{language};

	my $help_page;
	if($action =~ /^(about|new|view|edit)$/){
		my $tmpl_name = "openprovider_$action.html";
		$help_page = parse_template(
			path => __PACKAGE__ . '::help::' . uc($language),
			name => $tmpl_name
		);
	}

	return $help_page;
}

1;
