package HSPC::Plugin::PM::BT_Clieop03;

use strict;

use HSPC::PluginToolkit::General qw(string argparam get_help_url);
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use Encode qw(from_to);

sub view_form {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		name        => undef,
		type_info   => undef,
		number_link => undef,
		@_
	);

	my %th = (
		type_txt    => string($h{type_info}->{type_title_id}),
		holder_name => $h{public_data}->{holder_name},
		number_link => $h{number_link},
	);

	## Fill private data
	if ($h{secure_data}) {
		$th{only_public}    = 0;
		$th{folder_title}   = string('bapm_clieop03_private_data_uc');
		$th{account_number} = $h{secure_data}->{account_number}
	} else {
		$th{only_public}    = 1;
		$th{folder_title}   = string('bapm_clieop03_public_data_uc');
		$th{account_number} = $h{name};
	}

	return {
		content => parse_template(path => __PACKAGE__, name => 'pm_view_form.tmpl', data => \%th),
	};
}

sub edit_form {
	my $class = shift;
	my %h = (
		public_data   => undef,
		secure_data   => undef,
		name          => undef,
		allowed_types => undef,
		@_
	);

	my %th = (
		holder_name  => $h{public_data}->{holder_name},
		type_txt     => $h{allowed_types}->[1]->[1],
		agree        => $h{public_data}->{agree},
		no_agreement => $h{public_data}->{no_agreement},
		date         => $h{public_data}->{date},
	);

	## Fill private data
	if ($h{secure_data}) {
		$th{only_public}    = 0;
		$th{folder_title}   = string('bapm_clieop03_private_data_uc');
		$th{account_number} = $h{secure_data}->{account_number}
	} else {
		$th{only_public}    = 1;
		$th{folder_title}   = string('bapm_clieop03_public_data_uc');
		$th{account_number} = $h{name};
	}

	return {
		content => parse_template(path => __PACKAGE__, name => 'pm_edit_form.tmpl', data => \%th),
		js_onsubmit => undef
	};
}

sub add_form {
	my $class = shift;
	my %h = (
		public_data  => undef,
		secure_data  => undef,
		account_data => undef,
		@_
	);

	my %th = (
		account_number => $h{secure_data} ? $h{secure_data}->{account_number} : undef,
		agreement_text => string('bapm_clieop03_agreement',
			link_open => '<a href="#" onclick="'.$class->__help_js.'">',
			link_close => '</a>'
		)
	);

	## Prepopulate holder name
	if ($h{account_data}) {
		if ($h{account_data}->{is_corporate}) {
			$th{holder_name} = $h{account_data}->{billing_fname} .
				' ' . $h{account_data}->{billing_lname};
		} else {
			$th{holder_name} = $h{account_data}->{admin_fname} .
				' ' . $h{account_data}->{admin_lname};
		}
	} elsif ($h{public_data}) {
		$th{holder_name} = $h{public_data}->{holder_name};
	}

	return {
		content => parse_template(path => __PACKAGE__, name => 'pm_add_form.tmpl', data => \%th),
		js_onsubmit => ''
	};
}

sub collect_data {
	my $class = shift;
	my %h = (
		secure_data  => undef,
		public_data  => undef,
		account_data => undef,
		@_
	);

	my %data;
	if ($h{secure_data} && $h{secure_data}->{account_number}) {
		## Update payment method number
		$data{account_number} = $h{secure_data}->{account_number};
	} else {
		## Set a new one
		$data{account_number} = &argparam('account_number');
	}

	$data{agree} = &argparam('agree');
	$data{no_agreement} = &argparam('no_agreement');
	$data{date} = &argparam('date');

	my $holder_name = &argparam('holder_name');
	from_to($holder_name, 'utf8', 'latin1');
	
	if (length($holder_name) > 35) {
		$holder_name = substr($holder_name,0,35);
		from_to($holder_name, 'latin1', 'utf8');
		$data{holder_name} = $holder_name;
	}
	else { $data{holder_name} = $holder_name; }
	
	return \%data;
}

sub __help_js {
	my $class = shift;

	my $help_url = get_help_url(
		action        => 'view_agreement',
		customer_view => 1,
	);
	
	my $js = qq{
		window.open('$help_url','popup','width=500,height=720,location=0,toolbar=0,scrollbars=1,resizable=1');
		return false;
	};

	return $js;
}

sub get_help_page {
	my $class = shift;
	my %h = (
		action        => undef,
		template_id   => undef,
		language      => undef,
		customer_view => undef,
		@_
	);
	
	if ($h{action} eq 'view_agreement') {
		return parse_template(path => __PACKAGE__, name => 'agreement.tmpl', data => \%h);
	} elsif ($h{action}) {
		my $path = __PACKAGE__ . '::' . uc($h{language});
		return parse_template(path => $path, name => "pm_help_$h{action}.html", data => \%h);
	}
}

1;
