package HSPC::Plugin::PM::OP_CCard;

use strict;

use HSPC::PluginToolkit::General qw(
	string argparam geo_get_countries geo_get_states
	geo_get_country_name geo_get_state_name last_month_day
	split_date_string datetime_gmt_now convert_quotes
);
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(throw_exception get_help_url);

sub view_form {
	my $class = shift;
	my %h = (
		secure_data      => undef,
		public_data      => undef,
		is_admin         => undef,
		type             => undef,
		expire_date      => undef,
		name             => undef,
		type_info        => undef,
		keep_secure_code => undef,
		number_link      => undef,
		@_
	);

	my %th = (
		type_txt         => string($h{type_info}->{type_title_id}),
		type_img         => '<img_'.$h{type_info}->{image_id}.'>',
		keep_secure_code => $h{keep_secure_code},
		number_link      => $h{number_link},
	);
	## Calculate the expiration date options
	my %expire_date = split_date_string($h{expire_date});
	$th{expire_date} = "$expire_date{month}/$expire_date{year}";

	my %valid_from  = split_date_string($h{public_data}->{valid_from});
	$th{valid_from} = "$valid_from{month}/$valid_from{year}" 
			if $valid_from{year};

	## Fill private data
	if ($h{is_admin}) {
		$th{only_public}   = 0;
		$th{folder_title}  = string('private_card_data_uc');
		$th{card_number}   = $h{secure_data}->{card_number};
		$th{cvv}           = $h{secure_data}->{cvv};
		$th{issue_number}  = $h{secure_data}->{issue_number};
		
		$th{holder_name}   = $h{secure_data}->{holder_name} . ' ' . $h{secure_data}->{holder_name2};

		$th{address1}       = $h{secure_data}->{address1};
		$th{address2}       = $h{secure_data}->{address2};
		$th{city}           = $h{secure_data}->{city};
		$th{state_uscanada} = geo_get_state_name(
			$h{secure_data}->{country}, $h{secure_data}->{state_uscanada})
		;
		$th{state}          = $h{secure_data}->{state};
		$th{country}        = geo_get_country_name($h{secure_data}->{country});
		$th{zip}            = $h{secure_data}->{zip};

		$th{phone}          = $h{secure_data}->{phone};
	} else {
		$th{only_public}   = 1;
		$th{card_number}   = $h{name};
		$th{cvv}           = $h{secure_data}->{cvv};
		$th{folder_title}  = string('cc_pub_data_data_uc');
	}

	$th{has_issue} = grep { $h{type} eq $_ } ('W', 'S', 'T', 'L');
	## Switch, Solo, Delta, Electron

	return {
		content => parse_template(path => __PACKAGE__, name => 'view_form.tmpl', data => \%th),
	};
}

sub edit_form {
	my $class = shift;
	my %h = (
		secure_data      => undef,
		public_data      => undef,
		is_admin         => undef,
		name             => undef,
		type             => undef,
		expire_date      => undef,
		allowed_types    => undef,
		keep_secure_code => undef,
		@_
	);

	my %th = (
		allowed_types    => $h{allowed_types},
		type             => $h{type},
		keep_secure_code => $h{keep_secure_code},
	);

	## Fill private data
	if ($h{is_admin}) {
		$th{only_public}   = 0;
		$th{folder_title}  = string('private_card_data_uc');
		$th{card_number}   = $h{secure_data}->{card_number};
		$th{cvv}           = $h{secure_data}->{cvv};
		$th{cvv_disable}   = $th{cvv} ? 0 : 1;
		$th{issue_number}  = $h{secure_data}->{issue_number};

		$th{fname}         = convert_quotes($h{secure_data}->{holder_name});
		$th{lname}         = convert_quotes($h{secure_data}->{holder_name2});

		$th{address1}       = convert_quotes($h{secure_data}->{address1});
		$th{address2}       = convert_quotes($h{secure_data}->{address2});
		$th{city}           = convert_quotes($h{secure_data}->{city});
		$th{state_uscanada} = convert_quotes($h{secure_data}->{state_uscanada});
		$th{state}          = $h{secure_data}->{state};
		$th{country}        = $h{secure_data}->{country};
		$th{zip}            = convert_quotes($h{secure_data}->{zip});

		$th{phone_country_code} = $h{secure_data}->{phone_src}->{country_code};
		$th{phone_area_code}    = $h{secure_data}->{phone_src}->{area_code};
		$th{phone_number}       = $h{secure_data}->{phone_src}->{number};
		$th{phone_extension}    = $h{secure_data}->{phone_src}->{extension};

		$th{states_list}    = geo_get_states();
		$th{countries_list} = geo_get_countries();
		unshift @{$th{countries_list}}, ['', string('select_dd')];
	} else {
		$th{only_public}   = 1;
		$th{card_number}   = $h{name};
		$th{cvv}           = $h{secure_data}->{cvv};
		$th{folder_title}  = string('cc_pub_data_data_uc');
	}

	## Should an issue number be requested
	$th{has_issue} = scalar grep {
		$_->[0] eq 'W' || $_->[0] eq 'S' || $_->[0] eq 'T' || $_->[0] eq 'L'
	} @{$h{allowed_types}}; ## Switch, Solo, Delta, Electron

	$th{has_issue} ||= grep { $h{type} eq $_ } ('W', 'S', 'T', 'L');
	## Switch, Solo, Delta, Electron

	## Calculate the expiration date options
	my %expire_date = split_date_string($h{expire_date});
	my ($year_now) = (datetime_gmt_now() =~ /(\d\d\d\d)/);

	$th{expire_month_opt} = [map { [$_, ($_ < 10 ? "0$_" : $_)] } (1 .. 12)];
	$th{expire_month} = int($expire_date{month}) || 1;
	$th{expire_year_opt} = [map { [$_, $_] } ($year_now .. ($year_now + 10))];
	$th{expire_year} = $expire_date{year} || $year_now + 1;
	
	## Calculate the valid from date options
	my %valid_from_date = split_date_string($h{public_data}->{valid_from});

	$th{valid_from_month_opt} = [['','-'], map { [$_, ($_ < 10 ? "0$_" : $_)] } (1 .. 12)];
	$th{valid_from_month}     = int($valid_from_date{month});
	$th{valid_from_year_opt}  = [['','-'], map { [$_, $_] } ($year_now - 10 .. $year_now)];
	$th{valid_from_year}      = $valid_from_date{year};

	my $js = __help_js();
	$th{help_link} = qq|<a href="#" onclick="$js">| . string('what_is_cvv_question') . '</a>';

	return {
		content => parse_template(path => __PACKAGE__, name => 'edit_form.tmpl', data => \%th),
		js_onsubmit => undef
	};
}

sub add_form {
	my $class = shift;
	my %h = (
		secure_data      => undef,
		public_data      => undef,
		account_data     => undef,
		name             => undef,
		type             => undef,
		expire_date      => undef,
		allowed_types    => undef,
		keep_secure_code => undef,
		@_
	);

	my %th = (
		allowed_types    => $h{allowed_types},
		type             => $h{type},
		keep_secure_code => $h{keep_secure_code},
		card_number      => $h{secure_data} ? $h{secure_data}->{card_number} : undef,
		cvv              => $h{secure_data} ? $h{secure_data}->{cvv} : undef,
		issue_number     => $h{secure_data} ? $h{secure_data}->{issue_number} : undef,
	);

	## Should an issue number be requested
	$th{has_issue} = scalar grep {
		$_->[0] eq 'W' || $_->[0] eq 'S' || $_->[0] eq 'T' || $_->[0] eq 'L'
	} @{$h{allowed_types}}; ## Switch, Solo, Delta, Electron

	## Prepopulate holder name
	if ($h{account_data}) {
		if ($h{account_data}->{is_corporate}) {
			$th{fname} = convert_quotes($h{account_data}->{billing_fname});
			$th{lname} = convert_quotes($h{account_data}->{billing_lname});
		} else {
			$th{fname} = convert_quotes($h{account_data}->{admin_fname});
			$th{lname} = convert_quotes($h{account_data}->{admin_lname});
		}
	} elsif ($h{secure_data}) {
		$th{fname} = convert_quotes($h{secure_data}->{holder_name});
		$th{lname} = convert_quotes($h{secure_data}->{holder_name2});
	}

	## Calculate the expiration date options
	my %expire_date = split_date_string($h{expire_date});
	my ($year_now) = (datetime_gmt_now() =~ /(\d\d\d\d)/);

	$th{expire_month_opt} = [map { [$_, ($_ < 10 ? "0$_" : $_)] } (1 .. 12)];
	$th{expire_month} = int($expire_date{month}) || 1;
	$th{expire_year_opt} = [map { [$_, $_] } ($year_now .. ($year_now + 10))];
	$th{expire_year} = $expire_date{year} || $year_now + 1;

	## Calculate the valid from date options
	my %valid_from_date = split_date_string($h{public_data}->{valid_from_date});

	$th{valid_from_month_opt} = [['','-'], map { [$_, ($_ < 10 ? "0$_" : $_)] } (1 .. 12)];
	$th{valid_from_month}     = int($valid_from_date{month});
	$th{valid_from_year_opt}  = [['','-'], map { [$_, $_] } (($year_now - 10) .. $year_now)];
	$th{valid_from_year}      = $valid_from_date{year};

	my $js = __help_js();
	$th{help_link} = qq|<a href="#" onclick="$js">| . string('what_is_cvv_question') . '</a>';

	$th{address1} = convert_quotes( $h{secure_data} ?
		$h{secure_data}->{address1} : $h{account_data} ?
			$h{account_data}->{address_address1} : undef );
	$th{address2} = convert_quotes($h{secure_data} ?
		$h{secure_data}->{address2} : $h{account_data} ?
			$h{account_data}->{address_address2} : undef);
	$th{city} = convert_quotes($h{secure_data} ?
		$h{secure_data}->{city} : $h{account_data} ?
			$h{account_data}->{address_city} : undef);
	$th{state_uscanada} = convert_quotes($h{secure_data} ?
		$h{secure_data}->{state_uscanada} : $h{account_data} ?
			$h{account_data}->{address_state_uscanada} : undef);
	$th{states_list} = geo_get_states();
	$th{state} = convert_quotes($h{secure_data} ?
		$h{secure_data}->{state} : $h{account_data} ?
			$h{account_data}->{address_state} : undef);
	$th{country} = convert_quotes($h{secure_data} ?
		$h{secure_data}->{country} : $h{account_data} ?
			$h{account_data}->{address_country} : undef);
	$th{countries_list} = geo_get_countries();
	unshift @{$th{countries_list}}, ['', string('select_dd')];
	
	$th{zip} = $h{secure_data} ?
		$h{secure_data}->{zip} : $h{account_data} ?
			$h{account_data}->{address_zip} : undef;

	$th{phone_country_code} = $h{secure_data} ?
		$h{secure_data}->{phone_src}->{country_code} : $h{account_data} ?
			$h{account_data}->{address_phone}->{country_code} : undef;
	$th{phone_area_code} = $h{secure_data} ?
		$h{secure_data}->{phone_src}->{area_code} : $h{account_data} ?
			$h{account_data}->{address_phone}->{area_code} : undef;
	$th{phone_number} = $h{secure_data} ?
		$h{secure_data}->{phone_src}->{number} : $h{account_data} ?
			$h{account_data}->{address_phone}->{number} : undef;
	$th{phone_extension} = $h{secure_data} ?
		$h{secure_data}->{phone_src}->{extension} : $h{account_data} ?
			$h{account_data}->{address_phone}->{extension} : undef;

	if (!$h{secure_data}) {
		$th{account_id} = $h{account_data} ? $h{account_data}->{id} : undef;
	}

	return {
		content => parse_template(path => __PACKAGE__, name => 'add_form.tmpl', data => \%th),
		js_onsubmit => 'if (!CheckCard(form)) return false;'
	};
}

sub collect_data {
	my $class = shift;
	my %h = (
		secure_data      => undef,
		public_data      => undef,
		account_data     => undef,
		only_public      => undef,
		@_
	);

	my %data;
	%data = %{$h{secure_data}} if (ref($h{secure_data}) eq 'HASH');

	my $type = &argparam('type');
	$data{type} = $type if ( $type && lc($type) ne 'none' );

	$data{expire_date} = last_month_day(sprintf(
		"%04d-%02d-01 00:00:00",
		&argparam('exp_year'),
		&argparam('exp_month')
	));
	
	$data{valid_from}   = last_month_day(sprintf(
		"%04d-%02d-01 00:00:00",
		&argparam('valid_from_year'),
		&argparam('valid_from_month')
	)) if &argparam('valid_from_year') && &argparam('valid_from_month');

	if ($h{keep_secure_code}) {
		if ( &argparam('no_cvc1_or_cvc2') ) {
			$data{cvv} = '';
		} else {
			$data{cvv} = &argparam('card_code');
			unless ( $data{cvv} ) {
				throw_exception(
					type    => 'parameter',
					message => string('specify_or_disable_cvv')
				);
			}
		}
	}


	unless ( $h{only_public} ) {

		if ($h{secure_data} && $h{secure_data}->{card_number}) {
			## Update payment method number
			$data{card_number} = $h{secure_data}->{card_number};
		} else {
			## Set a new one
			$data{card_number} = &argparam('num');

			## Remove non-numeric characters
			$data{card_number} =~ tr/0-9//cd;

			unless ( $data{card_number} ) {
				throw_exception(
					type    => 'parameter',
					message => string('wrong_cc_number_format', num => '')
				);
			}
		}

		$data{holder_name} = &argparam('fname');
		$data{holder_name2} = &argparam('lname');
		$data{issue_number} = &argparam('issue_code');

		if (&argparam('new_address')) {
			$data{address1} = &argparam('address1');
			$data{address2} = &argparam('address2');
			$data{city} = &argparam('city');

			$data{state_uscanada} = &argparam('state_uscanada');
			$data{state} = &argparam('state');

			$data{country} = &argparam('country');
			$data{zip} = &argparam('zip');
			$data{phone_src} = {
				country_code => &argparam('phone_country_code'),
				area_code => &argparam('phone_area_code'),
				number => &argparam('phone_number'),
				extension => &argparam('phone_extension'),
			};
		} else {
			$data{address1} = $h{account_data}->{address_address1};
			$data{address2} = $h{account_data}->{address_address2};
			$data{city} = $h{account_data}->{address_city};

			$data{state_uscanada} = $h{account_data}->{address_state_uscanada};
			$data{state} = $h{account_data}->{address_state};

			$data{country} = $h{account_data}->{address_country};
			$data{zip} = $h{account_data}->{address_zip};
			$data{phone_src} = {
				country_code => $h{account_data}->{address_phone}->{country_code},
				area_code => $h{account_data}->{address_phone}->{area_code},
				number => $h{account_data}->{address_phone}->{number},
				extension => $h{account_data}->{address_phone}->{extension},
			};
		}

		$data{phone} = '+'. $data{phone_src}->{country_code};
		$data{phone} .= ' ('. $data{phone_src}->{area_code} . ')'
			if ($data{phone_src}->{area_code});
		$data{phone} .= ' '. $data{phone_src}->{number};
		$data{phone} .= ' '. $data{phone_src}->{extension}
			if ($data{phone_src}->{extension});
	}

	return \%data;
}

sub __help_js {
	my $class = shift;

	my $help_url = get_help_url(
		action        => 'cvv',
		customer_view => 1,
	);
	my $js = qq{
		window.open('$help_url','popup','width=500,height=720,location=0,toolbar=0,scrollbars=1,resizable=1');
		return false;
	};

	return $js;
}


sub get_help_page {
	my $class = shift;
	my %h = (
		action        => undef,
		template_id   => undef,
		language      => undef,
		customer_view => undef,
		@_
	);

	if ($h{action} eq 'cvv') {
		return parse_template(path => __PACKAGE__, name => 'cvv_expl.tmpl', data => \%h);
	} else {
		my $path = __PACKAGE__ . '::' . uc($h{language});
		return parse_template(path => $path, name => "help_$h{action}.html", data => \%h);
	}
}

1;
