package HSPC::Plugin::PP::BT_Clieop03;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(argparam throw_exception string get_help_url);

use HSPC::MT::Plugin::PM::BT_Clieop03;

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config = $h{config};
	my %data = (
		order_party_name   => $config->{order_party_name},
		account_number     => $config->{account_number},
		sender_id          => $config->{sender_id},
		fixed_desc         => $config->{fixed_desc},
		item_desc          => $config->{item_desc} || '%ACC_ID% %INV_NUM%',
		test_code          => $config->{test_code},
	);

	$data{agreement_text} = parse_template(
		path => 'HSPC::Plugin::PM::BT_Clieop03',
		name => 'agreement.tmpl',
		data => {}
	);

	return parse_template(
		path => __PACKAGE__,
		name => 'bt_clieop03_view.tmpl',
		data => \%data,
	);
}

sub edit_form{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);

	my $config = $h{config};
	my %data = (
		order_party_name   => argparam('order_party_name') || $config->{order_party_name},
		account_number     => argparam('account_number') || $config->{account_number},
		sender_id          => argparam('sender_id') || $config->{sender_id},
		fixed_desc         => argparam('fixed_desc') || $config->{fixed_desc},
		item_desc          => argparam('item_desc') || $config->{item_desc} || '%ACC_ID% %INV_NUM%',
		test_code          => argparam('test_code') || $config->{test_code} ? 1 : 0,
	);

	return parse_template(
		path => __PACKAGE__,
		name => 'bt_clieop03_edit.tmpl',
		data => \%data,
	);
}

sub collect_data{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config = $h{config};
	
	## read data from config to keep it not updated if plugin configuration has been changed
	my $current_file_sequence_num = $config->{file_sequence_num};
	my $current_last_sequence_date = $config->{last_sequence_date};

	$config->{order_party_name} = argparam('order_party_name');
	$config->{account_number}   = argparam('account_number');
	$config->{sender_id}        = argparam('sender_id');
	$config->{fixed_desc}       = argparam('fixed_desc');
	$config->{item_desc}        = argparam('item_desc') || '%ACC_ID% %INV_NUM%';
	$config->{test_code}        = argparam('test_code');

	$config->{file_sequence_num}  = $current_file_sequence_num if (defined $current_file_sequence_num);
	$config->{last_sequence_date} = $current_last_sequence_date if (defined $current_last_sequence_date);
	
	throw_exception(
		type    => 'parameter',
		message => {
				string_id => 'bapm_clieop03_account_number_is_wrong',
				string_args => {
						acc => $config->{account_number}
                                }
		}
	) unless HSPC::MT::Plugin::PM::BT_Clieop03->account_number_check($config->{account_number});
	
	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_clieop03_order_party_name_is_wrong'}
	) if (!$config->{order_party_name} || length($config->{order_party_name}) > 35);
	
	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_clieop03_sender_id_is_wrong'}
	) if (!$config->{sender_id} || length($config->{sender_id}) > 5);
	
	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_clieop03_fixed_desc_is_wrong'}
	) if (!$config->{fixed_desc} || length($config->{fixed_desc}) > 32);

	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_clieop03_item_desc_is_wrong'}
	) if (!$config->{item_desc} || length($config->{item_desc}) > 32);

	return $config;
}

sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "pp_help_$action.html",
				data => {
					about_url =>
					  get_help_url( action => 'about', language => $language, )
				},
			);

	return $html;
}

1;
