package HSPC::Plugin::PP::BT_Norma19;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(argparam throw_exception string get_help_url);

use HSPC::MT::Plugin::PM::BT_Norma19;

use constant DEFAULT_OPT => {
	## default values for optional fields
	1  => '',
	2  => '%COMPANY%',
	3  => 'CIF: %CIF%',
	4  => 'FACTURA: %DOCNUM%',
	5  => 'NIF: %NIF%',
	6  => '',
	7  => '%DOCDESC%',
	8  => 'PVP Euros %CLRSUM%',
	9  => 'Plan: %HP%',
	10 => 'IVA Euros %TAXSUM%',
	11 => 'Date: %DATE%',
	12 => 'Total Euros %TOTSUM%',
	13 => 'Host: %DOMAIN%',
	14 => '',
	15 => '',
	16 => '',
};

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config = $h{config};
	my %data = (
		issuer_code        => $config->{issuer_code},
		issuer_name        => $config->{issuer_name},
		account_info       => sprintf(
			"%04d %04d %02d %010s",
			$config->{issuer_agency},
			$config->{issuer_branch},
			$config->{issuer_control_dig},
			$config->{issuer_account}
		),
		opt_address_add    => $config->{opt_address_add} ?
			string('yes') : string('no'),
	);

	for (my $i = 1; $i <= 16; $i++) {
		$data{"opt_field_$i"} = $config->{"opt_field_$i"};
	}

	return parse_template(
		path => __PACKAGE__,
		name => 'bt_norma19_view.tmpl',
		data => \%data,
	);
}

sub edit_form{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);

	my $config = $h{config};
	my %data = (
		issuer_code        => $config->{issuer_code},
		issuer_name        => $config->{issuer_name},
		issuer_agency      => $config->{issuer_agency},
		issuer_branch      => $config->{issuer_branch},
		issuer_control_dig => $config->{issuer_control_dig},
		issuer_account     => $config->{issuer_account},
		opt_address_add    => $config->{opt_address_add},
	);

	for (my $i = 1; $i <= 16; $i++) {
		$data{"opt_field_$i"} = defined $config->{"opt_field_$i"} ?
			$config->{"opt_field_$i"} : DEFAULT_OPT->{$i};
	}

	return parse_template(
		path => __PACKAGE__,
		name => 'bt_norma19_edit.tmpl',
		data => \%data,
	);
}

sub collect_data{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config = $h{config};

	$config->{issuer_code}        = argparam('issuer_code');
	$config->{issuer_name}        = argparam('issuer_name');
	$config->{issuer_agency}      = argparam('issuer_agency');
	$config->{issuer_branch}      = argparam('issuer_branch');
	$config->{issuer_control_dig} = argparam('issuer_control_dig');
	$config->{issuer_account}     = argparam('issuer_account');
	$config->{opt_address_add}    = argparam('opt_address_add');

	for (my $i = 1; $i <= 16; $i++) {
		$config->{"opt_field_$i"} = argparam("opt_field_$i");
	}

	throw_exception(
		type    => 'parameter',
		message => {
			string_id => 'bapm_norma19_issuer_name_is_required',
		}
	) unless $config->{issuer_name};

	throw_exception(
		type    => 'parameter',
		message => {
			string_id => 'bapm_norma19_issuer_code_1_12_chars',
		}
	) if (!$config->{issuer_code} || length($config->{issuer_code}) > 12);



	if ( !$config->{issuer_agency} ||
		$config->{issuer_agency} == 0 ||
		$config->{issuer_agency} =~ /\D/ ||
		length($config->{issuer_agency}) > 4
	) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_issuer_agency_1_4_digits',
			}
		);
	}

	if (length($config->{issuer_branch}) == 0
		|| $config->{issuer_branch} =~ /\D/ 
		|| length($config->{issuer_branch}) > 4
	) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_issuer_branch_1_4_digits',
			}
		);
	}

	if (!$config->{issuer_account} 
		|| $config->{issuer_account} == 0
		|| $config->{issuer_account} =~ /\D/ 
		|| length($config->{issuer_account}) > 10
	) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_issuer_account_1_10_digits',
			}
		);
	}

	my $control_dig = HSPC::MT::Plugin::PM::BT_Norma19->get_checksumm(
		agency_number  => $config->{issuer_agency},
		branch_number  => $config->{issuer_branch},
		account_number => $config->{issuer_account},
	);

	if ( $control_dig != $config->{issuer_control_dig} ) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_control_digits_mismatch',
			}
		);
	}

	return $config;
}

sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "pp_help_$action.html",
				data => {
					about_url =>
					  get_help_url( action => 'about', language => $language, )
				},
			);

	return $html;
}

1;