##
## Copyright (C) 2000 - 2004  SWsoft
## All rights reserved.
##
## Interface methods for Assist Payment Plugin configuration
##

package HSPC::Plugin::PP::OP_Assist;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(string argparam throw_exception get_help_url);

use constant OLD_ASSIST_PURCHASE_URL   => 'https://secure.assist.ru/shops/purchase.cfm';
use constant OLD_ASSIST_RESULT_URL     => 'https://secure.assist.ru/results/results.cfm';
use constant NEW_ASSIST_PURCHASE_URL   => 'https://payments.paysecure.ru/pay/order.cfm';
use constant NEW_ASSIST_RESULT_URL     => 'https://payments.paysecure.ru/resultbydate/resultbydate.cfm';
use constant CHECK_STATS_MIN_VALUE => 5; ##  5 minutes

use constant TEST_RESPONSES => {
	AS000 => '(AS000) Authorisation completed',
	AS001 => '(AS001) Authorisation completed (with CVC2)',
	AS010 => '(AS010) Preauthorisation completed',
	AS011 => '(AS011) Preauthorisation completed (with CVC2)',
	AS020 => '(AS020) Postauthorisation completed',
	AS100 => '(AS100) Authorisation refused',
	AS101 => '(AS101) Authorisation refused (invalid card number)',
	AS102 => '(AS102) Authorisation refused (limit exceeded)',
	AS104 => '(AS104) Authorisation refused (invalid exparation date)',
	AS105 => '(AS105) Authorisation refused (limit exceeded)',
	AS106 => '(AS106) Authorisation refused (invalid PIN-code)',
	AS107 => '(AS107) Authorisation refused (data transmition error)',
	AS108 => '(AS108) Authorisation refused (fraud suspicion)',
	AS109 => '(AS109) Authorisation refused (limit exceeded)',
	AS200 => '(AS200) Authorisation should be repeated',
	AS998 => '(AS998) Internal error',
};

sub edit_form{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config = $h{config};

	my $merchant_id = argparam('merchant_id') || $config->{merchant_id};
	my $old_api = argparam('old_api');
	if ($old_api eq '') {
		if ($config->{'old_api'} ne '') {
			$old_api = $config->{'old_api'}
		} else {
			$old_api = $config->{merchant_id} ? '1' : '0';
		}
	}
	my $purchase_api_url = $old_api
		? &OLD_ASSIST_PURCHASE_URL
		: &NEW_ASSIST_PURCHASE_URL;
	my $result_api_url = $old_api
		? &OLD_ASSIST_RESULT_URL
		: &NEW_ASSIST_RESULT_URL;

	## Escape
	$purchase_api_url =~ s/\\/\\\\/g;
	$purchase_api_url =~ s/'/\\'/g;
	$result_api_url =~ s/\\/\\\\/g;
	$result_api_url =~ s/'/\\'/g;

	my $html;
	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_assist_edit.tmpl',
				data => {
					old_api					=> $old_api,
					merchant_id             => $merchant_id,
					login                   => argparam('username')			|| $config->{login},
					purchase_url            => argparam('purchase_url')		|| $config->{purchase_url} || $purchase_api_url,
					result_url              => argparam('result_url')		|| $config->{result_url} || $result_api_url,
					new_purchase_url		=> &NEW_ASSIST_PURCHASE_URL,
					old_purchase_url		=> &OLD_ASSIST_PURCHASE_URL,
					new_result_url			=> &NEW_ASSIST_RESULT_URL,
					old_result_url			=> &OLD_ASSIST_RESULT_URL,
					prefix                  => argparam('prefix')			|| $config->{prefix},
					passwd                  => argparam('passwd')			|| $config->{passwd},
					test_mod                => argparam('test_mod')			|| $config->{test_mod},
					test_response           => argparam('test_response')	|| $config->{test_response},
					test_responses          => &TEST_RESPONSES,
					check_status_period_min => argparam('check_status_period_min')	|| $config->{check_status_period_min},
					num_of_tries            => argparam('num_of_tries')				|| $config->{num_of_tries},
				}
			);

	return $html;
}

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config             = $h{config};

	my $html;

	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_assist_view.tmpl',
				data => {
					merchant_id             => $config->{merchant_id},
					login                   => $config->{login},
					purchase_url            => $config->{purchase_url},
					result_url              => $config->{result_url},
					prefix                  => $config->{prefix},
					passwd                  => $config->{passwd},
					test_mod                => $config->{test_mod},
					test_response           => $config->{test_response},
					test_responses          => &TEST_RESPONSES,
					check_status_period_min => $config->{check_status_period_min},
					num_of_tries            => $config->{num_of_tries},

				}
		);

	return $html;
}

sub collect_data {
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config             = $h{config};

	$config->{old_api} = argparam('old_api');

	$config->{merchant_id}  = argparam('merchant_id') || $config->{merchant_id};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'assist_merchant_id_not_defined'
		}
	) unless $config->{merchant_id};

	$config->{login} = argparam('username') || $config->{login};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'assist_login_not_defined'
		}
	) unless $config->{login};

	$config->{purchase_url}         = argparam('purchase_url') || $config->{purchase_url};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'assist_purchase_url_not_defined'
		}
	) unless $config->{purchase_url};

	$config->{result_url}         = argparam('result_url') || $config->{result_url};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'assist_result_url_not_defined'
		}
	) unless $config->{result_url};

	$config->{prefix}         = argparam('prefix') || $config->{prefix};

	$config->{passwd} = argparam('passwd') || $config->{passwd};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'assist_passwd_not_defined'
		}
	) unless $config->{passwd};

	$config->{test_mod} = argparam('test_mod');

	$config->{test_response} = argparam('test_response') || $config->{test_response};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'assist_test_response_not_defined'
		}
	) if $config->{test_mod} && !$config->{test_response};

	$config->{check_status_period_min} = argparam('check_status_period_min') || $config->{check_status_period_min};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'assist_check_status_period_min_not_defined'
		}
	) unless $config->{check_status_period_min};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'assist_check_status_period_min_incorrect'
		}
	) unless $config->{check_status_period_min} =~ /^\d+$/;
	throw_exception(
		type           => 'error',
		vendor_message =>{
			string_id => 'assist_check_status_period_to_small',
			string_args => {
				min => &CHECK_STATS_MIN_VALUE
			}
		}
	) if $config->{check_status_period_min} < &CHECK_STATS_MIN_VALUE;

	$config->{num_of_tries} = argparam('num_of_tries') || $config->{num_of_tries};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'assist_num_of_tries_not_defined'
		}
	) unless $config->{num_of_tries};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'assist_num_of_tries_incorrect'
		}
	) unless $config->{num_of_tries} =~ /^\d+$/;

	return $config;
}

sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "assist_$action.html",
				data => {
					about_url =>
					get_help_url( action => 'about', language => $language, )
				}
			);

	return $html;
}

1;
