package HSPC::Plugin::PP::OP_Atos;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General
  qw(string argparam throw_exception get_data_dir get_help_url);
use HSPC::PluginToolkit::PP qw(callback_url);
use HSPC::MT::Plugin::PP::OP_Atos;

sub view_form {
    my $class = shift;
    my %h     = (
        config => undef,
        @_
    );

    my $config = $h{config};

    my $cert_exists = 0;
    $cert_exists = 1
      if ( -e HSPC::MT::Plugin::PP::OP_Atos->cert_file( config => $config ) );

    my %tmpl_args = (
        merchant_id => $config->{merchant_id},
        cert_exists => $cert_exists,
    );

    return parse_template(
        path => __PACKAGE__,
        name => 'op_atos_view.tmpl',
        data => \%tmpl_args
    );
}

sub edit_form {
    my $class = shift;
    my %h     = (
        config => undef,
        @_
    );

    my $config = $h{config};

    my $cert_exists = 0;
    $cert_exists = 1
      if ( -e HSPC::MT::Plugin::PP::OP_Atos->cert_file( config => $config ) );

    my $callback_url = callback_url( action => "notify" );

    my %tmpl_args = (
        callback_url => $callback_url,
        merchant_id  => argparam('merchant_id') || $config->{merchant_id},
        merchant_cert => $cert_exists,
    );

    return parse_template(
        path => __PACKAGE__,
        name => 'op_atos_edit.tmpl',
        data => \%tmpl_args
    );

}

sub collect_data {
    my $class = shift;
    my %h     = (
        config => undef,
        @_
    );

    my $config = $h{config};

    my $cert_exists = 0;
    $cert_exists = 1
      if ( -e HSPC::MT::Plugin::PP::OP_Atos->cert_file( config => $config ) );

    $config->{merchant_id} = argparam('merchant_id');

    throw_exception(
        type           => 'error',
        vendor_message => string('atos_merchant_id_not_defined')
    ) unless $config->{merchant_id};

	throw_exception(
        type           => 'error',
        vendor_message => string('atos_merchant_cert_not_defined')
    ) if ( !($cert_exists || argparam('cert_file') ) );

    my $cert_file_fh = argparam('cert_file');
    if ( $cert_file_fh && "GLOB" eq ref($cert_file_fh) ) {
        open FILE,
          '>' . HSPC::MT::Plugin::PP::OP_Atos->cert_file( config => $config )
          or die "Cannot open certificate file for writing: $!";

       	print FILE while <$cert_file_fh>;
        close FILE;
    }

    HSPC::MT::Plugin::PP::OP_Atos->path_file();
    HSPC::MT::Plugin::PP::OP_Atos->parmcom_file( config => $config );

    return $config;
}

sub get_help_page {
    my $class = shift;
    my %h     = (
        action   => undef,
        language => undef,
        @_
    );

    my $action   = $h{action};
    my $language = $h{language};

    my $html = parse_template(
        path => __PACKAGE__ . '::' . uc($language),
        name => "atos_$action.html",
        data => {
            about_url =>
              get_help_url( action => 'about', language => $language, )
        }
    );

    return $html;
}
1;
