##
## Copyright (C) 2000 - 2004  SWsoft
## All rights reserved.
##
## Interface methods for BBVA Payment Plugin configuration
##

package HSPC::Plugin::PP::OP_BBVA;
use strict;

use HSPC::MT::Plugin::PP::OP_BBVA;

use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(string argparam throw_exception get_help_url);


use constant SERVER_URL => 'https://w3.grupobbva.com/TLPV/tlpv/TLPV_pub_rpcrouter';

sub edit_form{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $html;
	my $config             = $h{config};

	my %tmpl_args = (
		merchant_id       => argparam('merchant_id')
									|| $config->{merchant_id},
		payment_url       => argparam('payment_url')
									|| $config->{payment_url}
									|| &SERVER_URL,
		terminal_id       => argparam('terminal_id')
									|| $config->{terminal_id},
		secure_admin_key  => argparam('secure_admin_key')
									|| $config->{secure_admin_key},
		secure_word       => argparam('secure_word')
									|| $config->{secure_word},
	);

	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_bbva_edit.tmpl',
				data => \%tmpl_args
			);

	return $html;
}

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config             = $h{config};

	my $html;

	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_bbva_view.tmpl',
				data => {
					merchant_id       => $config->{merchant_id},
					payment_url       => $config->{payment_url},
					terminal_id       => $config->{terminal_id},
					secure_admin_key  => $config->{secure_admin_key},
					secure_word       => $config->{secure_word},
				}
		);

	return $html;
}

sub collect_data {
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config = $h{config};

	$config->{merchant_id} = argparam('merchant_id') || $config->{merchant_id};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'bbva_merchant_id_not_defined'
		}
	) unless $config->{merchant_id};

	$config->{payment_url} = argparam('payment_url') || $config->{payment_url};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'bbva_payment_url_not_defined'
		}
	) unless $config->{payment_url};

	$config->{terminal_id} = argparam('terminal_id') || $config->{terminal_id};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'bbva_terminal_id_not_defined'
		}
	) unless $config->{terminal_id};

	$config->{secure_admin_key}      = argparam('secure_admin_key') || $config->{secure_admin_key};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'bbva_secure_admin_key_not_defined'
		}
	) unless $config->{secure_admin_key};

	$config->{secure_word}      = argparam('secure_word') || $config->{secure_word};
	throw_exception(
		type           => 'error',
		vendor_message => {
			string_id => 'bbva_secure_word_not_defined'
		}
	) unless $config->{secure_word};

	return $config;
}


sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "bbva_$action.html",
				data => {
					about_url =>
					get_help_url( action => 'about', language => $language, )
				}
			);

	return $html;
}

1;
