package HSPC::Plugin::PP::OP_BancaSella;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(string argparam throw_exception get_help_url);
use HSPC::PluginToolkit::PP qw(callback_url);

use constant BANCASELLA_SERVER_URL => 'https://ecomm.sella.it/gestpay/pagam.asp';

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config = $h{config};
	my $html;

	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_bancasella_view.tmpl',
				data => {
					server_url 		=> $config->{server_url},
					merchant_id 	=> $config->{merchant_id},
					callback_url    => $config->{callback_url},
					basic_agreement => $config->{basic_agreement},
					callback_url_notify => callback_url(
						action => 'notify',
					),
					callback_url_error => callback_url(
						action => 'result',
						result => 'Failure'
					),
					callback_url_ok => callback_url(
						action => 'result',
						result => 'Success'
					)
				}
		);

	return $html;
}

sub edit_form{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $html;
	my $config = $h{config};

	my %tmpl_args = (
		merchant_id => argparam('merchant_id') || $config->{merchant_id},
		server_url => argparam('server_url')
		  || $config->{server_url}
		  || &BANCASELLA_SERVER_URL,
		basic_agreement => argparam('basic_agreement')
			  || $config->{basic_agreement},
	);

	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_bancasella_edit.tmpl',
				data => \%tmpl_args
			);
	return $html;
}

sub collect_data{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config = $h{config};

	$config->{server_url}		= argparam('server_url');
	throw_exception(type => 'error', vendor_message => string('bancasella_server_url_not_defined'), )
		unless $config->{server_url};

	$config->{merchant_id}		= argparam('merchant_id');
	throw_exception(type => 'error', vendor_message => string('bancasella_merchant_id_not_defined'), )
		unless $config->{merchant_id};

	$config->{basic_agreement}	= argparam('basic_agreement');
	return $config;
}


sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "bancasella_$action.html",
				data => {
					about_url =>
						get_help_url( action => 'about', language => $language, )
				},
			);

	return $html;
}

1;