package HSPC::Plugin::PP::OP_BancaSellaSSL;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(string argparam throw_exception get_help_url);
use HSPC::PluginToolkit::PP qw(callback_url);

use constant BANCASELLASSL_SERVER_URL => 'https://ecomm.sella.it/gestpay/pagam.asp';

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config = $h{config};
	my $html;

	$html = parse_template(
		path => __PACKAGE__,
		name => 'op_bancasellassl_view.tmpl',
		data => {
			server_url         => $config->{server_url},
			merchant_id        => $config->{merchant_id},
			callback_url_notify => callback_url(
				action => 'notify',
			),
		},
	);

	return $html;
}

sub edit_form{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $html;
	my $config = $h{config};

	my %tmpl_args = (
		server_url => argparam('server_url')
		  || $config->{server_url}
		  || &BANCASELLASSL_SERVER_URL,
		merchant_id => argparam('merchant_id') || $config->{merchant_id},
	);

	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_bancasellassl_edit.tmpl',
				data => \%tmpl_args
			);

	return $html;
}

sub collect_data{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config = $h{config};

	$config->{server_url}		= argparam('server_url');
	throw_exception(type => 'error', vendor_message => string('bancasellassl_server_url_not_defined'), )
		unless $config->{server_url};

	$config->{merchant_id}		= argparam('merchant_id');
	throw_exception(type => 'error', vendor_message => string('bancasellassl_merchant_id_not_defined'), )
		unless $config->{merchant_id};

	return $config;
}

sub get_help_page {
	my $class = shift;
    my %h = (
        action   => undef,
        language => undef,
        @_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "bancasellassl_$action.html",
				data => {
					about_url =>
					  get_help_url( action => 'about', language => $language, )
				},
			);

	return $html;
}

1;