##
## Copyright (C) 2000 - 2004  SWsoft
## All rights reserved.
##
## Interface methods for Nordea Payment Plugin configuration
##

package HSPC::Plugin::PP::OP_Nordea;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(string argparam throw_exception get_help_url);

use HSPC::MT::Plugin::PP::OP_Nordea;

use constant NORDEA_SERVICES_PAYMENT_DEFAULT
	=> 'https://solo3.nordea.fi/cgi-bin/SOLOPM01';
use constant NORDEA_SERVICES_REFUND_DEFAULT
	=> 'https://solo3.nordea.fi/cgi-bin/SOLOPM09';
use constant NORDEA_LANGUAGE_DEFAULT => 2;


sub edit_form{
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config             = $h{config};

	my $html;
	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_nordea_edit.tmpl',
				data => {
					payment_url       => argparam('payment_url')
												|| $config->{payment_url}
												|| &NORDEA_SERVICES_PAYMENT_DEFAULT,
					merchant_id  => argparam('merchant_id')
												|| $config->{merchant_id},
					merchant_mac => argparam('merchant_mac')
												|| $config->{merchant_mac},
					mac_ver      => argparam('mac_ver')
												|| $config->{mac_ver},
					language     => argparam('language')
												|| $config->{language}
												|| &NORDEA_LANGUAGE_DEFAULT,
					test_mode    => argparam('test_mode')
												|| $config->{test_mode},
					languages    => HSPC::MT::Plugin::PP::OP_Nordea->ND_LANGUAGES,
				}
			);

	return $html;
}

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config             = $h{config};

	my $html;

	$html = parse_template(
				path => __PACKAGE__,
				name => 'op_nordea_view.tmpl',
				data => {
					payment_url       => $config->{payment_url},
					merchant_id  => $config->{merchant_id},
					merchant_mac => $config->{merchant_mac},
					mac_ver      => $config->{mac_ver},
					language          => $config->{language},
					test_mode         => $config->{test_mode},
					languages         => HSPC::MT::Plugin::PP::OP_Nordea->ND_LANGUAGES,
				}
		);

	return $html;
}

sub collect_data {
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config             = $h{config};

	$config->{payment_url}  = argparam('payment_url') || $config->{payment_url};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_payment_url_not_defined'
		}
	) unless $config->{payment_url};

	$config->{test_mode}        = argparam('test_mode');

	$config->{merchant_id} = argparam('merchant_id') || $config->{merchant_id};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_merchant_id_not_defined'
		}
	) if !$config->{test_mode} && !$config->{merchant_id};
	## merchant_id: alphanumeric < 15 symb
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_merch_id_fmt_err'
		}
	) if !$config->{test_mode} && !($config->{merchant_id} =~ /^[\w\d]{1,15}$/);

	$config->{merchant_mac}         = argparam('merchant_mac') || $config->{merchant_mac};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_merchant_mac_not_defined'
		}
	) if !$config->{test_mode} && !$config->{merchant_mac};
	## merchant_mac: alphanumeric < 32 symb
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_merch_mac_fmt_err'
		}
	) if !$config->{test_mode} && !($config->{merchant_mac} =~ /^[\w\d]{1,32}$/);

	$config->{mac_ver}         = argparam('mac_ver') || $config->{mac_ver};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_mac_ver_not_defined'
		}
	) if !$config->{test_mode} && !$config->{mac_ver};
	## mac_ver: numeric < 4 symb
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_mac_ver_fmt_err'
		}
	) if !$config->{test_mode} && !($config->{mac_ver} =~ /^[\d]{1,4}$/);

	$config->{language}         = argparam('language') || $config->{language};
	throw_exception(
		type           => 'error',
		vendor_message => {
			 string_id => 'nordea_language_not_defined'
		}
	) unless $config->{language};


	return $config;
}

sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "nordea_$action.html",
				data => {
					about_url =>
					get_help_url( action => 'about', language => $language, )
				}
			);

	return $html;
}

1;
