package HSPC::Plugin::PP::OP_Ogone;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(string argparam throw_exception get_help_url);

use constant DEFAULT_GATEWAY_URL => 'https://secure.ogone.com/ncol/test/orderdirect.asp';

sub view_form {
	my $class = shift;
	my %h =@_;

	my $config = $h{config};

	return parse_template(
				path => __PACKAGE__,
				name => 'op_ogone_view.tmpl',
				data => {
					server_url 		=> $config->{server_url},
					merchant_id 	=> $config->{merchant_id},
					user_id 		=> $config->{user_id},
					password		=> $config->{password},					
				}
		);
}

sub edit_form{
	my $class = shift;
	my %h = @_;

	my $config = $h{config};

	return parse_template(
				path => __PACKAGE__,
				name => 'op_ogone_edit.tmpl',
				data => {
					server_url 		=> argparam('server_url') || $config->{server_url} || &DEFAULT_GATEWAY_URL,
					merchant_id 	=> argparam('merchant_id') || $config->{merchant_id},
					user_id 		=> argparam('user_id') || $config->{user_id},
					password		=> argparam('password') || $config->{password},					
				}
			);
}

sub collect_data{
	my $class = shift;
	my %h = @_;

	my $config = $h{config};

	$config->{server_url} = argparam('server_url');
	throw_exception( vendor_message => { string_id => 'ogone_server_url_not_defined' })
		unless $config->{server_url};

	$config->{merchant_id} = argparam('merchant_id');
	throw_exception( vendor_message => { string_id => 'ogone_merchant_id_not_defined' })
		unless $config->{merchant_id};
		
	$config->{user_id} = argparam('user_id');
	throw_exception( vendor_message => { string_id => 'ogone_user_id_not_defined' })
		unless $config->{user_id};

	$config->{password}	= argparam('password');
	throw_exception( vendor_message => { string_id => 'ogone_passwd_not_defined' })
		unless $config->{password};

	return $config;
}

sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my $html = parse_template(
		path => __PACKAGE__ . '::' . uc($language),
		name => "ogone_$action.html",
		data => {
			about_url =>
			  get_help_url( action => 'about', language => $language, )
		},
	);

	return $html;
}
1;