package HSPC::PluginToolkit::DM::PGPTool;

use strict;
use SWsoft::protect;

use vars qw(@ISA @EXPORT_OK);
use HSPC::ParamsValidate qw(:DEFAULT);

use HSPC::MT::DM::GPGKeyManager;

use base qw(Exporter);
our @EXPORT_OK = qw(import_keyblock clearsign_text verify_text);

## Description:
##		Import keyblock from ASCII armored
## Input params:
##		data => ASCII armored keyblock
## Return:
##		HASHREF:
##	{
##		error => undef,
##		data => [
##			{
##				hex_id => '558B8A4E7F8E3F4E6DEFB2168B31DEAFBD992410',
##				name => 'Casper Kamp <domains@cee-kay.net>',
##				is_sec => '0'
##			},
##			... - if keyblock includes more than one keys
##		]
##	}
##
sub import_keyblock {
	my %h = validate_params(
		@_,
		{
			data	=> {type => SCALAR},
		}
	);

	my $gpg_obj = HSPC::MT::DM::GPGKeyManager->new();
	return $gpg_obj->import_keyblock(block_data => $h{data});
}

## Description:
##		Clear sign test message by specified key
## Input params:
##		hex_id		=> key's hex_id
##		passphrase	=> passphrase for secret key
##		text		=> text messages wich want to sign
## Return:
##		HASHREF: {
##			error	=> undef,
##			data	=> signed text messages
##		}
##
sub clearsign_text {
	my %h = validate_params(
		@_,
		{
			hex_id		=> {type => SCALAR},
			passphrase	=> {type => SCALAR},
			text		=> {type => SCALAR},
		}
	);

	my $gpg_obj = HSPC::MT::DM::GPGKeyManager->new();
	return $gpg_obj->clearsign_text(
		hex_id		=> $h{hex_id},
		passphrase	=> $h{passphrase},
		text		=> $h{text},
	);
}

## Description:
##		Clear sign test message by specified key
## Input params:
##		text		=> signed text messages
## Return:
##		HASHREF: {
##			hex_id	=> '558B8A4E7F8E3F4E6DEFB2168B31DEAFBD992410',
##			text	=> decoded text message
##		}
##
sub verify_text {
	my %h = validate_params(
		@_,
		{
			text	=> {type => SCALAR},
		}
	);

	my $gpg_obj = HSPC::MT::DM::GPGKeyManager->new();
	return $gpg_obj->verify_text(text => $h{text});
}

1;
