package HSPC::PluginToolkit::DM::ToolsEx;

use strict;

## use HSPC::MT::DM::PluginEngineEx;
use HSPC::MT::DM::Contact;
use HSPC::MT::DM::ContactExtData;
use HSPC::MT::DM::Constants;
use HSPC::MT::PluginTools qw(get_request_data);
use HSPC::MT::DM::ContactsFactory;
use HSPC::WebDB;
use Storable qw(thaw freeze);

use base qw(Exporter);
use vars qw(@EXPORT);

@EXPORT = qw(
	process_domain_contacts
);

my (
	%domains_accounts
		## map <domain> => <account_no>
);

sub _prepare_contacts_data {
	## map <domain> => <account_no>
	my $sql = qq[
		select bd.domain, bd.account_no from dm_basedomain bd
	];
	my $all_domains = select_rows($sql);
	%domains_accounts = map {$_->[0] => $_->[1]} @$all_domains;
}

## plugin_result - structure, that was returned from plug-in
##      HASHREF { domain1 => {is_success => 1|0, message => '...', contacts => HASHREF, contacts_extdata => HASHREF, domain_extdata => HASHREF}
## print_messages - 1|0 (print messages about domains processing into stdout or not)
## plugin_id - this parameter can be specified only if this sub is used not from plug-ins
sub process_domain_contacts {
	my %h = (
		plugin_result => undef,
		print_messages => 0,
		plugin_id => undef,
		@_
	);
	my $domains_info = $h{plugin_result};
	if (defined($domains_info->{is_success}) && !$domains_info->{is_success}) {
		print '  ' . $domains_info->{message} . "\n" if $h{print_messages};
		return;
	}

	my $plugin_id = $h{plugin_id} || get_request_data('plugin_config_id');
	_prepare_contacts_data();

	my $plugin_obj = HSPC::MT::DM::PluginEngineEx->new(
		plugin_id => $plugin_id,
	);

	while (my ($domain, $domain_data) = each %{$domains_info}) {
		## if is_success=0, then contacts will not be saved (domain extdata will be saved, if exists in hash)
		print '  Domain "' . $domain . '"' . "\n" .
			'    status: ' . ($domain_data->{is_success} ? 'success' : 'error') . "\n" .
			'    message: ' . $domain_data->{message} . "\n"
		if $h{print_messages};

		unless (exists($domains_accounts{$domain})) {
			## If no value for this domain exist in hash %domains_accounts => domain doesn't present in HSPC
			print "    This domain doesn't exist in HSPC. Skipped\n"
				if $h{print_messages};
			next;
		}

		next unless $domain_data->{is_success};

		## print warnings
		if ($h{print_messages}) {
			while (my ($contact_type,$contact) = each %{$domain_data->{contacts}}) {
				print "      Warning: $contact_type contact has no e-mail. Not saved\n"
					unless ($contact->{email});
			}
		}
		## now save into HSPC database
		HSPC::MT::DM::ContactsFactory->save_imported_contacts(
			domain => $domain,
			contacts => $domain_data->{contacts},
			contacts_extdata => $domain_data->{contacts_extdata},
		);
		
	}

	print "  Saving domain extdata into DB...\n"
		if $h{print_messages};
	my %domains_extdata =
		map {$_ => $domains_info->{$_}->{domain_extdata}}
		grep {exists($domains_info->{$_}->{domain_extdata})} ## earlier we grepped by is_success
		keys %{$domains_info};

	$plugin_obj->add_domains_extdata(
		domains_extdata => \%domains_extdata,
	);
}

1;