package HSPC::PluginToolkit::HTMLTemplate;

use strict;
use SWsoft::protect;

use HSPC::ParamsValidate qw(:DEFAULT);
use HSPC::Logger qw(sw_log_debug sw_log_warn sw_log sw_die);
use HSPC::Localization;

use base qw(Exporter);
our @EXPORT_OK = qw(parse_template);

sub parse_template {
	my %h = validate_params(
		@_,
		{
			path => {
				type => SCALAR | UNDEF, 
				optional => 1,
				default => __PACKAGE__
			},
			name => {type => SCALAR},
			data => {type => HASHREF | UNDEF, optional => 1},
		}
	);

	my $template_path = $h{path};
	my $template_name = $h{name};
	my $input = $h{data};
	my %args=(@_);
	my $sys = $ENV{HSPC_SYSTEM};

	unless ($sys)
	{
		require HSPC::System;
		$sys = HSPC::System->new();
	}

	if($template_path=~m|\:\:|g){
		$template_path =~ s#::#/#g;
	};
	$template_path .= ($template_path ? '/' : '').$template_name
		if defined $template_name;

	$template_path = $sys->find_text_template_path(tmpl_path=>$template_path);

	if (! -r $template_path) {
		my $error = "[WEBSYSTEMLIB] ERROR : file '$template_name' not found by full path '$template_path'";
		sw_log_warn( $error );
		return;
	}

	$args{TYPE} = 'FILE';
	$args{SOURCE} = $template_path;

	my $result;
	my $template = $sys->find_text_template(%args);
	if(ref($input) eq 'HASH' || !$input){
		$result = $template->fill_in(HASH => $input, BROKEN => \&template_broken,BROKEN_ARG=>\$template_path,%args);
	}else{
		$result = $template->fill_in(PACKAGE  => $input, BROKEN => \&template_broken,BROKEN_ARG=>\$template_path,%args);
	}
	$result=~s/<txt_([^\s>]+(\s+\w+=\"(|.*?[^\\])\")*?)\/?>/_js_escaped_string($1)/ge;
	$result=~s/<env_(\w+)\/?>/$ENV{$1}/ge;
	$result=~s/<img_(\w+)\/?>/$sys->ui_image_src(id=>$1)/ge;
	$result=~s/<imgpath_(\w+)\/?>/$sys->ui_image_path(id=>$1)/ge;## handle tag <imgpath_id>
	$result=~s/<img_(\w+)\s/'<img src = "'.$sys->ui_image_path(id=>$1).'" '/ge;## handle tag <img_id ...>
	return $result;
}

sub _js_escaped_string {
	my $s = shift;
	$s = string($s);
	$s =~ s/'/\&rsquo;/g;
	return $s;
}##_js_escaped_str

sub template_broken {
	my %args = @_;
	my $ref_arg = $args{arg};
	my $c = "[WEBSYSTEMLIB] ERROR : template '$$ref_arg' broken at line number $args{lineno}: $args{error}";
	sw_log_warn $c;
 	return qq|<H3><font color="red">Template broken</font></H3><!-- $args{error} --></font></H3>|;
}

1;
