package HSPC::Profiler;

use strict;
use SWsoft::protect;	

use Benchmark qw(timediff timestr :hireswallclock);
## do not use 'use' instead of 'require'
require HSPC::MT::Core::Profiler;
use HSPC::Config;

use HSPC::Localization::Date qw(secs_to_time);
use Apache();
use Apache::Table();
use HSPC::Logger;
use HSPC::MT::Core::Constants qw(:log_dirs);

sub profiler {
	my %h = (
		comment	=> undef,
		url	=> undef,
		@_
	);

	my $url = $h{url};
	if(!$url && $ENV{layout}) {
		$url = $ENV{layout}->uri();
	}
	if(!$h{comment}) {
		my (undef, $filename, $line, $subr) = caller(1);
		$h{comment} = "Line: $line, file: $filename, function: $subr";
	}

	## this require needs to be inside subroutine
	## outside it may produce errors during compilation (due to cyclic uses)
	return HSPC::Profiler->new(comment=>"$h{comment}", url=>"$url");
}

    
sub new {
	my $class = shift;
	my %h = (
		comment => undef, ## comment
		url => undef,
		@_
	);

	my $self = {
		comment => $h{comment},
		timer1 => Benchmark->new(),
		url => $h{url},
		log_start => 0,
		sql_start => 0,
		flushed => 0,
	};
	bless($self,$class);
	
	if($ENV{system_obj}) {
		my $so = $ENV{system_obj};
		$self->{log_start} = $so->{_log_log} && @{$so->{_log_log}} ?
			scalar(@{$so->{_log_log}}) : 0;
		$self->{sql_start} = $so->{_sql_stack} && @{$so->{_sql_stack}} ?
			scalar(@{$so->{_sql_stack}}) : 0;
		
	}
	return $self;
};

sub flush {
	my $self = shift;
	
	return undef if !$CONFIG{CORE_PROFILER};
	
	my $td = timediff(Benchmark->new(), $self->{timer1});
	my $prof_mt = HSPC::MT::Core::Profiler->new();
	my $time_str = timestr($td);
	my ($wc,$ut,$st) = ($time_str =~ /([\d\.]+)\D+([\d\.]+)\D+([\d\.]+)/);
	
	
	$prof_mt->wallclock($wc);
	$prof_mt->user_time($ut);
	$prof_mt->system_time($st);
	$prof_mt->url($self->{url});
	$prof_mt->comment($self->{comment});
	$prof_mt->remote_ip($ENV{REMOTE_ADDR} || '');
	$prof_mt->starttime(secs_to_time($self->{timer1}->[0]));
	$prof_mt->endtime(secs_to_time(time));
	my $request_headers = Apache->request()->headers_in();
	my $headers_str = join("\n", map {$_ . ': ' . $request_headers->{$_}} keys(%$request_headers));
	$prof_mt->request_headers($headers_str);
	$prof_mt->request_method(Apache->request()->method());
	
	$prof_mt->save(); ## first save to get unique ID for names of logfiles
	my $hspc_log_filename = &HSPC_PROF_DIR . 'profile_hspc_' . $prof_mt->id();
	my $sql_log_filename = &HSPC_PROF_DIR . 'profile_sql_' . $prof_mt->id();
	$prof_mt->hspc_log($hspc_log_filename);
	$prof_mt->sql_log($sql_log_filename);

	if (open LOGFILE, '>', $hspc_log_filename) {
		my $hspc_log = HSPC::Logger->get_debug_log();
		print LOGFILE $hspc_log;
		close LOGFILE;
	}
	if (open LOGFILE, '>', $sql_log_filename) {
		my $sql_log = HSPC::Logger->get_sql_log();
		print LOGFILE $sql_log;
		close LOGFILE;
	}
	
	$self->{flushed} = 1;
	$prof_mt->save();
}

##--------------------------------------------------
## destructor
sub DESTROY {
	my $self = shift;
	return undef if $self->{flushed};
	$self->flush();
};




1;

