package HSPC::String;

use strict;

## Mark this module for encryption upon build
use SWsoft::protect;

use HSPC::Localization();


use overload (
    '""'       =>   'text',
    'ne'       =>   'text',
    'eq'       =>   'text',
    'cmp'      =>   'text',
);

##--------------------------------------------------
## Usage:  HSPC::String->new(string_id=>'xxx', string_args=>{'yyy'=>'xxx'});
## Description: Constructor
## Parameters:
##		string_id =>>
##      string_args =>
## Return value:
## 		HSPC::String obj ref
sub new {
	my $class = shift;
	my %h     = (
		string_id   => undef,
		string_args => {},
		@_
	);

	my $self = {};
	bless( $self, $class );

	$self->{string_id}   = $h{string_id} || die "HSPC::String creation try w/o string_id";
	$self->{string_args} = $h{string_args};

	return $self;
}

sub set_arg {
	my $self  = shift;
	my $key   = shift;
	my $value = shift;

	$self->{string_args}->{$key} = $value;
}

## get text string for specified language
sub text_for_lang {
	my $self = shift;
	my %h = (
		lang    => undef, ## language for localize.
		__level => 0,     ## __level, to prevent unlimited recursion
		@_
	);
	my $lang = $h{lang} || die "HSPC::String text_for_lang w/o lang specified";
	my $level = $h{__level} || 0;
	die ("HSPC::String unlimited recursion") if ($level > 100);
	
	## first = convert  all arguments to localized strings.
	my %args;

	while ( my ( $key, $value ) = each( %{ $self->{string_args} } ) ) {
		if ( ref($value) ) {
			## object of HSPC::String or HSPC::MT::LangValue type.
			## Both of them support method text_for_lang
			$args{$key} = $value->text_for_lang(
				lang    => $lang,
				__level => $level + 1
			);
		} else {
			## already usual string
			$args{$key} = $value;
		}
	}

	return HSPC::Localization::string_for_lang( $lang, $self->{string_id}, %args );
};

## text for current language 
sub text {
	my $self = shift;
	return $self->text_for_lang( lang => HSPC::Localization::current_lang() );
}

1;
__END__
