package HSPC::System;
use strict;

use Error;
use HSPC::Exc;

use HSPC::Config;
use HSPC::Logger qw(sw_atrace sw_die);
use Text::Template;
use HSPC::ErrorStack;

use HSPC::MT::Core::Constants;
use SWsoft::protect;	# Mark this module for encryption upon build
use HSPC::Localization;
use POSIX qw(strftime);


use base 'HSPC::Application';

# >  class
sub new {
	my ($class, %h) = @_;

	my $self = $class->SUPER::new(%h);
	$ENV{system_obj} = $self;

	# init error object #
	$ENV{error_obj} = HSPC::ErrorStack->new();	

	return $self;
}

### >> self
### << current provider config, ref to HSPC::MT::GM::ProviderConfig instance
sub provider_config{
	my $self = shift;
	return {};
}


### >> self
### << current provider config, ref to HSPC::MT::GM::ProviderConfig instance
sub hsp_provider_config{
	my $self = shift;
	return {};
}


sub is_console{
	return 1;
}

## Search skin and store it in enviroment
## >> class
## << HSPC::MT::CU::Skin || undef
sub find_skin {
	my $class = shift;
	
	##my $skin = "winxp_blue";
	##$ENV{skin_obj} = $skin;
	
	##return $skin;
}


# >>  self
# =>> SOURCE : template_path
# =>> any arguments for Text::Template
# < ref to Text::Template instance
sub find_text_template{
	my ($self,%args) = @_;
	
	my $template = new Text::Template( %args );
	unless($template->compile){
		sw_die(
			qq|Error: template '$args{SOURCE}' is broken : |
			.$Text::Template::ERROR
		);
	}
	return $template;
}


## check if template has been customized and return 
## path to template to be parsed
## >> self
## =>> base_path - base path to templates directory
## =>> tmpl_path - relative path to template # REQ_Y #
## < absolute path to template
sub find_text_template_path {
	my ($self,%args) = @_;
	
	## base path to templates directory
	my $base_path = $args{base_path} || $CONFIG{HSPCROOT_ROOT}.
			$CONFIG{HSPCROOT_DIR_TEMPLATE}.'/';
	my $file = $args{file};

	## relative path to template
	my $tmpl_path = $args{tmpl_path};

	## detach base path from template path (if needed)
	if(substr($tmpl_path,0,length($base_path)) eq $base_path) {
		$tmpl_path = substr($tmpl_path,length($base_path));
	}

	## return if template path is absolute (nothing to be done)
	return $tmpl_path if substr($tmpl_path,0,1) eq '/';
	
	## Search skin-dependent templates
	
	## retrieve skin object
	##my $skin = $ENV{skin_obj} || HSPC::MT::CU->find_default_skin();
	my $path;
	##if ($skin) { 
	##	$path = $skin->find_resource_path( rel_path => '/template/'.$tmpl_path);
	##} else { ## search in default folder if skin is not defined
		$path = $CONFIG{HSPCROOT_ROOT}.'/template/'.$tmpl_path;
	##}
	return $path;
}

# >> self
# >> template_path/class_name/object
# >> template_name
# >  input
# >  named args for Text::Template->fill_in()
sub parse_text_template{
	my $self = shift;
	my $template_path = shift;
	my $template_name = shift;
	my $input = shift;	
	my %args=(@_);
		
	if($template_path=~m|\:\:|g){		
		$template_path =~ s#::#/#g;
	};
	$template_path .= '/'.$template_name if defined $template_name;

	$template_path = $self->find_text_template_path(tmpl_path=>$template_path);

	if (! -r $template_path) {
		my $error = "File '$template_name' not found by full path $template_path";
		return $error;
	}

	$args{TYPE} = 'FILE';
	$args{SOURCE} = $template_path;
	my $result;
	my $template = $self->find_text_template(%args);	
	
	if(ref($input) eq 'HASH' || !$input){								
		$result = $template->fill_in(
			HASH => $input, 
			BROKEN => \&_template_broken,
			BROKEN_ARG=>\$template_path,%args
		);		
	}else{		
		$result = $template->fill_in(
			PACKAGE  => $input, 
			BROKEN => \&_template_broken,
			BROKEN_ARG=>\$template_path,%args
		);
	}	
	
	$result=~s/<txt_([^>]+)>/string($1)/ge;
	$result=~s/<env_(\w+)>/$ENV{$1}/ge;
	
	return $result;
}

################# show calls stack
### >> class | => self
### no results
sub warn_calls_stack{
	warn('System::warn_calls_stack usage is deprecated. Use HSPC::Logger');
}

################# show calls stack
### >> class | => self
### << text report with \n
sub calls_stack{
	warn('System::calls_stack usage is deprecated. Use HSPC::Logger');
}


sub DESTROY{
	delete $ENV{error_obj} if defined $ENV{error_obj};
}


1;

__END__


=== /SESSION VARIABLES ===

=== ENV VARIABLES ===

HSPC_SYSTEM or system_obj: ref to HSPC::System instance
HSPC_SYSTEM_DBH : database connections (hash)

=== /ENV_VARIABLES ===

==== HSPC::System object methods ====

==== /HSPC::System object methods ====

==== HSPC::System object private attributes ====	
	_sql_stack : sql log
	_log_log: log with all all log() and log_debug() messages
	_hsp_provider_config : ref to HSP provider config
==== /HSPC::System object private attributes ====

