package HSPC::System::SSLTransport;


use strict;
use SWsoft::protect;	# Mark this module for encryption upon build

use Socket;
use Net::SSLeay qw(die_now die_if_ssl_error) ;


use HSPC::Logger qw(sw_log_debug sw_die);


## >> class
## =>>hostname
## =>>port
## < self
sub new
{
	my ($class,%h) = @_;

	my $self = {};
	bless $self,$class;

	##Required params
	$self->{_hostname} = $h{hostname} || sw_die("No hostname");
	$self->{_port} = $h{port} || sw_die("No port");
	##/Required params

	return $self;
}##/new


##>>self
##< error
sub set_connection
{
	my $self = shift;

	##initialize Net::SSLeay
	Net::SSLeay::load_error_strings();
	Net::SSLeay::SSLeay_add_ssl_algorithms();
	Net::SSLeay::randomize();

	my $dest_serv = $self->{_hostname};
	my $port = $self->{_port};
	my $dest_ip = gethostbyname ($dest_serv);
	##i18n
	return "Cannot resolve $dest_serv" unless ($dest_ip);
	my $dest_serv_params  = sockaddr_in($port, $dest_ip);

	socket(S, &AF_INET, &SOCK_STREAM,0) or return "Failed to create socket: $!";
	connect(S, $dest_serv_params) or return "Failed to connect to server: $!";
	select  (S); $| = 1; select (STDOUT);## Eliminate STDIO buffering

	##The network connection is now open, lets fire up SSL
	my $ctx;
	unless($ctx = Net::SSLeay::CTX_new()) {
		close(S);
		return "Failed to create SSL CTX: $!";
	}
	my $res;
	if(Net::SSLeay::CTX_set_options($ctx, &Net::SSLeay::OP_ALL)) {
		close(S);
		return "Failed to set SSL CTX options";
	}
	my $ssl;
	unless($ssl = Net::SSLeay::new($ctx)) {
		close(S);
		return"Failed to create SSL connection: $!";
	}
	Net::SSLeay::set_fd($ssl, fileno(S)); ##Must use fileno
	$res = Net::SSLeay::connect($ssl);
	if($res) {
		my ($msg,$ssl_err);
		while ($ssl_err = Net::SSLeay::ERR_get_error()) {
			$msg .= Net::SSLeay::ERR_error_string($ssl_err);
		}
		if($ssl_err) {
			close(S);
			return "Failed to connect to server via SSL: $! : $msg";
		}
	}

	&sw_log_debug("Cipher '" . Net::SSLeay::get_cipher($ssl) . "'\n");

	##set self before return
	$self->{_ssl} = $ssl;
	$self->{_sock} = *S{IO};
	$self->{_ctx} = $ctx;
	
	return undef;
}##/set_connection


## >> self
## => size
## < data
sub read_data
{
	my $self = shift;
	my %h = (@_);

	my $got = Net::SSLeay::ssl_read_until($self->{_ssl},undef,$h{size});
	
	return $got;
}##/read_data


## >> self
## =>>data
## < error
sub write_data
{
	my ($self,%h) = @_;
	my $data = $h{data} || sw_die("No data");

	Net::SSLeay::write ($self->{_ssl},$data) or return "SSL error: $!";
	my $err = Net::SSLeay::print_errs("SSL write");

	return $err;
}##/write_data


## >> self
sub close_connection
{
	my $self = shift;

	Net::SSLeay::free($self->{_ssl});           # Tear down connection
	close $self->{_sock};
}##/close_connection


1;
