package HSPC::Trace;

use strict;
use SWsoft::protect;	# Mark this module for encryption upon build


use HSPC::Logger;
use HSPC::MT::Core::Constants;

my $tid = 0;
my @trace_stack = ();

##--------------------------------------------------
## Usage:  HSPC::Trace->new("start order processing");
## Description: Constructor
## Parameters:
##		comment >>
## Return value:
## 		HSPC::Trace object
sub new {
	my $class = shift;
	my $comment = shift;

	my $self = {};
	bless($self,$class);

	my $trace_item = { comment => $comment };
	push @trace_stack, $trace_item;
	$self->{trace_item} = $trace_item;

	$tid++;
	$self->{tid} = $tid;

	my $prefix = $self->__prefix;
	my $msg = "$prefix+> $comment";

	local $Log::Log4perl::caller_depth += 1;
	HSPC::Logger->debug($msg);
	$self;
};

##--------------------------------------------------
## destructor
sub DESTROY {
	my $self = shift;

	my $prefix = $self->__prefix;
	my $trace_item = pop @trace_stack;
	my $comment = $trace_item->{comment};
	my $append_comment = $trace_item->{append_comment};

	my $msg = "$prefix-> $comment";
	$msg .= " => $append_comment" if ($append_comment);

	local $Log::Log4perl::caller_depth += 1;
	HSPC::Logger->debug($msg);
};

##--------------------------------------------------
## Usage:  $trace->add("found 3 objects");
## Description: add string to trace message.
## Parameters:
##		comment >>
## Return value:
##		undef
sub add {
	my $self = shift;
	my $comment = shift;

	$self->{trace_item}->{append_comment} .= " ".$comment;
	return undef;
};

sub addok {
	my $self = shift;
	return $self->add("Ok");
}

## this function should be used only in sw_die function to mark sw_die
## occurance
sub add_comment_to_latest_trace {
	my $class = shift;
	my $comment = shift;

	my $latest_trace_item = $trace_stack[-1];

	$latest_trace_item->{append_comment} .= " ".$comment
		if ($latest_trace_item);

	return undef;
}

##--------------------------------------------------
## prepare prefix fo log string
sub __prefix {
	my $self = shift;

	my $level = scalar( @trace_stack );
	$level = sprintf("%02d", $level);

	my $tid = sprintf("%05d", $self->{tid});

	my $prefix = "TRACE [$level]: ";
	$prefix .= ". " x ($level-1);
	return $prefix;
};

1;
__END__
