package HSPC::Util::DM;

use strict;
use Net::IDN::Encode qw(domain_to_ascii);
use Net::IDN::Nameprep qw(nameprep);
use Encode qw(encode _utf8_on _utf8_off);

use vars qw(@ISA @EXPORT @EXPORT_OK);

use Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(
d2ascii d2utf domain2utf prepare_idn_value norm_dname d2ascii_utf
);

sub d2ascii {
	my $value = shift;

	Encode::_utf8_on($value); ## Set utf8 flag
	my $res_value;
	eval {
		$res_value = domain_to_ascii($value);
	};

	my $result;
	if ($@) {
		$result = {
			error => 'invalid_domain',
		};
	} else {
		Encode::_utf8_off($res_value);
		$result = {
			value => $res_value
		};
	}

	return $result;
}

sub d2utf {
	my $value = shift;

	my $res_value;
	eval {
		$res_value = domain_to_unicode($value);
	};

	my $result;
	if ($@) {
		$result = {
			error => 'invalid_domain',
		}
	} else {
		$res_value = encode('utf8', $res_value);
		$result = {
			value => $res_value
		}
	}

	return $result;
}

## argument may be and ascii and utf encoded string
sub d2ascii_utf {
	my $value = shift;
	
	my $ascii = d2ascii($value);
	return $ascii if $ascii->{error};
	
	my $utf = d2utf($ascii->{value});
	return $utf if $utf->{error};
	
	return {
		utf_val => $utf->{value},
		ascii_val => $ascii->{value},
		error => undef,
	};
}


## Convert domain name (or it part) to utf if necessary
## > value
## < converted value
sub domain2utf {
	my $value = shift;

	my $res_value;
	eval {
		$res_value = domain_to_unicode($value);
	};

	if ($@) {
		$res_value = $value;
	} else {
		$res_value = encode('utf8', $res_value);
	}

	return $res_value;
}

## Convert all inplaced domain name parts to utf if necessary
## > value
## < converted value
sub prepare_idn_value {
	my $value = shift;

	my @places = $value =~ m#(xn--[\w|\-]+)#g;
	foreach my $ascii_part (@places) {
		my $new_value = domain2utf($ascii_part);
		$value =~ s#$ascii_part#$new_value#;
	}

	return $value;
}

## Domain name normalizing
## > value - some name in UTF8/ASCII format
## < normalized value
sub norm_dname {
	my $value = shift;

	eval {
		_utf8_on($value);
		$value = join '.', map {nameprep($_)} split(/\./, $value);
		$value = encode('utf8', $value);
	};

	if ($@) {
		sw_log_warn("Nameprep ERROR: $@");
		sw_die("Nameprep ERROR. is_fqdn must used before name normalizing.");
	}

	return $value;
}

1;
