package HSPC::Util::Translit;
use strict;
use SWsoft::protect;

use Exporter;
use vars qw(@ISA @EXPORT);
##
## Translit Rules ##############################################################
##
##
## Macroconstants for Translit Rules
##
use constant KOI8R_LV	=> "[\xc1\xc5\xa3\xc9\xcf\xd5\xd9\xdc\xc0\xd1]";
use constant KOI8R_UV	=> "[\xe1\xe5\xb3\xe9\xef\xf5\xf9\xfc\xe0\xf1]";
use constant KOI8R_S	=> "[\xdf\xd8]|[\xff\xf8]";
use constant KOI8R_V	=> &KOI8R_LV . "|" . &KOI8R_UV;
use constant KOI8R_NC	=> &KOI8R_V . "|" . &KOI8R_S;
##
##
## List of All Translit Rules
##
## Each rule is given by following form:
##     _Rule_Name_ => [_Old_Lexeme_,_New_Lexeme_]
## where
##     _Rule_Name_  - Name of tihs rule (example: 'koi8r_ascii7_l01_1')
##     _Old_Lexeme_ - RegExp for search in source line... WHAT replace?
##                    (example: "\xc1")
##     _New_Lexeme_ - String for replacement... replace by WHAT?
##                    (example: "a")
## 
## NOTE: Use alredy created rule - don't create new rule with other name!
##
use constant T_RULES => {
	'koi8r_ascii7_l01_1' => ["\xc1","a"],
	'koi8r_ascii7_l02_1' => ["\xc2","b"],
	'koi8r_ascii7_l03_1' => ["\xd7","v"],
	'koi8r_ascii7_l04_1' => ["\xc7","g"],
	'koi8r_ascii7_l05_1' => ["\xc4","d"],
	'koi8r_ascii7_l06_1' => ["\xc5","e"],
	'koi8r_ascii7_l07_1' => ["\xa3","e"],
	'koi8r_ascii7_l07_2' => ["^\xa3","yo"],
	'koi8r_ascii7_l07_3' => ["(" . &KOI8R_NC . "|yo|YO)\xa3","yo"],
	'koi8r_ascii7_l07_4' => ["\xa3","jo"],
	'koi8r_ascii7_l07_5' => ["\xa3","yo"],
	'koi8r_ascii7_l08_1' => ["\xd6","zh"],
	'koi8r_ascii7_l09_1' => ["\xda","z"],
	'koi8r_ascii7_l10_1' => ["\xc9","i"],
	'koi8r_ascii7_l11_1' => ["\xca","j"],
	'koi8r_ascii7_l11_2' => ["\xca","jj"],
	'koi8r_ascii7_l12_1' => ["\xcb","k"],
	'koi8r_ascii7_l13_1' => ["\xcc","l"],
	'koi8r_ascii7_l14_1' => ["\xcd","m"],
	'koi8r_ascii7_l15_1' => ["\xce","n"],
	'koi8r_ascii7_l16_1' => ["\xcf","o"],
	'koi8r_ascii7_l17_1' => ["\xd0","p"],
	'koi8r_ascii7_l18_1' => ["\xd2","r"],
	'koi8r_ascii7_l19_1' => ["\xd3","s"],
	'koi8r_ascii7_l20_1' => ["\xd4","t"],
	'koi8r_ascii7_l21_1' => ["\xd5","u"],
	'koi8r_ascii7_l22_1' => ["\xc6","f"],
	'koi8r_ascii7_l23_1' => ["\xc8","kh"],
	'koi8r_ascii7_l23_2' => ["\xc8","x"],
	'koi8r_ascii7_l24_1' => ["\xc3","ts"],
	'koi8r_ascii7_l24_2' => ["\xc3","c"],
	'koi8r_ascii7_l24_3' => ["\xc3","cz"],
	'koi8r_ascii7_l25_1' => ["\xde","ch"],
	'koi8r_ascii7_l26_1' => ["\xdb","sh"],
	'koi8r_ascii7_l27_1' => ["\xdd","shch"],
	'koi8r_ascii7_l27_2' => ["\xdd","shh"],
	'koi8r_ascii7_l28_1' => ["\xdf",""],
	'koi8r_ascii7_l28_2' => ["()\xdf(" . &KOI8R_V . "|yo|YO)","'"],
	'koi8r_ascii7_l28_3' => ["\xdf",'"'],
	'koi8r_ascii7_l28_4' => ["\xdf",'``'],
	'koi8r_ascii7_l29_1' => ["\xd9","y"],
	'koi8r_ascii7_l29_2' => ["\xd9","y'"],
	'koi8r_ascii7_l30_1' => ["\xd8",""],
	'koi8r_ascii7_l30_2' => ["()\xd8(" . &KOI8R_V . "|yo|YO)","'"],
	'koi8r_ascii7_l30_3' => ["\xd8","'"],
	'koi8r_ascii7_l30_4' => ["\xd8",'`'],
	'koi8r_ascii7_l31_1' => ["\xdc","eh"],
	'koi8r_ascii7_l31_2' => ["\xdc","e'"],
	'koi8r_ascii7_l32_1' => ["\xc0","yu"],
	'koi8r_ascii7_l32_2' => ["\xc0","ju"],
	'koi8r_ascii7_l33_1' => ["\xd1","ya"],
	'koi8r_ascii7_l33_2' => ["\xd1","ja"],

	'koi8r_ascii7_u01_1' => ["\xe1","A"],
	'koi8r_ascii7_u02_1' => ["\xe2","B"],
	'koi8r_ascii7_u03_1' => ["\xf7","V"],
	'koi8r_ascii7_u04_1' => ["\xe7","G"],
	'koi8r_ascii7_u05_1' => ["\xe4","D"],
	'koi8r_ascii7_u06_1' => ["\xe5","E"],
	'koi8r_ascii7_u07_1' => ["\xb3","E"],
	'koi8r_ascii7_u07_2' => ["^\xb3","YO"],
	'koi8r_ascii7_u07_3' => ["(" . &KOI8R_NC . "|yo|YO)\xb3","YO"],
	'koi8r_ascii7_u07_4' => ["\xb3","JO"],
	'koi8r_ascii7_u07_5' => ["\xb3","YO"],
	'koi8r_ascii7_u08_1' => ["\xf6","ZH"],
	'koi8r_ascii7_u09_1' => ["\xfa","Z"],
	'koi8r_ascii7_u10_1' => ["\xe9","I"],
	'koi8r_ascii7_u11_1' => ["\xea","J"],
	'koi8r_ascii7_u11_2' => ["\xea","JJ"],
	'koi8r_ascii7_u12_1' => ["\xeb","K"],
	'koi8r_ascii7_u13_1' => ["\xec","L"],
	'koi8r_ascii7_u14_1' => ["\xed","M"],
	'koi8r_ascii7_u15_1' => ["\xee","N"],
	'koi8r_ascii7_u16_1' => ["\xef","O"],
	'koi8r_ascii7_u17_1' => ["\xf0","P"],
	'koi8r_ascii7_u18_1' => ["\xf2","R"],
	'koi8r_ascii7_u19_1' => ["\xf3","S"],
	'koi8r_ascii7_u20_1' => ["\xf4","T"],
	'koi8r_ascii7_u21_1' => ["\xf5","U"],
	'koi8r_ascii7_u22_1' => ["\xe6","F"],
	'koi8r_ascii7_u23_1' => ["\xe8","KH"],
	'koi8r_ascii7_u23_2' => ["\xe8","X"],
	'koi8r_ascii7_u24_1' => ["\xe3","TS"],
	'koi8r_ascii7_u24_2' => ["\xe3","C"],
	'koi8r_ascii7_u24_3' => ["\xe3","CZ"],
	'koi8r_ascii7_u25_1' => ["\xfe","CH"],
	'koi8r_ascii7_u26_1' => ["\xfb","SH"],
	'koi8r_ascii7_u27_1' => ["\xfd","SHCH"],
	'koi8r_ascii7_u27_2' => ["\xfd","SHH"],
	'koi8r_ascii7_u28_1' => ["\xff",""],
	'koi8r_ascii7_u28_2' => ["()\xff(" . &KOI8R_V . "|yo|YO)","'"],
	'koi8r_ascii7_u28_3' => ["\xff",'"'],
	'koi8r_ascii7_u28_4' => ["\xff",'``'],
	'koi8r_ascii7_u29_1' => ["\xf9","Y"],
	'koi8r_ascii7_u29_2' => ["\xf9","Y'"],
	'koi8r_ascii7_u30_1' => ["\xf8",""],
	'koi8r_ascii7_u30_2' => ["()\xf8(" . &KOI8R_V . "|yo|YO)","'"],
	'koi8r_ascii7_u30_3' => ["\xf8","'"],
	'koi8r_ascii7_u30_4' => ["\xf8",'`'],
	'koi8r_ascii7_u31_1' => ["\xfc","EH"],
	'koi8r_ascii7_u31_2' => ["\xfc","E'"],
	'koi8r_ascii7_u32_1' => ["\xe0","YU"],
	'koi8r_ascii7_u32_2' => ["\xe0","JU"],
	'koi8r_ascii7_u33_1' => ["\xf1","YA"],
	'koi8r_ascii7_u33_2' => ["\xf1","JA"],

	'spain_ascii7_s01_1' => ["\x23","\x20"],
	'spain_ascii7_s02_1' => ["\x2c","\x20"],
	'spain_ascii7_s03_1' => ["\x2d","\x20"],
	'spain_ascii7_s04_1' => ["\x2f","\x20"],
	'spain_ascii7_s05_1' => ["\x3b","\x20"],
	'spain_ascii7_s06_1' => ["\x5c\x5c","\x20"],
	'spain_ascii7_s07_1' => ["\x5f","\x20"],
	'spain_ascii7_s08_1' => ["\xaa","a"],
	'spain_ascii7_s09_1' => ["\xb7","\."],
	'spain_ascii7_s10_1' => ["\xba","o"],
	'spain_ascii7_s11_1' => ["\xc0","A"],
	'spain_ascii7_s12_1' => ["\xc1","A"],
	'spain_ascii7_s13_1' => ["\xc4","A"],
	'spain_ascii7_s14_1' => ["\xc8","E"],
	'spain_ascii7_s15_1' => ["\xc9","E"],
	'spain_ascii7_s16_1' => ["\xcb","E"],
	'spain_ascii7_s17_1' => ["\xcc","I"],
	'spain_ascii7_s18_1' => ["\xcd","I"],
	'spain_ascii7_s19_1' => ["\xcf","I"],
	'spain_ascii7_s20_1' => ["\xd1","N"],
	'spain_ascii7_s22_1' => ["\xd2","O"],
	'spain_ascii7_s23_1' => ["\xd3","O"],
	'spain_ascii7_s24_1' => ["\xd6","O"],
	'spain_ascii7_s25_1' => ["\xd9","U"],
	'spain_ascii7_s26_1' => ["\xda","U"],
	'spain_ascii7_s27_1' => ["\xdc","U"],
	'spain_ascii7_s28_1' => ["\xe0","a"],
	'spain_ascii7_s29_1' => ["\xe1","a"],
	'spain_ascii7_s30_1' => ["\xe4","a"],
	'spain_ascii7_s31_1' => ["\xe8","e"],
	'spain_ascii7_s32_1' => ["\xe9","e"],
	'spain_ascii7_s33_1' => ["\xeb","e"],
	'spain_ascii7_s34_1' => ["\xec","i"],
	'spain_ascii7_s35_1' => ["\xed","i"],
	'spain_ascii7_s36_1' => ["\xef","i"],
	'spain_ascii7_s37_1' => ["\xf1","n"],
	'spain_ascii7_s38_1' => ["\xf2","o"],
	'spain_ascii7_s39_1' => ["\xf3","o"],
	'spain_ascii7_s40_1' => ["\xf6","o"],
	'spain_ascii7_s41_1' => ["\xf9","u"],
	'spain_ascii7_s42_1' => ["\xfa","u"],
	'spain_ascii7_s43_1' => ["\xfc","u"],

	'german_ascii7_s01_1' => ["\xc4","Ae"],
	'german_ascii7_s02_1' => ["\xd6","Oe"],
	'german_ascii7_s03_1' => ["\xdc","Ue"],
	'german_ascii7_s04_1' => ["\xdf","ss"],
	'german_ascii7_s05_1' => ["\xe4","ae"],
	'german_ascii7_s06_1' => ["\xf6","oe"],
	'german_ascii7_s07_1' => ["\xfc","ue"],

	'nonascii_ascii7_s01_1' => ["[\x7f-\xff]",""],
};
##
## Translit Tables #############################################################
##
##
## Translit Table is list of ordered Translit Rule names.
##
##
## Translit Table "GOST_16876_71"
## 
## This table includes rules for transliteration from Cyrillic (KOI8R)
## to Latin (ASCII7) in accordance with GOST 16876-71.
##
use constant TT_GOST_16876_71 => [
	'koi8r_ascii7_l01_1','koi8r_ascii7_l02_1','koi8r_ascii7_l03_1',
	'koi8r_ascii7_l04_1','koi8r_ascii7_l05_1','koi8r_ascii7_l06_1',
	'koi8r_ascii7_l07_4','koi8r_ascii7_l08_1','koi8r_ascii7_l09_1',
	'koi8r_ascii7_l10_1','koi8r_ascii7_l11_2','koi8r_ascii7_l12_1',
	'koi8r_ascii7_l13_1','koi8r_ascii7_l14_1','koi8r_ascii7_l15_1',
	'koi8r_ascii7_l16_1','koi8r_ascii7_l17_1','koi8r_ascii7_l18_1',
	'koi8r_ascii7_l19_1','koi8r_ascii7_l20_1','koi8r_ascii7_l21_1',
	'koi8r_ascii7_l22_1','koi8r_ascii7_l23_1','koi8r_ascii7_l24_2',
	'koi8r_ascii7_l25_1','koi8r_ascii7_l26_1','koi8r_ascii7_l27_2',
	'koi8r_ascii7_l28_3','koi8r_ascii7_l29_1','koi8r_ascii7_l30_3',
	'koi8r_ascii7_l31_1','koi8r_ascii7_l32_2','koi8r_ascii7_l33_2',

	'koi8r_ascii7_u01_1','koi8r_ascii7_u02_1','koi8r_ascii7_u03_1',
	'koi8r_ascii7_u04_1','koi8r_ascii7_u05_1','koi8r_ascii7_u06_1',
	'koi8r_ascii7_u07_4','koi8r_ascii7_u08_1','koi8r_ascii7_u09_1',
	'koi8r_ascii7_u10_1','koi8r_ascii7_u11_2','koi8r_ascii7_u12_1',
	'koi8r_ascii7_u13_1','koi8r_ascii7_u14_1','koi8r_ascii7_u15_1',
	'koi8r_ascii7_u16_1','koi8r_ascii7_u17_1','koi8r_ascii7_u18_1',
	'koi8r_ascii7_u19_1','koi8r_ascii7_u20_1','koi8r_ascii7_u21_1',
	'koi8r_ascii7_u22_1','koi8r_ascii7_u23_1','koi8r_ascii7_u24_2',
	'koi8r_ascii7_u25_1','koi8r_ascii7_u26_1','koi8r_ascii7_u27_2',
	'koi8r_ascii7_u28_3','koi8r_ascii7_u29_1','koi8r_ascii7_u30_3',
	'koi8r_ascii7_u31_1','koi8r_ascii7_u32_2','koi8r_ascii7_u33_2'
];
##
##
## Translit Table "GOST_7_79_2000"
## 
## This table includes rules for transliteration from Cyrillic (KOI8R)
## to Latin (ASCII7) in accordance with GOST 7.79-2000.
##
use constant TT_GOST_7_79_2000 => [
	'koi8r_ascii7_l01_1','koi8r_ascii7_l02_1','koi8r_ascii7_l03_1',
	'koi8r_ascii7_l04_1','koi8r_ascii7_l05_1','koi8r_ascii7_l06_1',
	'koi8r_ascii7_l07_5','koi8r_ascii7_l08_1','koi8r_ascii7_l09_1',
	'koi8r_ascii7_l10_1','koi8r_ascii7_l11_1','koi8r_ascii7_l12_1',
	'koi8r_ascii7_l13_1','koi8r_ascii7_l14_1','koi8r_ascii7_l15_1',
	'koi8r_ascii7_l16_1','koi8r_ascii7_l17_1','koi8r_ascii7_l18_1',
	'koi8r_ascii7_l19_1','koi8r_ascii7_l20_1','koi8r_ascii7_l21_1',
	'koi8r_ascii7_l22_1','koi8r_ascii7_l23_2','koi8r_ascii7_l24_3',
	'koi8r_ascii7_l25_1','koi8r_ascii7_l26_1','koi8r_ascii7_l27_2',
	'koi8r_ascii7_l28_4','koi8r_ascii7_l29_2','koi8r_ascii7_l30_4',
	'koi8r_ascii7_l31_2','koi8r_ascii7_l32_1','koi8r_ascii7_l33_1',

	'koi8r_ascii7_u01_1','koi8r_ascii7_u02_1','koi8r_ascii7_u03_1',
	'koi8r_ascii7_u04_1','koi8r_ascii7_u05_1','koi8r_ascii7_u06_1',
	'koi8r_ascii7_u07_5','koi8r_ascii7_u08_1','koi8r_ascii7_u09_1',
	'koi8r_ascii7_u10_1','koi8r_ascii7_u11_1','koi8r_ascii7_u12_1',
	'koi8r_ascii7_u13_1','koi8r_ascii7_u14_1','koi8r_ascii7_u15_1',
	'koi8r_ascii7_u16_1','koi8r_ascii7_u17_1','koi8r_ascii7_u18_1',
	'koi8r_ascii7_u19_1','koi8r_ascii7_u20_1','koi8r_ascii7_u21_1',
	'koi8r_ascii7_u22_1','koi8r_ascii7_u23_2','koi8r_ascii7_u24_3',
	'koi8r_ascii7_u25_1','koi8r_ascii7_u26_1','koi8r_ascii7_u27_2',
	'koi8r_ascii7_u28_4','koi8r_ascii7_u29_2','koi8r_ascii7_u30_4',
	'koi8r_ascii7_u31_2','koi8r_ascii7_u32_1','koi8r_ascii7_u33_1'
];
##
##
## Translit Table "ISO_R9_1968"
## 
## This table includes rules for transliteration from Cyrillic (KOI8R)
## to Latin (ASCII7) in accordance with ISO-R9-1968.
##
use constant TT_ISO_R9_1968 => [
	'koi8r_ascii7_l01_1','koi8r_ascii7_l02_1','koi8r_ascii7_l03_1',
	'koi8r_ascii7_l04_1','koi8r_ascii7_l05_1','koi8r_ascii7_l06_1',
	'koi8r_ascii7_l07_1','koi8r_ascii7_l08_1','koi8r_ascii7_l09_1',
	'koi8r_ascii7_l10_1','koi8r_ascii7_l11_1','koi8r_ascii7_l12_1',
	'koi8r_ascii7_l13_1','koi8r_ascii7_l14_1','koi8r_ascii7_l15_1',
	'koi8r_ascii7_l16_1','koi8r_ascii7_l17_1','koi8r_ascii7_l18_1',
	'koi8r_ascii7_l19_1','koi8r_ascii7_l20_1','koi8r_ascii7_l21_1',
	'koi8r_ascii7_l22_1','koi8r_ascii7_l23_1','koi8r_ascii7_l24_1',
	'koi8r_ascii7_l25_1','koi8r_ascii7_l26_1','koi8r_ascii7_l27_1',
	'koi8r_ascii7_l28_3','koi8r_ascii7_l29_1','koi8r_ascii7_l30_3',
	'koi8r_ascii7_l31_1','koi8r_ascii7_l32_1','koi8r_ascii7_l33_1',

	'koi8r_ascii7_u01_1','koi8r_ascii7_u02_1','koi8r_ascii7_u03_1',
	'koi8r_ascii7_u04_1','koi8r_ascii7_u05_1','koi8r_ascii7_u06_1',
	'koi8r_ascii7_u07_1','koi8r_ascii7_u08_1','koi8r_ascii7_u09_1',
	'koi8r_ascii7_u10_1','koi8r_ascii7_u11_1','koi8r_ascii7_u12_1',
	'koi8r_ascii7_u13_1','koi8r_ascii7_u14_1','koi8r_ascii7_u15_1',
	'koi8r_ascii7_u16_1','koi8r_ascii7_u17_1','koi8r_ascii7_u18_1',
	'koi8r_ascii7_u19_1','koi8r_ascii7_u20_1','koi8r_ascii7_u21_1',
	'koi8r_ascii7_u22_1','koi8r_ascii7_u23_1','koi8r_ascii7_u24_1',
	'koi8r_ascii7_u25_1','koi8r_ascii7_u26_1','koi8r_ascii7_u27_1',
	'koi8r_ascii7_u28_3','koi8r_ascii7_u29_1','koi8r_ascii7_u30_3',
	'koi8r_ascii7_u31_1','koi8r_ascii7_u32_1','koi8r_ascii7_u33_1'
];
##
##
## Translit Table "ISO_R9_1968_WOA"
## 
## This table includes rules for transliteration from Cyrillic (KOI8R)
## to Latin (ASCII7) in accordance with ISO-R9-1968,
## but without apostrophe character.
##
use constant TT_ISO_R9_1968_WOA => [
	'koi8r_ascii7_l01_1','koi8r_ascii7_l02_1','koi8r_ascii7_l03_1',
	'koi8r_ascii7_l04_1','koi8r_ascii7_l05_1','koi8r_ascii7_l06_1',
	'koi8r_ascii7_l07_1','koi8r_ascii7_l08_1','koi8r_ascii7_l09_1',
	'koi8r_ascii7_l10_1','koi8r_ascii7_l11_1','koi8r_ascii7_l12_1',
	'koi8r_ascii7_l13_1','koi8r_ascii7_l14_1','koi8r_ascii7_l15_1',
	'koi8r_ascii7_l16_1','koi8r_ascii7_l17_1','koi8r_ascii7_l18_1',
	'koi8r_ascii7_l19_1','koi8r_ascii7_l20_1','koi8r_ascii7_l21_1',
	'koi8r_ascii7_l22_1','koi8r_ascii7_l23_1','koi8r_ascii7_l24_1',
	'koi8r_ascii7_l25_1','koi8r_ascii7_l26_1','koi8r_ascii7_l27_1',
	'koi8r_ascii7_l28_1','koi8r_ascii7_l29_1','koi8r_ascii7_l30_1',
	'koi8r_ascii7_l31_1','koi8r_ascii7_l32_1','koi8r_ascii7_l33_1',

	'koi8r_ascii7_u01_1','koi8r_ascii7_u02_1','koi8r_ascii7_u03_1',
	'koi8r_ascii7_u04_1','koi8r_ascii7_u05_1','koi8r_ascii7_u06_1',
	'koi8r_ascii7_u07_1','koi8r_ascii7_u08_1','koi8r_ascii7_u09_1',
	'koi8r_ascii7_u10_1','koi8r_ascii7_u11_1','koi8r_ascii7_u12_1',
	'koi8r_ascii7_u13_1','koi8r_ascii7_u14_1','koi8r_ascii7_u15_1',
	'koi8r_ascii7_u16_1','koi8r_ascii7_u17_1','koi8r_ascii7_u18_1',
	'koi8r_ascii7_u19_1','koi8r_ascii7_u20_1','koi8r_ascii7_u21_1',
	'koi8r_ascii7_u22_1','koi8r_ascii7_u23_1','koi8r_ascii7_u24_1',
	'koi8r_ascii7_u25_1','koi8r_ascii7_u26_1','koi8r_ascii7_u27_1',
	'koi8r_ascii7_u28_1','koi8r_ascii7_u29_1','koi8r_ascii7_u30_1',
	'koi8r_ascii7_u31_1','koi8r_ascii7_u32_1','koi8r_ascii7_u33_1'
];
##
##
## Translit Table "CUSTOM_CYRLAT"
## 
## This table includes rules for transliteration from Cyrillic (KOI8R)
## to Latin (ASCII7) in accordance with customized ISO-R9-1968 and customized
## "AS USSR Institute of Linguistics. The Rules of international
## transliteration of Russian personal name to Latin characters. 1951-1956"
##
use constant TT_CUSTOM_CYRLAT => [
	'koi8r_ascii7_l07_3',
	'koi8r_ascii7_u07_3',
	'koi8r_ascii7_l07_2',
	'koi8r_ascii7_u07_2',
	'koi8r_ascii7_l28_2',
	'koi8r_ascii7_l30_2',
	'koi8r_ascii7_u28_2',
	'koi8r_ascii7_u30_2',
	'koi8r_ascii7_l01_1','koi8r_ascii7_l02_1','koi8r_ascii7_l03_1',
	'koi8r_ascii7_l04_1','koi8r_ascii7_l05_1','koi8r_ascii7_l06_1',
	'koi8r_ascii7_l07_1','koi8r_ascii7_l08_1','koi8r_ascii7_l09_1',
	'koi8r_ascii7_l10_1','koi8r_ascii7_l11_1','koi8r_ascii7_l12_1',
	'koi8r_ascii7_l13_1','koi8r_ascii7_l14_1','koi8r_ascii7_l15_1',
	'koi8r_ascii7_l16_1','koi8r_ascii7_l17_1','koi8r_ascii7_l18_1',
	'koi8r_ascii7_l19_1','koi8r_ascii7_l20_1','koi8r_ascii7_l21_1',
	'koi8r_ascii7_l22_1','koi8r_ascii7_l23_1','koi8r_ascii7_l24_1',
	'koi8r_ascii7_l25_1','koi8r_ascii7_l26_1','koi8r_ascii7_l27_1',
	'koi8r_ascii7_l28_1','koi8r_ascii7_l29_1','koi8r_ascii7_l30_1',
	'koi8r_ascii7_l31_1','koi8r_ascii7_l32_1','koi8r_ascii7_l33_1',
	'koi8r_ascii7_u01_1','koi8r_ascii7_u02_1','koi8r_ascii7_u03_1',
	'koi8r_ascii7_u04_1','koi8r_ascii7_u05_1','koi8r_ascii7_u06_1',
	'koi8r_ascii7_u07_1','koi8r_ascii7_u08_1','koi8r_ascii7_u09_1',
	'koi8r_ascii7_u10_1','koi8r_ascii7_u11_1','koi8r_ascii7_u12_1',
	'koi8r_ascii7_u13_1','koi8r_ascii7_u14_1','koi8r_ascii7_u15_1',
	'koi8r_ascii7_u16_1','koi8r_ascii7_u17_1','koi8r_ascii7_u18_1',
	'koi8r_ascii7_u19_1','koi8r_ascii7_u20_1','koi8r_ascii7_u21_1',
	'koi8r_ascii7_u22_1','koi8r_ascii7_u23_1','koi8r_ascii7_u24_1',
	'koi8r_ascii7_u25_1','koi8r_ascii7_u26_1','koi8r_ascii7_u27_1',
	'koi8r_ascii7_u28_1','koi8r_ascii7_u29_1','koi8r_ascii7_u30_1',
	'koi8r_ascii7_u31_1','koi8r_ascii7_u32_1','koi8r_ascii7_u33_1'
];
##
##
## Translit Table "SPAIN_BASE"
##
## This table includes rules for transliteration from Spain
## to Latin (ASCII7) that was received from Marc Bote Serrate.
##
use constant TT_SPAIN_BASE => [
	'spain_ascii7_s01_1','spain_ascii7_s02_1','spain_ascii7_s03_1',
	'spain_ascii7_s04_1','spain_ascii7_s05_1','spain_ascii7_s06_1',
	'spain_ascii7_s07_1','spain_ascii7_s08_1','spain_ascii7_s09_1',
	'spain_ascii7_s10_1',
	'spain_ascii7_s11_1','spain_ascii7_s12_1','spain_ascii7_s13_1',
	'spain_ascii7_s14_1','spain_ascii7_s15_1','spain_ascii7_s16_1',
	'spain_ascii7_s17_1','spain_ascii7_s18_1','spain_ascii7_s19_1',
	'spain_ascii7_s20_1',
	'spain_ascii7_s21_1','spain_ascii7_s22_1','spain_ascii7_s23_1',
	'spain_ascii7_s24_1','spain_ascii7_s25_1','spain_ascii7_s26_1',
	'spain_ascii7_s27_1','spain_ascii7_s28_1','spain_ascii7_s29_1',
	'spain_ascii7_s30_1',
	'spain_ascii7_s31_1','spain_ascii7_s32_1','spain_ascii7_s33_1',
	'spain_ascii7_s34_1','spain_ascii7_s35_1','spain_ascii7_s36_1',
	'spain_ascii7_s37_1','spain_ascii7_s38_1','spain_ascii7_s39_1',
	'spain_ascii7_s40_1',
	'spain_ascii7_s41_1','spain_ascii7_s42_1','spain_ascii7_s43_1'
];
##
##
## Translit Table "GERMAN_BASE"
##
## This table includes rules for transliteration from German
## to Latin (ASCII7) that was received from old Denic DM Plugin.
##
use constant TT_GERMAN_BASE => [
	'german_ascii7_s01_1','german_ascii7_s02_1','german_ascii7_s03_1',
	'german_ascii7_s04_1','german_ascii7_s05_1','german_ascii7_s06_1',
	'german_ascii7_s07_1'
];
##
##
## Translit Table "ISO_8859_1"
##
## This table includes rules for transliteration from ISO_8859_1
## to Latin (ASCII7).
##
use constant TT_ISO_8859_1 => [
	'nonascii_ascii7_s01_1'
];

##
##
## Translit Table Names
## 
use constant TTN_GOST_16876_71		=> 'GOST_16876_71';
use constant TTN_GOST_7_79_2000		=> 'GOST_7_79_2000';
use constant TTN_ISO_R9_1968		=> 'ISO_R9_1968';
use constant TTN_ISO_R9_1968_WOA	=> 'ISO_R9_1968_WOA';
use constant TTN_CUSTOM_CYRLAT		=> 'CUSTOM_CYRLAT';
use constant TTN_SPAIN_BASE			=> 'TT_SPAIN_BASE';
use constant TTN_GERMAN_BASE		=> 'TT_GERMAN_BASE';
use constant TTN_ISO_8859_1			=> 'TT_ISO_8859_1';

##
##
## Translit Table Names list for export
## 
use constant TTN_LIST => [
	&TTN_GOST_16876_71,
	&TTN_GOST_7_79_2000,
	&TTN_ISO_R9_1968,
	&TTN_ISO_R9_1968_WOA,
	&TTN_CUSTOM_CYRLAT,
	&TTN_SPAIN_BASE,
	&TTN_GERMAN_BASE,
	&TTN_ISO_8859_1
];
##
##
## "Translit Table Name <-> Translit Table Reference" Table
## 
use constant T_TABLES => {
	&TTN_GOST_16876_71		=> &TT_GOST_16876_71,
	&TTN_GOST_7_79_2000		=> &TT_GOST_7_79_2000,
	&TTN_ISO_R9_1968		=> &TT_ISO_R9_1968,
	&TTN_ISO_R9_1968_WOA	=> &TT_ISO_R9_1968_WOA,
	&TTN_CUSTOM_CYRLAT		=> &TT_CUSTOM_CYRLAT,
	&TTN_SPAIN_BASE			=> &TT_SPAIN_BASE,
	&TTN_GERMAN_BASE		=> &TT_GERMAN_BASE,
	&TTN_ISO_8859_1			=> &TT_ISO_8859_1
};
##
## Translit Engine #############################################################
##
## 
## translit - Transliteration string in first parameter using table with name
##            in second parameter.
## Input:
##     Source - Source string for transliteration (default Blank).
##     TTableName - Name of Translit Table (default TTN_ISO_R9_1968).
##
## Output:
##     Destination - Transliterated source string.
##
sub translit {
	my $source = shift;
	my $ttable_name = shift;
	$ttable_name = &TTN_ISO_R9_1968 if !defined $ttable_name;
	my $ttable = &T_TABLES->{$ttable_name};
	return $source if !$source || !$ttable;
	foreach my $trule_name (@$ttable){
		my $trule = &T_RULES->{$trule_name};
		my $old = $trule->[0];
		my $new = $trule->[1];
		$source =~ s#$old#$1$new$2#g;
	}
	return $source;
}
##
## Export ######################################################################
##
@ISA    = qw(Exporter);
@EXPORT = qw(
	TTN_LIST

	TTN_GOST_16876_71
	TTN_GOST_7_79_2000
	TTN_ISO_R9_1968
	TTN_ISO_R9_1968_WOA
	TTN_CUSTOM_CYRLAT
	TTN_SPAIN_BASE
	TTN_GERMAN_BASE
	TTN_ISO_8859_1

	translit
);

1;
