package HSPC::WebUpload;
use strict;
use HSPC::Config;

use HSPC::Localization;
use File::Basename; 
#use Image::Info qw(image_info dim);
use SWsoft::protect;	# Mark this module for encryption upon build

use HSPC::Logger qw(sw_log_warn sw_die sw_log_debug sw_log_error);


# =>>  view_name
# =>>  base_path : with last /
# =>   new_rel_path : with last /
# =>   old_rel_path : full path to file, uses with overwrite=>1
# =>   old_file_name : name uses for overwrite=>1
# =>   only_images
# =>   debug
##=> file_name - to force filename
# <  ref to hash
#		view_name=> 
#   	base_path => base_path
#   	rel_path => relative from base_path=>
#		file_name=>
#		size=>
#		info=>
#		mime_type=>
#		width=>
#		height=>
#		is_image>
#		overwrited=> file was overwrited
#   or 
# < error_string
#   or   
# < undef : file was not selected in upload_control
###
sub upload_file{
	my %h=(@_);
	sw_die('view_name=> undefined') unless defined $h{view_name};
	sw_die('base_path=> undefined') unless defined $h{base_path};
	
	#
	##arn 'upload_file : loc_path='.$h{local_path};
	##arn 'upload_file : overwrite='.$h{overwrite};	
	
	# get data #
	my $q = $ENV{q};
	my %res;
		
	$res{file} = $q->param($h{view_name});	
	return undef unless $res{file};
			
	$res{mime_type} = $q->uploadInfo($res{file})->{'Content-Type'};
	$res{is_image} =  $res{mime_type}=~m|^image/.*|;
	return \%res if $h{only_images} && not $res{is_image};
	
	{# remove path from file_name #
		$res{file_name} =  $h{file_name} || basename $res{file};		
		my $file_name = $res{file_name};
		$file_name=~s|^.*\\||;
		##remove bad symbols
		$file_name =~ s![^A-Za-z0-9\.\-\_ ]!_!g;
		$res{file_name} = $file_name;
	}
	
	## Check file name
	return string('upload_file_bad_name')
		if($res{file_name} =~ /[^A-Za-z0-9\.\-\_ ]/);
	
	# open file #
	my $file_path;		
	
	if( 
		(length($h{old_file_name})>0 && ($h{old_file_name} eq $res{file_name})) ||
		$h{file_name}
	){	
		return string('cant_make_dir') if (
			system("mkdir -p $h{base_path}$h{old_rel_path}")
		);
		if ( $h{file_name} ) {
			$file_path = $h{base_path}.$res{file_name};
		} else {
			$file_path = $h{base_path}.$h{old_rel_path}.$h{old_file_name};
		}
		$res{base_path} = $h{base_path};
		$res{rel_path} = $h{old_rel_path};
		$res{overwrited} = 1;
	}else{		
		### create new file
		$res{overwrited} = 0;
		$res{base_path} = $h{base_path};
		$res{rel_path} = $h{new_rel_path};
	
		$file_path = $h{base_path}.$h{new_rel_path};
		my $folder;
				
		while (open DEST, "< $file_path".($folder?"$folder/":'').$res{file_name}){
			$folder++;
			close DEST;
		}
		
		# create new folder #
		if($folder){
			unless( -d $file_path.$folder){
				unless( mkdir($file_path.$folder, 0777) ){
					sw_log_error(
						'perlib:WebUpload : can`t make directory '.$file_path.$folder
					);
					return string('cant_make_dir');
				};
			}
			$file_path.=$folder.'/';
			$res{rel_path} .= $folder.'/';
		}
		#####			
		return string('cant_make_dir') if ( system("mkdir -p $file_path") );
		$file_path.=$res{file_name};	
	}

				
	# upload file #	
	if( not open(OUT,">$file_path")){
		 ##arn "Can't open outfile for writing: $file_path!";
		 ##arn 'upload_file: upload error';
		 return string('cant_write_into') . ": $file_path!";  # TODO: return erorr_status
	}
	
    my $bytesread = 0;
    my $buffer;    
    
    $res{size} = 0;
	$res{file} = $q->upload($h{view_name})->fh
		unless $q->isa('CGI');
	while ($bytesread = read($res{file}, $buffer, 1024)) {
		$res{size}+=$bytesread;
		print OUT $buffer;
	}
	
    close(OUT);
    
    
    ## get image_info 
	if ($res{is_image}) {
		my $info = image_info($file_path);
		$res{is_image} =  ($info->{file_media_type}=~m|^image/.*| && $info->{error} eq '')?1:0;
		if( $info->{error} ){
			sw_log_warn('[WEBUPLOAD] '.$info->{error});
		}
		if($res{is_image}){
			($res{width},$res{height}) = dim($info);
		}
	}
	
	map{sw_log_debug("image_upload, $_=".$res{$_})} keys %res if $h{debug};
		
	##arn 'upload_file : path='.$res{path};
	return \%res;
}

1;                                          
